@@ -465,9 +465,23 @@ class Config():
                 self.file_path = "__inline__"
 
     def fetch(self, profile:str|None=None):
+        from relationalai.environments import runtime_env, TerminalEnvironment
         profile = profile or os.environ.get("RAI_PROFILE", None)
         cfg, path = _get_full_config(profile if profile != TOP_LEVEL_PROFILE_NAME else None)
-        self.profile = profile or cfg.get("active_profile", TOP_LEVEL_PROFILE_NAME)
+        cfg_profile = cfg.get("active_profile", TOP_LEVEL_PROFILE_NAME)
+
+        self.profile = profile or cfg_profile
+
+        # Handle the case where the config does not exist and we are in a terminal environment
+        if profile is None and isinstance(runtime_env, TerminalEnvironment):
+            from relationalai.errors import NonExistentConfigFileError, InvalidActiveProfileError
+            if cfg == {}:
+                raise NonExistentConfigFileError(DEFAULT_CONFIG_FILE)
+
+            # Handle the scenario where the active profile is set to invalid profile name
+            if set(cfg.keys()) == {"active_profile"}:
+                raise InvalidActiveProfileError(cfg.get("active_profile", "unknown"))
+
         self.file_path = path
         self.props = cfg
 
