@@ -3,14 +3,15 @@ from __future__ import annotations
 from collections import defaultdict
 from functools import reduce
 import re
-from typing import Any, Dict, List, Tuple, cast
+from typing import Any, Dict, List, Tuple, cast, Optional
 import base64
 
 import numpy as np
 from pandas import DataFrame
+from pandas.api.extensions import ExtensionDtype, ExtensionArray, register_extension_dtype
 import pandas as pd
 import decimal
-
+from relationalai.tools.constants import Generation
 from relationalai import debugging
 from .. import metamodel as m
 
@@ -21,6 +22,89 @@ from .. import metamodel as m
 UNIXEPOCH = 62135683200000
 MILLISECONDS_PER_DAY = 24 * 60 * 60 * 1000
 
+#--------------------------------------------------
+# Custom Int128 dtype
+#--------------------------------------------------
+
+@register_extension_dtype
+class Int128Dtype(ExtensionDtype):
+    name = "Int128"
+    type = int
+
+    @ExtensionDtype.kind.getter
+    def kind(self):
+        return "i"
+
+    @ExtensionDtype.na_value.getter
+    def na_value(self):
+        return pd.NA
+
+    @classmethod
+    def construct_array_type(cls):
+        return Int128Array
+
+
+class Int128Array(ExtensionArray):
+    def __init__(self, values):
+        # Convert None/pd.NA/np.nan to pd.NA, keep ints as Python int
+        self._data = np.array(
+            [pd.NA if v is None or v is pd.NA or (isinstance(v, float) and np.isnan(v)) else int(v)
+             for v in values],
+            dtype=Int128Dtype
+        )
+
+    @classmethod
+    def _from_sequence(cls, scalars, dtype=None, copy=False):
+        return cls(scalars)
+
+    @classmethod
+    def _from_factorized(cls, values, original):
+        return Int128Array(values)
+
+    def __getitem__(self, item):
+        if isinstance(item, int):
+            return self._data[item]
+        return type(self)(self._data[item])
+
+    def __len__(self):
+        return len(self._data)
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, Int128Array):
+            return False
+        return self._data == other._data
+
+    @property
+    def dtype(self):
+        return Int128Dtype()
+
+    @property
+    def nbytes(self):
+        return self._data.nbytes
+
+    def isna(self):
+        return np.array([x is pd.NA for x in self._data])
+
+    def take(self, indexer, *, allow_fill=False, fill_value=None):
+        from pandas.api.extensions import take
+        if allow_fill and fill_value is None:
+            fill_value = self.dtype.na_value
+        return type(self)(take(self._data, indexer, fill_value=fill_value, allow_fill=allow_fill))
+
+    def copy(self):
+        return type(self)(self._data.copy())
+
+    def __repr__(self):
+        return f"Int128Array({list(self._data)})"
+
+    def __add__(self, other):
+        if isinstance(other, Int128Array):
+            other = other._data
+        return Int128Array(
+            [pd.NA if a is pd.NA or b is pd.NA else a + b for a, b in zip(self._data, other)]
+        )
+
+
 #--------------------------------------------------
 # Result formatting
 #--------------------------------------------------
@@ -43,7 +127,7 @@ def format_value(value, type):
         return None
     return value
 
-def format_columns(result_frame:DataFrame, types:List[str]):
+def format_columns(result_frame:DataFrame, types:List[str], generation:Optional[Generation]=Generation.V0):
     for i, col in enumerate(result_frame.columns):
         if "UInt128" in types[i]:
             result_frame[col] = result_frame[col].apply(decode_uint128)
@@ -57,7 +141,10 @@ def format_columns(result_frame:DataFrame, types:List[str]):
                 else:
                     result_frame[col] = result_frame[col].apply(lambda x: decode_fixed_decimal(x, scale))
         elif "Int128" in types[i]:
-            result_frame[col] = result_frame[col].apply(decode_int128)
+            if generation == Generation.V0:
+                result_frame[col] = result_frame[col].apply(decode_int128)
+            else:
+                result_frame[col] = result_frame[col].apply(decode_int128).astype(Int128Dtype())
         elif "Missing" in types[i]:
             result_frame[col] = result_frame[col].apply(lambda _: None)
         elif types[i] == "Dates.DateTime":
@@ -142,7 +229,7 @@ def merge_columns(left:DataFrame, right:DataFrame, on:List[str]):
         return pd.merge(left, right, how='cross')
     return pd.merge(left, right, on=on, how='outer')
 
-def format_results(results, task:m.Task|None, result_cols:List[str]|None = None) -> Tuple[DataFrame, List[Any]]:
+def format_results(results, task:m.Task|None, result_cols:List[str]|None = None, generation:Optional[Generation]=Generation.V0) -> Tuple[DataFrame, List[Any]]:
     with debugging.span("format_results"):
         data_frame = DataFrame()
         problems = defaultdict(
@@ -286,13 +373,13 @@ def format_results(results, task:m.Task|None, result_cols:List[str]|None = None)
                             props[row.iloc[1]] = format_value(row.iloc[2], relation_id.split("/")[-1])
 
                 elif "/:__pyrel_debug_watch" in relation_id:
-                    result_frame = format_columns(result_frame, types)
+                    result_frame = format_columns(result_frame, types, generation)
                     from relationalai.experimental.inspect import _print_watch_frame
                     _print_watch_frame(result_frame)
 
                 # Process other results
                 else:
-                    result_frame = format_columns(result_frame, types)
+                    result_frame = format_columns(result_frame, types, generation)
                     result["table"] = result_frame
                     if ":output/:cols" in result["relationId"]:
                         matched = re.search(r":col([0-9]+)", result["relationId"])
