@@ -4,7 +4,7 @@ import uuid
 
 from relationalai import debugging
 from relationalai.clients.cache_store import GraphIndexCache
-from relationalai.clients.util import poll_with_specified_overhead
+from relationalai.clients.util import get_pyrel_version, poll_with_specified_overhead
 from relationalai.errors import (
     ERPNotRunningError,
     EngineAutoCreateFailed,
@@ -12,8 +12,8 @@ from relationalai.errors import (
     SnowflakeTableObjectsException,
     SnowflakeTableObject,
 )
-from relationalai.tools.cli_controls import SpanSpinner, Spinner
-from relationalai.tools.constants import WAIT_FOR_STREAM_SYNC
+from relationalai.tools.cli_controls import DebuggingSpan, Spinner
+from relationalai.tools.constants import WAIT_FOR_STREAM_SYNC, Generation
 
 if TYPE_CHECKING:
     from relationalai.clients.snowflake import Resources
@@ -34,6 +34,7 @@ class UseIndexPoller:
         engine_size: Optional[str],
         program_span_id: Optional[str],
         headers: Optional[Dict],
+        generation: Optional[Generation] = None,
     ):
         self.res = resource
         self.app_name = app_name
@@ -66,8 +67,7 @@ class UseIndexPoller:
         # execution_id is allowed to group use_index call, which belongs to the same loop iteration
         self.execution_id = str(uuid.uuid4())
 
-        from .. import __version__
-        self.pyrel_version = f"pyrel/{__version__}"
+        self.pyrel_version = get_pyrel_version(generation)
 
         self.source_info = self.res._check_source_updates(self.sources)
 
@@ -75,17 +75,19 @@ class UseIndexPoller:
         """
         Standard stream-based polling for use_index.
         """
-        self._maybe_delete_stale()
         with Spinner(
-            "Preparing your data...",
-            "Data preparation complete",
+            "Initializing data index... ",
+            "Setup complete",
             leading_newline=True,
             trailing_newline=True,
         ) as spinner:
+            spinner.update_messages({ "message": "Validating data sources..." })
+            self._maybe_delete_stale(spinner)
+            spinner.update_messages({ "message": "Indexing your data in the background. (This may take a while)..." })
             self._poll_loop(spinner)
-        self._post_check(spinner)
+            self._post_check(spinner)
 
-    def _maybe_delete_stale(self) -> None:
+    def _maybe_delete_stale(self, spinner) -> None:
         with debugging.span("check_sources"):
             # Source tables that have been altered/changed since the last stream creation
             stale_sources = [
@@ -94,7 +96,7 @@ class UseIndexPoller:
                 if info["state"] == "STALE"
             ]
         if stale_sources:
-            with SpanSpinner("validate_sources", "Validating data sources...", "Data sources validated", leading_newline=True):
+            with DebuggingSpan("validate_sources"):
                 try:
                     # Delete all stale streams, so use_index could recreate them again
                     from relationalai.clients.snowflake import PYREL_ROOT_DB
@@ -181,13 +183,13 @@ class UseIndexPoller:
                             break_loop = True
 
                         if status == "created":
-                            spinner.update_messages({ "message": f"Preparing {data.get('fq_object_name')}..." })
+                            spinner.update_messages({ "message": f"Indexing {data.get('fq_object_name')}..." })
                         if (status != "synced" or data.get("pending_batches_count") > 0):
                             batches_count = data.get("pending_batches_count")
                             if batches_count:
-                                spinner.update_messages({ "message": f"Preparing {data.get('fq_object_name')}, remaining batches: {data.get('pending_batches_count')}..." })
+                                spinner.update_messages({ "message": f"Indexing {data.get('fq_object_name')}, remaining batches: {data.get('pending_batches_count')}..." })
                             else:
-                                spinner.update_messages({ "message": f"Preparing {data.get('fq_object_name')}..." })
+                                spinner.update_messages({ "message": f"Indexing {data.get('fq_object_name')}..." })
                     self.counter += 1
 
                 if not ready and engines:
@@ -263,7 +265,7 @@ class UseIndexPoller:
 
             if num_tables_altered > 0:
                 s = "s" if num_tables_altered > 1 else ""
-                spinner.update_messages({ "message": f"Enabled change tracking on {num_tables_altered} table{s}. Still preparing your data..." })
+                spinner.update_messages({ "message": f"Enabled change tracking on {num_tables_altered} table{s}..." })
                 self._poll_loop(spinner)
 
 class DirectUseIndexPoller(UseIndexPoller):
@@ -280,8 +282,9 @@ class DirectUseIndexPoller(UseIndexPoller):
         engine_size: Optional[str],
         program_span_id: Optional[str],
         headers: Optional[Dict],
+        generation: Optional[Generation] = None,
     ):
-        super().__init__(resource, app_name, sources, model, engine_name, engine_size, program_span_id, headers)
+        super().__init__(resource, app_name, sources, model, engine_name, engine_size, program_span_id, headers, generation)
         from relationalai.clients.snowflake import DirectAccessResources
         self.res: DirectAccessResources = cast(DirectAccessResources, self.res)
 
