@@ -156,13 +156,7 @@ def is_called_by_streamlit_write():
     stack_frames = inspect.stack()
 
     for frame in stack_frames[1:]:
-        frame_info = frame
-        context = frame_info.code_context[0] if frame_info.code_context else ""
-
-        if (
-            frame_info.frame.f_globals.get("__name__") == "streamlit.runtime.scriptrunner.magic_funcs"
-            and "st.write" in context
-        ):
+        if ("streamlit" in frame.filename and "write" in frame.function):
             return True
 
     return False
@@ -443,11 +437,11 @@ class Context():
                 raise e
             except RAIException as e:
                 handle_missing_integration(e)
-                raise e.clone() from None
+                raise e.clone(self.graph._config) from None
         else:
             self.graph._pop(self, exec=False)
             if isinstance(args[1], RAIException):
-                raise args[1].clone() from None
+                raise args[1].clone(self.graph._config) from None
         return False
 
     def __iter__(self):
@@ -1749,7 +1743,7 @@ class Graph:
             self.resources.cancel_pending_transactions()
             raise e
         except RAIException as e:
-            raise e.clone() from None
+            raise e.clone(self._config) from None
 
     def install_raw(self, code:str, name:str|None = None, overwrite=False):
         self._client.install_raw(code, name, overwrite)
