@@ -4,7 +4,8 @@ Builder API for RelationalAI.
 
 from relationalai.early_access.builder.builder import (
     Model, Concept, Relationship, RelationshipReading, Expression, Fragment, Error,
-    String, Integer, Float, Decimal, Decimal64, Decimal128, Bool, Date, DateTime,
+    String, Integer, Int64, Int128, Float, Decimal, Decimal64, Decimal128, Bool,
+    Date, DateTime,
     RawSource, Hash,
     select, where, require, define, distinct, union, data,
     rank, asc, desc,
@@ -14,7 +15,8 @@ from relationalai.early_access.builder.builder import (
 
 __all__ = [
     "Model", "Concept", "Relationship", "RelationshipReading", "Expression", "Fragment", "Error",
-    "String", "Integer", "Float", "Decimal", "Decimal64", "Decimal128", "Bool", "Date", "DateTime",
+    "String", "Integer", "Int64", "Int128", "Float", "Decimal", "Decimal64", "Decimal128", "Bool",
+    "Date", "DateTime",
     "RawSource", "Hash",
     "select", "where", "require", "define", "distinct", "union", "data",
     "rank", "asc", "desc",
