@@ -11,6 +11,19 @@ from . import builder as b, annotations as anns
 from relationalai import debugging
 from snowflake.snowpark.context import get_active_session
 
+#--------------------------------------------------
+# Helpers
+#--------------------------------------------------
+
+def schema_dict_to_fields(schema: dict[str, str|b.Concept]) -> list[b.Field]:
+    fields = []
+    for name, type in schema.items():
+        if isinstance(type, str):
+            fields.append(b.Field(name=name, type_str=type))
+        elif isinstance(type, b.Concept):
+            fields.append(b.Field(name=name, type_str=type._name, type=type))
+    return fields
+
 #--------------------------------------------------
 # Globals
 #--------------------------------------------------
@@ -27,12 +40,20 @@ def get_session():
             # TODO: we need a better way to handle global config
 
             # using the resource constructor to differentiate between direct access and
-            # regular resource as this impacts the construction of the session. 
+            # regular resource as this impacts the construction of the session.
             resources = Resources()
             assert isinstance(resources, SnowflakeResources), "Can only retrieve session from Snowflake resources"
             _session = resources.get_sf_session()
     return _session
 
+def is_direct_access_enabled():
+    from relationalai import Resources
+    resources = Resources()
+    if hasattr(resources, "is_direct_access_enabled"):
+        return resources.is_direct_access_enabled()
+    return False
+
+
 #--------------------------------------------------
 # Snowflake sources
 #--------------------------------------------------
@@ -47,7 +68,7 @@ SFTypes = {
     "TIMESTAMP_NTZ": "DateTime",
     "FLOAT": "Float",
     "REAL": "Float",
-    "BOOLEAN": "Boolean",
+    "BOOLEAN": "Bool",
 }
 
 SF_ID_REGEX = re.compile(r'^[A-Za-z_][A-Za-z0-9_$]*$')
@@ -108,17 +129,22 @@ class Table():
     _schemas:dict[tuple[str, str], SchemaInfo] = {}
     _used_sources:OrderedSet[Table] = ordered_set()
 
-    def __init__(self, fqn:str, cols:list[str]|None=None) -> None:
+    def __init__(self, fqn:str, cols:list[str]|None=None, schema:dict[str, str|b.Concept]|None=None) -> None:
         self._fqn = fqn
         parser = IdentityParser(fqn, require_all_parts=True)
         self._database, self._schema, self._table, self._fqn = parser.to_list()
         self._inited = False
-        self._ref = b.Integer.ref("row_id")
+        self._concept = b.Concept(self._fqn, extends=[b.Concept.builtins["RowId"]])
+        b.Concept.globals[self._fqn.lower()] = self._concept
+        self._ref = self._concept.ref("row_id")
         self._cols = {}
         self._col_names = cols
         info = self._schemas.get((self._database, self._schema))
         if not info:
             info = self._schemas[(self._database, self._schema)] = SchemaInfo(self._database, self._schema)
+        if schema:
+            info.fetched.add(self._table)
+            info.tables[self._table] = TableInfo(self, schema_dict_to_fields(schema), [])
         info.tables[self._table].source = self
 
     def _lazy_init(self):
@@ -129,7 +155,7 @@ class Table():
         if self._table not in schema_info.fetched:
             schema_info.fetch()
         table_info = schema_info.tables[self._table]
-        self._rel = b.Relationship(self._fqn, fields=[b.Field(name="RowId", type_str="RowId")] + table_info.fields)
+        self._rel = b.Relationship(self._fqn, fields=[b.Field(name="RowId", type_str=self._fqn, type=self._concept)] + table_info.fields)
         self._rel.annotate(anns.external).annotate(anns.from_cdc)
 
     def __getattr__(self, name: str):
@@ -145,6 +171,16 @@ class Table():
         self._lazy_init()
         key_dict = {sanitize_identifier(k._column_name.lower()): k for k in keys}
 
+        # if no keys are defined but the concept has a reference scheme, try to match
+        # reference schemes with column fields by name
+        if not keys and concept._reference_schemes:
+            sanitized_fields = [sanitize_identifier(n.lower()) for n in self._rel._field_names]
+            for scheme in concept._reference_schemes:
+                if scheme[0]._name in sanitized_fields:
+                    key = Column(self, scheme[0]._name)
+                    keys.append(key)
+                    key_dict[scheme[0]._name] = key
+
         # TODO: this is correct, but the rel backend does the wrong thing with it
         # me = concept.new(**key_dict)
         # items = [me]
@@ -163,15 +199,32 @@ class Table():
             me = self._rel._field_refs[0]
             b.where(self).define(concept(me))
         for field in self._rel._fields[1:]:
-            if sanitize_identifier(field.name.lower()) not in key_dict:
-                update = getattr(concept, field.name.lower())(me, getattr(self, field.name))
+            field_name = sanitize_identifier(field.name.lower())
+            if field_name not in key_dict:
+                r = b.Relationship(
+                    f"{{{concept}}} has {{{field_name}:{field.type_str}}}",
+                    parent=concept,
+                    short_name=field_name,
+                    model=self._rel._model
+                )
+                setattr(concept, field_name, r)
+                relationship = getattr(concept, field_name)
+                table_name = getattr(self, field.name)
+                update = relationship(me, table_name)
                 b.define(update)
 
+    def _to_type(self) -> b.Concept:
+        return self._concept
+
+    def _to_keys(self) -> list[b.Ref]:
+        self._lazy_init()
+        return [self._rel._field_refs[0]]
+
     def _compile_lookup(self, compiler:b.Compiler, ctx:b.CompilerContext):
         self._lazy_init()
         Table._used_sources.add(self)
         compiler.lookup(self._rel, ctx)
-        return
+        return compiler.lookup(b.RelationshipFieldRef(None, self._rel, 0), ctx)
 
 class Column(b.Producer):
     def __init__(self, source:Table, column_name:str) -> None:
@@ -179,6 +232,9 @@ class Column(b.Producer):
         self._column_name = column_name
         self._source = source
 
+    def _to_keys(self) -> list[b.Ref]:
+        return self._source._to_keys()
+
     def _compile_lookup(self, compiler:b.Compiler, ctx:b.CompilerContext):
         compiler.lookup(self._source, ctx)
         for ix, field in enumerate(self._source._rel._field_refs):
