@@ -30,3 +30,9 @@ def isinf(value: _Number) -> b.Expression:
 
 def isnan(value: _Number) -> b.Expression:
     return _make_expr("isnan", value)
+
+def ceil(value: _Number) -> b.Expression:
+    return _make_expr("ceil", value, b.Number.ref("res"))
+
+def floor(value: _Number) -> b.Expression:
+    return _make_expr("floor", value, b.Number.ref("res"))
\ No newline at end of file
