@@ -20,7 +20,7 @@ def concat(s0: _String, s1: _String, *args: _String) -> b.Expression:
     return res
 
 def len(s: _String) -> b.Expression:
-    return _make_expr("num_chars", s, b.Integer.ref("res"))
+    return _make_expr("num_chars", s, b.Int128.ref("res"))
 
 def startswith(s0: _String, s1: _String) -> b.Expression:
     return _make_expr("starts_with", s0, s1)
@@ -43,3 +43,6 @@ def lower(s: _String) -> b.Expression:
 
 def upper(s: _String) -> b.Expression:
     return _make_expr("upper", s, b.String.ref("res"))
+
+def strip(s: _String) -> b.Expression:
+    return _make_expr("strip", s, b.String.ref("res"))
