@@ -5,6 +5,7 @@ from pandas import DataFrame
 from typing import Any, Union
 
 from relationalai.early_access.devmode import Compiler
+from relationalai.early_access.devmode.executor.result_helpers import format_duckdb_columns
 from relationalai.early_access.metamodel import ir, executor as e
 
 class DuckDBExecutor(e.Executor):
@@ -17,7 +18,8 @@ class DuckDBExecutor(e.Executor):
         """ Execute the SQL query directly. """
         connection = duckdb.connect()
         try:
-            result = connection.query(self.compiler.compile(model)).to_df()
-            return result
+            sql = self.compiler.compile(model, {"is_duck_db": True})
+            arrow_table = connection.query(sql).fetch_arrow_table()
+            return format_duckdb_columns(arrow_table.to_pandas(), arrow_table.schema)
         finally:
-            connection.close()
+            connection.close()
\ No newline at end of file
