@@ -1,5 +1,6 @@
 from .denormalize import Denormalize
 from .recursive_union import RecursiveUnion
 from .double_negation import DoubleNegation
+from .sort_output_query import SortOutputQuery
 
-__all__ = ["Denormalize", "RecursiveUnion", "DoubleNegation"]
+__all__ = ["Denormalize", "RecursiveUnion", "DoubleNegation", "SortOutputQuery"]
