@@ -75,6 +75,7 @@ class Select(Node):
     joins: Optional[list[Join]] = None
     group_by: Optional[list[VarRef]] = None
     limit: Optional[int] = None
+    is_output: bool = False
 
 @dataclass(frozen=True)
 class VarRef(Node):
@@ -181,7 +182,12 @@ def to_string(node) -> str:
 @dataclass(frozen=True)
 class Printer(BasePrinter):
 
-    def _join(self, args, sep=', ', indent=0):
+    _RESERVED_NAMES: frozenset[str] = frozenset({"right", "left"})
+
+    def _is_reserved_name(self, name: str) -> bool:
+        return isinstance(name, str) and name.lower() in self._RESERVED_NAMES
+
+    def _join(self, args, sep=', ', indent=0, is_output=False):
         for i, s in enumerate(args):
             if i != 0:
                 if indent != 0:
@@ -189,7 +195,7 @@ class Printer(BasePrinter):
                     self._nl()
                 else:
                     self._print(sep)
-            self.print_node(s, indent)
+            self.print_node(s, indent, is_output=is_output)
 
     def _print_value(self, value, quote_strings:bool = False):
         if isinstance(value, tuple):
@@ -211,7 +217,10 @@ class Printer(BasePrinter):
         else:
             self._print(str(value))
 
-    def print_node(self, node:Node, indent=0, inlined=False) -> None:
+    def _get_table_name(self, name: str) -> str:
+        return f'"{name}"' if self._is_reserved_name(name) else name
+
+    def print_node(self, node:Node, indent=0, inlined=False, is_output=False) -> None:
         # Table
         if isinstance(node, Program):
             for idx, d in enumerate(node.statements):
@@ -220,7 +229,7 @@ class Printer(BasePrinter):
                 if idx != len(node.statements) - 1:
                     self._nl()
         elif isinstance(node, Table):
-            self._print(f"{node.name} (")
+            self._print(f"{self._get_table_name(node.name)} (")
             self._join(node.columns)
             self._print(")")
         elif isinstance(node, Column):
@@ -232,12 +241,12 @@ class Printer(BasePrinter):
             self._print(f"CREATE TABLE {clause}{node.table};")
         elif isinstance(node, CreateView):
             # TODO - crying a bit inside :(
-            self._print_nl(f"DROP TABLE IF EXISTS {node.name};")
-            self._print(f"CREATE VIEW {node.name} AS ")
+            self._print_nl(f"DROP TABLE IF EXISTS {self._get_table_name(node.name)};")
+            self._print(f"CREATE VIEW {self._get_table_name(node.name)} AS ")
             self.print_node(node.query, indent, True)
             self._print(";")
         elif isinstance(node, Insert):
-            self._print(f"INSERT INTO {node.table} ")
+            self._print(f"INSERT INTO {self._get_table_name(node.table)} ")
             if len(node.columns) > 0:
                 self._print("(")
                 self._join(node.columns)
@@ -254,7 +263,7 @@ class Printer(BasePrinter):
                 self.print_node(node.select, indent, True)
             self._print(";")
         elif isinstance(node, Update):
-            self._print(f"UPDATE {node.table} SET ")
+            self._print(f"UPDATE {self._get_table_name(node.table)} SET ")
             self._join(node.set)
             if node.where is not None:
                 self.print_node(node.where, indent)
@@ -265,7 +274,7 @@ class Printer(BasePrinter):
             self._indent_print(indent, "SELECT ")
             if node.distinct and node.froms:
                 self._print("DISTINCT ")
-            self._join(node.vars)
+            self._join(node.vars, is_output=node.is_output or is_output)
             if node.froms:
                 self._print(" FROM ")
                 if isinstance(node.froms, Select):
@@ -295,12 +304,14 @@ class Printer(BasePrinter):
                     self._print(f'{node.name}."{node.column}"')
                 else:
                     self._print(f"{node.name}.{node.column}")
-            if node.alias and node.alias != node.column:
-                if '-' in node.alias:
-                    # If alias contains a dash, we need to quote it
+            if node.alias:
+                if is_output:
                     self._print(f' as "{node.alias}"')
-                else:
-                    self._print(f" as {node.alias}")
+                elif node.alias != node.column:
+                    if any(c in node.alias for c in ['-', '?']):
+                        self._print(f' as "{node.alias}"')
+                    else:
+                        self._print(f" as {node.alias}")
         elif isinstance(node, RowNumberVar):
             self._print("ROW_NUMBER() OVER (")
             if node.partition_by_vars:
@@ -308,7 +319,10 @@ class Printer(BasePrinter):
                 self._join(node.partition_by_vars)
             self._print(" ORDER BY ")
             self._join(node.order_by_vars)
-            self._print(f" ) as {node.alias}")
+            if is_output:
+                self._print(f' ) as "{node.alias}"')
+            else:
+                self._print(f' ) as {node.alias}')
         elif isinstance(node, OrderByVar):
             self._print(node.var)
             if node.is_ascending:
@@ -316,7 +330,7 @@ class Printer(BasePrinter):
             else:
                 self._print(" DESC")
         elif isinstance(node, From):
-            self._print(node.table)
+            self._print(self._get_table_name(node.table))
             if node.alias is not None:
                 self._print(f" AS {node.alias}")
         elif isinstance(node, Join):
@@ -325,7 +339,7 @@ class Printer(BasePrinter):
                 else "FULL OUTER JOIN" if isinstance(node, FullOuterJoin)
                 else "JOIN"
             )
-            self._print(f"{join_type} {node.table}")
+            self._print(f"{join_type} {self._get_table_name(node.table)}")
             if node.alias is not None:
                 self._print(f" AS {node.alias}")
             self._print(" ON ")
@@ -343,7 +357,7 @@ class Printer(BasePrinter):
             self._print("WITH ")
             if node.recursive:
                 self._print("RECURSIVE ")
-            self._print(f"{node.name} (")
+            self._print(f"{self._get_table_name(node.name)} (")
             self._join(node.columns)
             self._print_nl(") AS (")
             for i, s in enumerate(node.selects):
@@ -352,7 +366,7 @@ class Printer(BasePrinter):
                     self._indent_print_nl(indent + 2, "UNION ALL")
                 self.print_node(s, indent + 1, True)
             self._nl()
-            self._print(f") SELECT * FROM {node.name}")
+            self._print(f") SELECT * FROM {self._get_table_name(node.name)}")
             if not inlined:
                 self._print(";")
         # --------------------------------------------------
