@@ -1,14 +1,17 @@
 import warnings
 from collections import defaultdict
+from typing import Optional
+from relationalai import Config
 from relationalai.early_access.dsl.adapters.orm.model import ExclusiveInclusiveSubtypeFact, ExclusiveSubtypeFact, \
-    ORMRingType, ORMValueComparisonOperator
+    InclusiveSubtypeFact, ORMRingType, ORMValueComparisonOperator, ORMRange
 from relationalai.early_access.dsl.adapters.orm.parser import ORMParser
 from relationalai.early_access.dsl.core.utils import to_pascal_case
 from relationalai.early_access.dsl.orm.constraints import RingType, ValueComparisonType, Range
 from relationalai.early_access.dsl.orm.models import Model
-from relationalai.early_access.builder import Integer, String, DateTime, Decimal, Date, Float
+from relationalai.early_access.builder import Concept, Integer, String, DateTime, Decimal, Date, Float
 from relationalai.early_access.metamodel.util import NameCache
-
+from datetime import date, datetime
+from decimal import Decimal as PyDecimal
 
 class ORMAdapterQB:
     _datatype_mapping = {
@@ -33,6 +36,16 @@ class ORMAdapterQB:
         "TimeTemporalDataType": DateTime,
     }
 
+    _builtin_concepts_to_python_types = {
+        Concept.builtins["Int"]: int,
+        Concept.builtins["Float"]: float,
+        Concept.builtins["String"]: str,
+        Concept.builtins["Bool"]: bool,
+        Concept.builtins["Date"]: date,
+        Concept.builtins["DateTime"]: datetime,
+        Concept.builtins["Decimal128"]: PyDecimal
+    }
+
     _ring_type_mapping = {
         ORMRingType.IRREFLEXIVE: RingType.IRREFLEXIVE,
         ORMRingType.ANTISYMMETRIC: RingType.ANTISYMMETRIC,
@@ -55,15 +68,15 @@ class ORMAdapterQB:
         ORMValueComparisonOperator.EQUAL: ValueComparisonType.EQUAL
     }
 
-    def __init__(self, orm_file_path: str):
+    def __init__(self, orm_file_path: str, model_name: Optional[str] = None, config: Optional[Config] = None):
         self._parser = ORMParser(orm_file_path)
         self._relationship_role_value_constraints = defaultdict()
         self._relationships = {}
         self.name_cache = NameCache()
-        self.model = self.orm_to_model()
+        self.model = self._orm_to_model(model_name, config)
 
-    def orm_to_model(self):
-        model = Model(self._parser.model_name())
+    def _orm_to_model(self, model_name, config):
+        model = Model(model_name, config=config) if model_name else Model(self._parser.model_name(), config=config)
 
         self._add_value_types(model)
         self._add_entity_types(model)
@@ -82,18 +95,29 @@ class ORMAdapterQB:
 
     def _add_value_types(self, model):
         enum_to_values = defaultdict(list)
+        # Value types having a value type constraint with only string values are considered Enums
+        for v in self._parser.value_type_value_constraints().values():
+            if self._is_value_type_an_enum(v.id):
+                enum_to_values[v.id].extend(v.values)
+        # Value types playing a role in only one relationship that has a role value constraint with only
+        # string values are also considered Enums
         for rvc in self._parser.role_value_constraints().values():
-            if len(rvc.values) > 0 and isinstance(rvc.values[0], str):
-                enum_to_values[rvc.role.player].extend(rvc.values)
-        # create an Enum instead of ValueType when there is a role value constraint (only string sequence) played by this ValueType
+            if self._is_value_type_playing_role_an_enum(rvc.role.id):
+                if rvc.role.player not in enum_to_values.keys():
+                    enum_to_values[rvc.role.player].extend(rvc.values)
+
         cardinality_constraints = self._parser.objects_type_to_cardinality_constraints()
+        value_constraints = self._parser.value_type_value_constraints()
         for vt in self._parser.value_types().values():
             if vt.id in enum_to_values:
                 concept = model.Enum(to_pascal_case(vt.name), enum_to_values[vt.id])
             else:
                 concept = model.Concept(vt.name, extends=[self._datatype_mapping.get(vt.data_type, String)])
             if vt.name in cardinality_constraints:
-                model.cardinality(concept, self._build_constraint_ranges(cardinality_constraints[vt.name]))
+                model.cardinality(concept, self._build_constraint_ranges(cardinality_constraints[vt.name], int))
+            if vt.id in value_constraints:
+                tp = self._builtin_concepts_to_python_types[concept._extends[0]]
+                model.value_constraint(concept, self._build_constraint_ranges(value_constraints[vt.id], tp))
 
     def _add_entity_types(self, model):
         extended_concepts = [y.subtype_name for x in self._parser.subtype_facts().values() for y in x]
@@ -102,7 +126,7 @@ class ORMAdapterQB:
             if et.name not in extended_concepts:
                 concept = model.Concept(et.name)
                 if et.name in cardinality_constraints:
-                    model.cardinality(concept, self._build_constraint_ranges(cardinality_constraints[et.name]))
+                    model.cardinality(concept, self._build_constraint_ranges(cardinality_constraints[et.name], int))
 
         subtype_facts = self._parser.sorted_subtype_facts()
         for st_fact in subtype_facts:
@@ -111,34 +135,40 @@ class ORMAdapterQB:
             if parent_entity is None:
                 parent_entity = model.Concept(parent)
                 if parent in cardinality_constraints:
-                    model.cardinality(parent_entity, self._build_constraint_ranges(cardinality_constraints[parent]))
+                    model.cardinality(parent_entity, self._build_constraint_ranges(cardinality_constraints[parent], int))
             child = st_fact.subtype_name
             child_entity = model.Concept(child, extends=[parent_entity])
             if child in cardinality_constraints:
-                model.cardinality(child_entity, self._build_constraint_ranges(cardinality_constraints[child]))
+                model.cardinality(child_entity, self._build_constraint_ranges(cardinality_constraints[child], int))
 
     def _add_subtype_relationships(self, model):
         for parent, children in self._parser.subtype_facts().items():
-            exclusives = []
-            exclusives_and_inclusives = []
+            exclusive_subtypes = []
+            inclusive_subtypes = []
+            exclusive_inclusive_subtypes = []
             for child in children:
                 sub = model.lookup_concept(child.subtype_name)
                 if isinstance(child, ExclusiveInclusiveSubtypeFact):
-                    exclusives_and_inclusives.append(sub)
+                    exclusive_inclusive_subtypes.append(sub)
                 elif isinstance(child, ExclusiveSubtypeFact):
-                    exclusives.append(sub)
+                    exclusive_subtypes.append(sub)
+                elif isinstance(child, InclusiveSubtypeFact):
+                    inclusive_subtypes.append(sub)
 
-            if len(exclusives_and_inclusives) > 0:
-                model.exclusive_subtype_constraint(*exclusives_and_inclusives)
-                model.inclusive_subtype_constraint(*exclusives_and_inclusives)
-            if len(exclusives) > 0:
-                model.exclusive_subtype_constraint(*exclusives)
+            if len(exclusive_inclusive_subtypes) > 0:
+                model.exclusive_subtype_constraint(*exclusive_inclusive_subtypes)
+                model.inclusive_subtype_constraint(*exclusive_inclusive_subtypes)
+            if len(exclusive_subtypes) > 0:
+                model.exclusive_subtype_constraint(*exclusive_subtypes)
+            if len(inclusive_subtypes) > 0:
+                model.inclusive_subtype_constraint(*inclusive_subtypes)
 
     def _add_relationships(self, model):
         object_types = self._parser.object_types()
         unique_roles = self._parser.unique_roles()
         mandatory_roles = self._parser.mandatory_roles()
         role_value_constraints = self._parser.role_value_constraints()
+        role_cardinality_constraints = self._parser.role_cardinality_constraints()
         fact_type_to_internal_ucs = self._parser.fact_type_to_internal_ucs()
         fact_type_to_complex_ucs = self._parser.fact_type_to_complex_ucs()
         fact_type_to_roles = self._parser.fact_type_to_roles()
@@ -171,6 +201,9 @@ class ORMAdapterQB:
                     model.mandatory(role)
                 if role_id in role_value_constraints.keys():
                     self._relationship_role_value_constraints[role] = role_value_constraints[role_id].values
+                if role_id in role_cardinality_constraints.keys():
+                    model.role_cardinality(role, self._build_constraint_ranges(role_cardinality_constraints[role_id], int))
+
 
             # Adding alternative readings
             self._validate_role_verbalization(reading_orders)
@@ -211,14 +244,24 @@ class ORMAdapterQB:
             # Identifying external UCs
             if uc.identifies:
                 entity = model.lookup_concept(object_types[uc.identifies].name)
-                identifying_rel = []
+                identifying_relationships = []
                 for ro in uc.roles:
                     relationship = self._relationships[roles[ro].relationship_name]
-                    # TODO: this will change when we can use readings or roles with identify_by
-                    if relationship._roles()[0]._concept._name != entity._name:
-                        raise ValueError(f"The concept {entity._name} must be the first player in all its identifying relationships.")
-                    identifying_rel.append(relationship)
-                entity.identify_by(*identifying_rel)
+                    first_player = relationship._roles()[0]._concept
+                    if first_player._name == entity._name:
+                        identifying_relationships.append(relationship)
+                    else:
+                        alt_id_reading = self._get_alternative_identifying_reading(entity, relationship)
+                        if alt_id_reading:
+                            identifying_relationships.append(alt_id_reading)
+                        else:
+                            # Unless there exists an alternative reading where the identified entity is the first player,
+                            # we cannot use this relationship as identifying. The issue is that without an explicit
+                            # reading, we have no way to name and index the identifying relationship which in turn
+                            # needs to be passed to qb identify_by.
+                            # See also: https://relationalai.atlassian.net/browse/RAI-39727.
+                            raise ValueError(f"The identifying relationship {first_player._name}.{relationship._name} must have a reading where the identified entity is the first player.")
+                entity.identify_by(*identifying_relationships)
             # Non identifying external UCs
             else:
                 role_list = []
@@ -267,8 +310,24 @@ class ORMAdapterQB:
             model.frequency(constraint_frequency, *constraint_roles)
 
     def _add_role_value_constraints(self, model):
-        for rel, values in self._relationship_role_value_constraints.items():
-            model.role_value_constraint(rel, values)
+        for role, values in self._relationship_role_value_constraints.items():
+            extends = role._concept._extends[0]
+            # If the value type was created as an Enum, we can use the values directly
+            if extends._name == "Enum":
+                model.role_value_constraint(role, values)
+            # Otherwise we will process a mix of single values and ORMRange objects
+            else:
+                tp = self._builtin_concepts_to_python_types.get(extends)
+                if tp:
+                    constraint_values = []
+                    for value in values:
+                        if isinstance(value, ORMRange):
+                            constraint_values.append(self._build_constraint_range(value, tp))
+                        else:
+                            constraint_values.append(tp(value))
+                    model.role_value_constraint(role, constraint_values)
+                else:
+                    raise ValueError(f"Unsupported type for role value constraint: {extends._name}.")
 
     def _build_reading(self, model, reading_order):
         object_types = self._parser.object_types()
@@ -317,14 +376,55 @@ class ORMAdapterQB:
         else:
             return [self._role_id_to_role_object(model, roles[ro]) for ro in orm_constraint.roles]
 
+    # A Value type that plays a given role is an Enum when there is a role value constraint containing only string
+    # values declared on this ValueType, and it does not play a role in any other relationship
+    def _is_value_type_playing_role_an_enum(self, role_id):
+        role_value_constraint = self._parser.role_value_constraints().get(role_id)
+        if role_value_constraint:
+            vt_id = role_value_constraint.role.player
+            return self._is_value_type_an_enum(vt_id) or (
+                    role_value_constraint.values
+                    and all(isinstance(v, str) for v in role_value_constraint.values)
+                    and self._count_relationships_for_player(vt_id) == 1
+            )
+        return False
+
+    # A Value Type is an Enum when it has a value type constraint containing only string values
+    def _is_value_type_an_enum(self, vt_id):
+        value_type_constraint = self._parser.value_type_value_constraints().get(vt_id)
+        return (value_type_constraint and value_type_constraint.ranges
+                and all(isinstance(v, str) for v in value_type_constraint.ranges))
+
+    def _count_relationships_for_player(self, player_id):
+        count = 0
+        for v in self._parser._role_to_player.values():
+            if v == player_id:
+                count += 1
+        return count
+
     @staticmethod
-    def _build_constraint_ranges(constraint):
+    def _get_alternative_identifying_reading(entity, id_relationship):
+        if len(id_relationship._readings) > 0:
+            for rd in id_relationship._readings[1:]:
+                roles = rd._roles()
+                if roles[0]._concept._name == entity._name:
+                    return rd
+        return None
+
+    @staticmethod
+    def _build_constraint_range(range: ORMRange, tp: type):
+        if range.range_from == range.range_to:
+            return tp(range.range_from)
+        return Range.between(tp(range.range_from), tp(range.range_to))
+
+    @staticmethod
+    def _build_constraint_ranges(constraint, tp: type):
         constraint_ranges = []
         for rg in constraint.ranges:
-            if rg.range_from == rg.range_to:
-                constraint_ranges.append(int(rg.range_from))
+            if isinstance(rg, ORMRange):
+                constraint_ranges.append(ORMAdapterQB._build_constraint_range(rg, tp))
             else:
-                constraint_ranges.append(Range.between(int(rg.range_from), int(rg.range_to)))
+                constraint_ranges.append(tp(rg))
         return constraint_ranges
 
     @staticmethod
