@@ -1,8 +1,7 @@
 from __future__ import annotations
 from dataclasses import dataclass
 from enum import Enum
-from typing import Optional, Any
-
+from typing import Optional, Union
 
 @dataclass
 class ORMEntityType:
@@ -44,6 +43,9 @@ class SubtypeFact:
 class ExclusiveSubtypeFact(SubtypeFact):
     pass
 
+@dataclass
+class InclusiveSubtypeFact(SubtypeFact):
+    pass
 
 @dataclass
 class ExclusiveInclusiveSubtypeFact(SubtypeFact):
@@ -137,7 +139,7 @@ class ORMReadingRole:
 @dataclass
 class ORMRoleValueConstraint:
     role: ORMRole
-    values: list[Any]
+    values: list[Union[str, ORMRange]]
 
 
 @dataclass
@@ -168,8 +170,8 @@ class ORMFrequencyConstraint:
 
 @dataclass
 class ORMRange:
-    range_from: int
-    range_to: int
+    range_from: str
+    range_to: str
 
 
 @dataclass
@@ -177,3 +179,16 @@ class ORMCardinalityConstraint:
     id: str
     object_type: str
     ranges: list[ORMRange]
+
+
+@dataclass
+class ORMRoleCardinalityConstraint:
+    id: str
+    role: str
+    ranges: list[ORMRange]
+
+@dataclass
+class ORMValueTypeValueConstraint:
+    id: str
+    value_type: str
+    ranges: list[ORMRange]
