@@ -1,14 +1,16 @@
 import json
 import re
+import warnings
 
 import xmltodict
 from collections import defaultdict
 from relationalai.early_access.dsl.adapters.orm.model import ORMEntityType, ORMValueType, ORMRole, ORMSubtypeFact, \
     ORMUniquenessConstraint, ORMExclusionConstraint, ORMMandatoryConstraint, ORMReadingRole, ORMReading, \
-    ExclusiveInclusiveSubtypeFact, SubtypeFact, ExclusiveSubtypeFact, ORMInclusionConstraint, \
+    ExclusiveInclusiveSubtypeFact, InclusiveSubtypeFact, ExclusiveSubtypeFact, SubtypeFact, ORMInclusionConstraint, \
     ORMRoleValueConstraint, ORMValueComparisonConstraint, ORMValueComparisonOperator, ORMRingConstraint, ORMRingType, \
     ORMRoleSubsetConstraint, ORMInclusiveRoleConstraint, ORMExclusiveRoleConstraint, ORMEqualityConstraint, \
-    ORMFrequencyConstraint, ORMCardinalityConstraint, ORMRange
+    ORMFrequencyConstraint, ORMCardinalityConstraint, ORMRange, ORMRoleCardinalityConstraint, \
+    ORMValueTypeValueConstraint
 from relationalai.early_access.metamodel.util import ordered_set
 from relationalai.util.graph import topological_sort
 
@@ -23,6 +25,8 @@ class ORMParser:
 
         self._role_to_player = dict()
         self._cardinality_constraints = dict()
+        self._role_cardinality_constraints = dict()
+        self._value_type_value_constraints = dict()
 
         self._model_name = self._parse_model_name()
         self._entity_types = self._parse_entity_types()
@@ -104,6 +108,12 @@ class ORMParser:
     def cardinality_constraints(self):
         return self._cardinality_constraints
 
+    def value_type_value_constraints(self):
+        return self._value_type_value_constraints
+    
+    def role_cardinality_constraints(self):
+        return self._role_cardinality_constraints
+
     def unique_roles(self):
         return self._unique_roles
 
@@ -177,8 +187,17 @@ class ORMParser:
                     value_types[id] = ORMValueType(id, name, data_type)
                     self._parse_role_to_player_reference(vt)
                     self._parse_cardinality_constraint(vt)
+                    self._parse_value_type_value_constraints(vt)
         return value_types
 
+    def _parse_value_type_value_constraints(self, vt):
+        vt_id = vt.get("@id")
+        value_constraint = self._get_nested(vt,  "orm:ValueRestriction", "orm:ValueConstraint")
+        if value_constraint:
+            vc_id = value_constraint.get("@id")
+            range_values = self._parse_constraint_range(value_constraint)
+            self._value_type_value_constraints[vt_id] = ORMValueTypeValueConstraint(vc_id, vt, range_values)
+
     def _parse_data_types(self):
         data_types = {}
         orm_data_types = self._get_nested(self._ontology, "ormRoot:ORM2", "orm:ORMModel", "orm:DataTypes")
@@ -202,6 +221,7 @@ class ORMParser:
                         role_player = self._get_nested(ro, "orm:RolePlayer", "@ref")
                         if role_player in self._object_types:
                             roles[role_id] = ORMRole(role_id, role_name, relationship_name, self._role_to_player[role_id])
+                        self._parse_role_cardinality_constraint(ro)
         return roles
 
     def _parse_role_to_player_reference(self, object_type):
@@ -218,17 +238,28 @@ class ORMParser:
                 roles = self._get_nested(ft, "orm:FactRoles", "orm:Role")
                 if roles and isinstance(roles, list):
                     for ro in roles:
-                        value_ranges = self._get_nested(ro, "orm:ValueRestriction", "orm:RoleValueConstraint",
-                                                        "orm:ValueRanges", "orm:ValueRange")
-                        if value_ranges:
-                            values = []
-                            for rvc in value_ranges:
-                                if rvc.get("@MinValue") != rvc.get("@MaxValue"):
-                                    raise Exception("Unsupported value range constraint.")
-                                values.append(rvc["@MinValue"])
-                            role_value_constraints[ro["@id"]] = ORMRoleValueConstraint(self._roles[ro["@id"]], values)
+                        role_value_constraint = self._get_nested(ro, "orm:ValueRestriction", "orm:RoleValueConstraint")
+                        if role_value_constraint:
+                            range_values = self._parse_constraint_range(role_value_constraint)
+                            role_value_constraints[ro["@id"]] = ORMRoleValueConstraint(self._roles[ro["@id"]], range_values)
         return role_value_constraints
 
+    def _parse_constraint_range(self, orm_constraint_object):
+        values = []
+        value_range = self._get_nested(orm_constraint_object, "orm:ValueRanges", "orm:ValueRange")
+        if value_range:
+            for rvc in self._single_object_to_list(value_range):
+                min_value = rvc.get("@MinValue")
+                max_value = rvc.get("@MaxValue")
+                if min_value and max_value:
+                    if min_value == max_value:
+                        values.append(min_value)
+                    else:
+                        values.append(ORMRange(min_value, max_value))
+                else:
+                    raise ValueError("Range value must have both MinValue and MaxValue defined.")
+        return values
+
     def _parse_inclusive_role_constraints(self):
         constraints = []
         for ec in self._inclusion_constraints.values():
@@ -421,61 +452,77 @@ class ORMParser:
         return fact_types_readings
 
     def _parse_fact_types_reading_order(self, reading_order):
-        expanded_data = self._get_nested(reading_order, "orm:Readings", "orm:Reading", "orm:ExpandedData")
-        if expanded_data:
-            front_text = expanded_data.get("@FrontText", None)
-            if front_text:
-                front_text = front_text.strip()
+        results = []
+        readings = self._get_nested(reading_order, "orm:Readings", "orm:Reading",)
+        if readings:
+            for rd in self._single_object_to_list(readings):
+                expanded_data = self._get_nested(rd,  "orm:ExpandedData")
+                if expanded_data:
+                    front_text = expanded_data.get("@FrontText", None)
+                    if front_text:
+                        front_text = front_text.strip()
 
-            role_texts = self._get_nested(expanded_data,"orm:RoleText")
-            if not role_texts:
-                return None
+                    role_texts = self._get_nested(expanded_data,"orm:RoleText")
+                    if role_texts:
+                        # Step 1: Build roles with whatever RoleText exists
+                        index_to_role = {}
+                        for rt in self._single_object_to_list(role_texts):
+                            index = int(rt["@RoleIndex"])
+                            index_to_role[index] = ORMReadingRole(
+                                index=index,
+                                prefix=rt.get("@PreBoundText", "").strip() or None,
+                                postfix=rt.get("@PostBoundText", "").strip() or None,
+                                text=rt.get("@FollowingText", "").strip() or None,
+                                role=None
+                            )
 
-            # Step 1: Build roles with whatever RoleText exists
-            index_to_role = {}
-            for rt in self._single_object_to_list(role_texts):
-                index = int(rt["@RoleIndex"])
-                index_to_role[index] = ORMReadingRole(
-                    index=index,
-                    prefix=rt.get("@PreBoundText", "").strip() or None,
-                    postfix=rt.get("@PostBoundText", "").strip() or None,
-                    text=rt.get("@FollowingText", "").strip() or None,
-                    role=None
-                )
+                        # Step 2: Ensure all roles from RoleSequence are represented, even if no RoleText exists
+                        roles = self._get_nested(reading_order, "orm:RoleSequence", "orm:Role")
+                        if roles:
+                            for i, role in enumerate(self._single_object_to_list(roles)):
+                                role_id = role.get("@ref")
+                                orm_role = self._roles.get(role_id, None)
+                                if i in index_to_role:
+                                    index_to_role[i].role = orm_role
+                                else:
+                                    # Fill in with None/defaults if RoleText is missing
+                                    index_to_role[i] = ORMReadingRole(
+                                        index=i,
+                                        prefix=None,
+                                        postfix=None,
+                                        text=None,
+                                        role=orm_role
+                                    )
+                        results.append(ORMReading(front_text, [index_to_role[i] for i in sorted(index_to_role)]))
+        if len(results) > 1:
+            warnings.warn("Multiple readings for the same reading order are not supported. Using the first one.")
+        return results[0] if results else None
 
-            # Step 2: Ensure all roles from RoleSequence are represented, even if no RoleText exists
-            roles = self._get_nested(reading_order, "orm:RoleSequence", "orm:Role")
-            if roles:
-                for i, role in enumerate(self._single_object_to_list(roles)):
-                    role_id = role.get("@ref")
-                    orm_role = self._roles.get(role_id, None)
-                    if i in index_to_role:
-                        index_to_role[i].role = orm_role
-                    else:
-                        # Fill in with None/defaults if RoleText is missing
-                        index_to_role[i] = ORMReadingRole(
-                            index=i,
-                            prefix=None,
-                            postfix=None,
-                            text=None,
-                            role=orm_role
-                        )
-
-            return ORMReading(front_text, [index_to_role[i] for i in sorted(index_to_role)])
+    def _parse_cardinality_constraint_range(self, cardinality_constraint):
+        ranges = self._get_nested(cardinality_constraint, "orm:Ranges", "orm:CardinalityRange")
+        # At least a range is always present within a cardinality constraint
+        orm_ranges = []
+        for rg in self._single_object_to_list(ranges):
+            range_from = rg.get("@From")
+            range_to = rg.get("@To")
+            orm_ranges.append(ORMRange(range_from, range_to))
+        return orm_ranges
 
     def _parse_cardinality_constraint(self, object_type):
         orm_cc = self._get_nested(object_type, "orm:CardinalityRestriction", "orm:CardinalityConstraint")
         if orm_cc:
             object_type_id = object_type.get("@id")
             cc_id = orm_cc.get("@id")
-            ranges = self._get_nested(orm_cc, "orm:Ranges", "orm:CardinalityRange")
-            # At least a range is always present within a cardinality constraint
-            orm_ranges = []
-            for rg in self._single_object_to_list(ranges):
-                range_from = rg.get("@From")
-                range_to = rg.get("@To")
-                orm_ranges.append(ORMRange(range_from, range_to))
-            self._cardinality_constraints[cc_id] = ORMCardinalityConstraint(cc_id, object_type_id, orm_ranges)
+            ranges = self._parse_cardinality_constraint_range(orm_cc)
+            self._cardinality_constraints[cc_id] = ORMCardinalityConstraint(cc_id, object_type_id, ranges)
+
+    def _parse_role_cardinality_constraint(self, role):
+        orm_rcc = self._get_nested(role, "orm:CardinalityRestriction", "orm:UnaryRoleCardinalityConstraint")
+        if orm_rcc:
+            role_id = role.get("@id")
+            cc_id = orm_rcc.get("@id")
+            ranges = self._parse_cardinality_constraint_range(orm_rcc)
+            self._role_cardinality_constraints[role_id] = ORMRoleCardinalityConstraint(cc_id, role_id, ranges)
 
     @staticmethod
     def _get_nested(d, *keys):
@@ -488,12 +535,22 @@ class ORMParser:
     def _build_subtype_object(self, subtype_arrow):
         sub_name = self._object_types[subtype_arrow.subtype].name
         sup_name = self._object_types[subtype_arrow.supertype].name
-        for ec, ec_metadata in self._exclusion_constraints.items():
+        # Look at exclusion constraints
+        for ec_metadata in self._exclusion_constraints.values():
             flat_role_sequence = [role for role_list in ec_metadata.roles for role in role_list]
-            if ec_metadata.inclusive and subtype_arrow.supertype_role_id in flat_role_sequence:
-                return ExclusiveInclusiveSubtypeFact(sub_name, sup_name)
-            elif subtype_arrow.supertype_role_id in flat_role_sequence:
-                return ExclusiveSubtypeFact(sub_name, sup_name)
+            if subtype_arrow.supertype_role_id in flat_role_sequence:
+                if ec_metadata.inclusive:
+                    return ExclusiveInclusiveSubtypeFact(sub_name, sup_name)
+                else:
+                    return ExclusiveSubtypeFact(sub_name, sup_name)
+        # Look at inclusion constraints
+        for ic_metadata in self._inclusion_constraints.values():
+            if subtype_arrow.supertype_role_id in ic_metadata.roles:
+                if ic_metadata.exclusive:
+                    return ExclusiveInclusiveSubtypeFact(sub_name, sup_name)
+                else:
+                    return InclusiveSubtypeFact(sub_name, sup_name)
+        # Default case, no constraints found
         return SubtypeFact(sub_name, sup_name)
 
     def _parse_mandatory_roles(self):
