@@ -7,7 +7,7 @@ from typing import Union
 
 import relationalai.early_access.builder as qb
 from relationalai.early_access.builder import std
-from relationalai.early_access.builder.builder import Not
+from relationalai.early_access.builder.builder import Not, Ref
 from relationalai.early_access.dsl.orm.relationships import Relationship, Role
 from relationalai.early_access.dsl.orm.types import Concept
 from relationalai.early_access.dsl.snow.common import ColumnRef, ForeignKey
@@ -104,6 +104,10 @@ class BindableAttribute(ABC):
     def decimal_size(self) -> Optional[int]:
         pass
 
+    @abstractmethod
+    def row_ref(self) -> Ref:
+        pass
+
 
 @dataclasses.dataclass(frozen=True)
 class Binding:
@@ -136,7 +140,7 @@ class Binding:
         ))
 
     Examples of transformations:
-        - `TransformWith` can be a QB built-in like `trim` or `lower`.
+        - `TransformWith` can be a QB built-in like `strip` or `lower`.
         - A single *binary* relationship can be used to transform the values in the column, where the type of the value
           must follow the data type defined in the model for the role.
         - A tuple of relationships can be used to transform the values in the column, chaining the transformations.
@@ -145,9 +149,9 @@ class Binding:
         # single string lowercase transformation
         {source}.COLUMN.binds(MyRelationship, transform_with=std.strings.lower)
 
-        # chain of transformations with a trim followed by an uppercase
+        # chain of transformations with a strip followed by an uppercase
         {source}.COLUMN.binds(MyRelationship, transform_with=(
-            std.strings.trim, std.strings.upper
+            std.strings.strip, std.strings.upper
         ))
 
         # manually defined transformation relationship
