@@ -29,7 +29,7 @@ class BindableCsvColumn(BindableColumn):
         We use a separate QB relationship per source column, as this allows proper typing.
         """
         short_name = f"{source.physical_name()}_{col_name}"
-        madlib = f"{short_name} {{Integer}} {{{col_type}}}"
+        madlib = f"{short_name} {{row_id:RowId}} {{{col_type}}}"
         return madlib, short_name
 
     def __call__(self, *args):
@@ -58,6 +58,12 @@ class BindableCsvColumn(BindableColumn):
     def basic_type(self):
         return self._column_basic_type
 
+    def row_ref(self):
+        assert len(self._field_refs) == 2, 'CSV Column relation must have exactly two field references'
+        row_ref = self._field_refs[0]
+        assert row_ref._thing._name == 'RowId', 'First field must be of type RowId'
+        return row_ref
+
     def decimal_scale(self) -> Optional[int]:
         if self.type() is qb.Decimal64:
             return DECIMAL64_SCALE
