@@ -28,7 +28,7 @@ class BindableSnowflakeColumn(BindableColumn):
         relation; hence the name is the same.
         """
         short_name = source.physical_name()
-        madlib = f"{short_name} {{Symbol}} {{Integer}} {{{col_type}}}"
+        madlib = f"{short_name} {{col_name:Symbol}} {{row_id:RowId}} {{{col_type}}}"
         return madlib, short_name
 
     def __call__(self, *args):
@@ -56,6 +56,12 @@ class BindableSnowflakeColumn(BindableColumn):
     def metadata(self):
         return self._metadata
 
+    def row_ref(self):
+        assert len(self._field_refs) == 3, 'Snowflake Column relation must have exactly three field references'
+        row_ref = self._field_refs[1]
+        assert row_ref._thing._name == 'RowId', 'First field must be of type RowId'
+        return row_ref
+
     def decimal_scale(self):
         return self._metadata.numeric_scale
 
