@@ -3,16 +3,15 @@ from typing import Union
 from relationalai.early_access.dsl.bindings.common import Binding, IdentifierConceptBinding, RoleBinding, \
     SubtypeConceptBinding, BindableTable, BindableAttribute, ReferentConceptBinding
 from relationalai.early_access.dsl.codegen.common import BoundIdentifierConstraint, BoundRelationship
-from relationalai.early_access.dsl.codegen.reasoner import OntologyReasoner
 from relationalai.early_access.dsl.orm.relationships import Role, Relationship
 from relationalai.early_access.metamodel.util import OrderedSet
 
 import relationalai.early_access.builder as qb
 
 class Binder:
-    def __init__(self, model, reasoner: OntologyReasoner):
+    def __init__(self, model):
         self._model = model
-        self._reasoner = reasoner
+        self._reasoner = self._model.reasoner()
 
         # bindings classification
         self._identifier_bindings: OrderedSet[IdentifierConceptBinding] = OrderedSet()
