@@ -1,17 +1,17 @@
 from collections import OrderedDict
 from itertools import product
 from types import MappingProxyType
-from typing import Union, Optional
+from typing import Union, Optional, Sequence
 
 from relationalai.early_access.builder import where
 from relationalai.early_access.dsl.bindings.common import Binding, RoleBinding, IdentifierConceptBinding, BindableTable, \
     BindableAttribute, SubtypeConceptBinding, ReferentConceptBinding
 from relationalai.early_access.dsl.codegen.binder import Binder
 from relationalai.early_access.dsl.codegen.common import BoundIdentifierConstraint, BoundRelationship
-from relationalai.early_access.dsl.codegen.reasoner import OntologyReasoner
 from relationalai.early_access.dsl.codegen.relations import ValueMap, DecimalValueConverter, \
     CompositeEntityMap, EntitySubtypeMap, UnionEntityMap, AbstractEntityMap, SimpleConstructorEntityMap, \
-    ReferentEntityMap, ValueConverter, filtering_view
+    ReferentEntityMap, ValueConverter, _filtering_view, InlineValueMap, Map, MaterializedEntityMap, \
+    AbstractInlineEntityMap
 from relationalai.early_access.metamodel.util import OrderedSet
 from relationalai.early_access.rel.rel_utils import DECIMAL64_SCALE, DECIMAL128_SCALE
 
@@ -19,8 +19,8 @@ import relationalai.early_access.builder as qb
 
 
 DEFAULT_WEAVER_CONFIG = MappingProxyType({
-    'decimal64_scale': DECIMAL64_SCALE,
-    'decimal128_scale': DECIMAL128_SCALE,
+    'inline_value_maps': False,
+    'inline_entity_maps': False,
 })
 
 
@@ -29,18 +29,22 @@ class Weaver:
     def __init__(self, model, config=None):
         self._model = model
         self._config = config or DEFAULT_WEAVER_CONFIG
-        self._reasoner = OntologyReasoner(self._model)
-        self._binder = Binder(self._model, self._reasoner)
+        self._reasoner = self._model.reasoner()
+        self._binder = Binder(self._model)
 
         # Lookup dictionaries for generated relations
         self._decimal_value_converters = OrderedDict()
-        self._binding_to_value_map: dict[Binding, ValueMap] = OrderedDict()
-        self._ctor_binding_to_entity_map: dict[IdentifierConceptBinding, SimpleConstructorEntityMap] = OrderedDict()
-        self._ref_binding_to_entity_map: dict[ReferentConceptBinding, ReferentEntityMap] = OrderedDict()
+        self._binding_to_value_map: dict[Binding, Union[ValueMap, InlineValueMap]] = OrderedDict()
+        self._ctor_binding_to_entity_map:\
+            dict[IdentifierConceptBinding, Union[AbstractInlineEntityMap, MaterializedEntityMap]] = OrderedDict()
+        self._ref_binding_to_entity_map:\
+            dict[ReferentConceptBinding, Union[AbstractInlineEntityMap, MaterializedEntityMap]] = OrderedDict()
         self._bound_identifier_to_composite_entity_map:\
-            dict[BoundIdentifierConstraint, OrderedSet[CompositeEntityMap]] = OrderedDict()
-        self._subtype_binding_to_entity_map: dict[Union[IdentifierConceptBinding, ReferentConceptBinding], EntitySubtypeMap] = OrderedDict()
-        self._filtering_subtype_binding_to_entity_map: dict[SubtypeConceptBinding, EntitySubtypeMap] = OrderedDict()
+            dict[BoundIdentifierConstraint, OrderedSet[Union[AbstractInlineEntityMap, MaterializedEntityMap]]] = OrderedDict()
+        self._subtype_binding_to_entity_map:\
+            dict[Union[IdentifierConceptBinding, ReferentConceptBinding], Union[AbstractInlineEntityMap, MaterializedEntityMap]] = OrderedDict()
+        self._filtering_subtype_binding_to_entity_map:\
+            dict[SubtypeConceptBinding, Union[AbstractInlineEntityMap, MaterializedEntityMap]] = OrderedDict()
         self._supertype_to_entity_map: dict[qb.Concept, UnionEntityMap] = OrderedDict()
 
     def generate(self):
@@ -55,7 +59,10 @@ class Weaver:
     def _generate_value_map(self, binding: Binding):
         role = self._binder.lookup_binding_role(binding)
         converter = self._get_value_converter(binding.column)
-        value_map = ValueMap(self._model, binding, role, converter)
+        if self._cfg_should_inline_value_maps():
+            value_map = InlineValueMap(self._model, binding, role, converter)
+        else:
+            value_map = ValueMap(self._model, binding, role, converter)
         self._binding_to_value_map[binding] = value_map
 
     def _look_up_value_map(self, binding: Binding):
@@ -107,6 +114,7 @@ class Weaver:
         identifier_to_role_map = self._lookup_supertype_identifier_role_maps(binding)
 
         entity_map = SimpleConstructorEntityMap(self._model, binding, role, identifier_to_role_map)
+        entity_map = self._try_materialize_entity_map(entity_map)
         self._ctor_binding_to_entity_map[binding] = entity_map
 
     def _lookup_supertype_identifier_role_maps(self, binding: IdentifierConceptBinding):
@@ -173,6 +181,7 @@ class Weaver:
         assert role is not None, "Referent concept binding must have a sibling role"
         constructing_role_map = self._lookup_constructing_role_map(binding, binding.entity_type)
         entity_map = ReferentEntityMap(self._model, binding, role, constructing_role_map)
+        entity_map = self._try_materialize_entity_map(entity_map)
         self._ref_binding_to_entity_map[binding] = entity_map
 
     def _generate_composite_entity_maps(self):
@@ -186,9 +195,10 @@ class Weaver:
         entity_concept = constraint.concept
         for role_maps in role_map_combinations:
             entity_map = CompositeEntityMap(self._model, entity_concept, *role_maps)
+            entity_map = self._try_materialize_entity_map(entity_map)
             self._bound_identifier_to_composite_entity_map.setdefault(constraint, OrderedSet()).add(entity_map)
 
-    def _composite_entity_map_combinations(self, bound_uc: BoundIdentifierConstraint) -> list[list[Union[ValueMap, SimpleConstructorEntityMap, ReferentEntityMap]]]:
+    def _composite_entity_map_combinations(self, bound_uc: BoundIdentifierConstraint) -> Sequence[Sequence[Map]]:
         role_bindings = bound_uc.role_bindings
         # Build a list of binding lists for each role in the right order
         role_binding_lists = [role_bindings[role] for role in bound_uc.constraint.roles()]
@@ -286,6 +296,7 @@ class Weaver:
             ctor_binding = self._binder.lookup_subtype_reference_binding(binding)
             ctor_entity_map = self._lookup_ref_role_map_by_ctor_binding(ctor_binding)
         entity_map = EntitySubtypeMap(self._model, binding, ctor_entity_map)
+        entity_map = self._try_materialize_entity_map(entity_map)
         self._subtype_binding_to_entity_map[binding] = entity_map
 
     def _generate_filtered_subtype_entity_maps(self):
@@ -296,6 +307,7 @@ class Weaver:
         ctor_binding = self._binder.lookup_subtype_reference_binding(binding)
         ctor_entity_map = self._lookup_ref_role_map_by_ctor_binding(ctor_binding)
         entity_map = EntitySubtypeMap(self._model, binding, ctor_entity_map)
+        entity_map = self._try_materialize_entity_map(entity_map)
         self._filtering_subtype_binding_to_entity_map[binding] = entity_map
 
     def _generate_supertype_entity_maps(self):
@@ -306,6 +318,8 @@ class Weaver:
                         .update(filtering_subtype_maps))
         for subtype_map in subtype_maps:
             self._generate_supertype_entity_map(subtype_map)
+        # after generating all supertype maps, try to materialize them
+        [entity_map.materialize_population() for entity_map in self._supertype_to_entity_map.values()]
 
     def _generate_supertype_entity_map(self, subtype_map: AbstractEntityMap):
         subtype = subtype_map.value_player()
@@ -351,12 +365,12 @@ class Weaver:
             raise ValueError(
                 f'Cannot generate semantic predicate for {relationship}, no role maps found')
         for entity_map in entity_maps:
-            row = qb.Integer.ref()
-            entity = entity_map.value_player().ref()
+            row_ref = entity_map.row_ref()
+            entity_ref, subformula_atoms = entity_map.formula(row_ref)
             where(
-                entity_map(row, entity),
-                filtering_view(row, binding.filter_by)
-            ).define(relationship(entity))
+                *subformula_atoms,
+                _filtering_view(row_ref, binding.filter_by)
+            ).define(relationship(entity_ref))
 
     def _generate_nary_predicate_rule(self, bound_relationship: BoundRelationship):
         relationship = bound_relationship.relationship
@@ -391,7 +405,28 @@ class Weaver:
         # if we got all roles with role maps, we can generate the rules
         role_map_combinations = [list(combination) for combination in product(*role_to_role_maps.values())]  # pyright: ignore[reportCallIssue]
         for role_maps in role_map_combinations:
-            row, vars = qb.Integer.ref(), [role_map.value_player().ref() for role_map in role_maps]
+            assert len(role_maps) > 0, "Must have at least one role map to derive a relationship"
+            row_ref = role_maps[0].row_ref()  # take the row handle from the first role map arbitrarily
+            var_refs, subformulas = zip(*(rm.formula(row_ref) for rm in role_maps))
             where(
-                *[role_map(row, var) for role_map, var in zip(role_maps, vars)]
-            ).define(relationship(*vars))
+                *(atom for subformula in subformulas for atom in subformula)
+            ).define(relationship(*var_refs))
+
+    def _try_materialize_entity_map(self, entity_map: AbstractInlineEntityMap) -> Union[AbstractInlineEntityMap, MaterializedEntityMap]:
+        if self._cfg_should_inline_entity_maps():
+            entity_map.materialize_population()
+            return entity_map
+        else:
+            return entity_map.materialize()
+
+    def _cfg_should_inline_value_maps(self):
+        """
+        Check if the config should inline value maps.
+        """
+        return self._config.get('inline_value_maps', DEFAULT_WEAVER_CONFIG['inline_value_maps'])
+
+    def _cfg_should_inline_entity_maps(self):
+        """
+        Check if the config should inline entity maps.
+        """
+        return self._config.get('inline_entity_maps', DEFAULT_WEAVER_CONFIG['inline_entity_maps'])
