@@ -13,7 +13,8 @@ from relationalai.early_access.dsl.core.utils import generate_stable_uuid, to_pa
 from relationalai.early_access.dsl.orm.constraints import Unique, Mandatory, RoleValueConstraint, Range, \
     ExclusiveSubtypeConstraint, InclusiveSubtypeConstraint, InclusiveRoleConstraint, ExclusiveRoleConstraint, \
     RingConstraint, ValueComparisonConstraint, RoleSubsetConstraint, EqualityConstraint, FrequencyConstraint, \
-    CardinalityConstraint
+    CardinalityConstraint, RoleCardinalityConstraint, ValueConstraint
+from relationalai.early_access.dsl.orm.reasoners import OntologyReasoner
 from relationalai.early_access.dsl.orm.relationships import Relationship, Role, RelationshipReading
 from relationalai.early_access.dsl.orm.types import Concept
 from relationalai.early_access.dsl.snow.api import Executor
@@ -41,6 +42,7 @@ class Model:
         self._relationship_iuc: dict[qb.Relationship, list[Unique]] = {}
         self._bindings = []
         self._executor = None
+        self._reasoner = OntologyReasoner()
         self._weaver = None
         self._queries = OrderedSet()
         self.Enum = self._qb_model.Enum
@@ -54,6 +56,9 @@ class Model:
 
     def qb_model(self):
         return self._qb_model
+    
+    def reasoner(self):
+        return self._reasoner
 
     def Relationship(self, reading: Any, short_name:str="") -> qb.Relationship:
         """
@@ -64,6 +69,7 @@ class Model:
     def constraint(self, constraint):
         # Generic constraint addition with validation
         self._constraints.add(constraint)
+        self._reasoner.new_constraint(constraint)
         constraint._desugar()
 
     def unique(self, *roles):
@@ -120,7 +126,14 @@ class Model:
     def cardinality(self, concept: Concept, values: Sequence[Union[int, Range[int]]]):
         self.constraint(CardinalityConstraint(concept, values))
 
+    def role_cardinality(self, role: Role, values: Sequence[Union[int, Range[int]]]):
+        self.constraint(RoleCardinalityConstraint(role, values))
+
+    def value_constraint(self, concept: Concept, values: Sequence[Union[T, Range[T]]]):
+        self.constraint(ValueConstraint(concept, values))
+
     def role_value_constraint(self, role: Role, values: Sequence[Union[T, Range[T]]]):
+        # TODO: check if a role value constraint is incompatible with an existing value constraint
         first_type = values[0]._type() if isinstance(values[0], Range) else type(values[0])
         for v in values[1:]:
             t = v._type() if isinstance(v, Range) else type(v)
@@ -238,9 +251,9 @@ class Model:
     def binding(self, binding: Binding):
         self._bindings.append(binding)
 
-    def generate_model_rules(self):
+    def generate_model_rules(self, config: Optional[dict]=None):
         if self._weaver is None:
-            self._weaver = Weaver(self)
+            self._weaver = Weaver(self, config)
         else:
             raise Exception("Model rules have already been generated.")
         self._weaver.generate()
