@@ -4,7 +4,7 @@ from relationalai.util.graph import topological_sort
 from relationalai.early_access.dsl.orm.constraints import Unique, Mandatory, RoleValueConstraint, \
     InclusiveSubtypeConstraint, ExclusiveSubtypeConstraint, Range, RingConstraint, InclusiveRoleConstraint, \
     ExclusiveRoleConstraint, ValueComparisonConstraint, RoleSubsetConstraint, EqualityConstraint, FrequencyConstraint, \
-    CardinalityConstraint
+    CardinalityConstraint, RoleCardinalityConstraint, ValueConstraint
 from relationalai.early_access.dsl.orm.models import Model
 from relationalai.early_access.dsl.orm.relationships import Role
 from relationalai.early_access.dsl.orm.types import Concept
@@ -27,6 +27,8 @@ class Printer(BasePrinter):
     _equality_constraints: OrderedSet[EqualityConstraint]
     _frequency_constraints: OrderedSet[FrequencyConstraint]
     _cardinality_constraints: OrderedSet[CardinalityConstraint]
+    _role_cardinality_constraints: OrderedSet[RoleCardinalityConstraint]
+    _value_constraints: OrderedSet[ValueConstraint]
 
     def __init__(self, io: typing.Optional[typing.IO[str]] = None):
         # Set the base class field (frozen)
@@ -46,6 +48,9 @@ class Printer(BasePrinter):
         object.__setattr__(self, '_equality_constraints', OrderedSet())
         object.__setattr__(self, '_frequency_constraints', OrderedSet())
         object.__setattr__(self, '_cardinality_constraints', OrderedSet())
+        object.__setattr__(self, '_role_cardinality_constraints', OrderedSet())
+        object.__setattr__(self, '_value_constraints', OrderedSet())
+
 
     def to_string(self, model: Model, enums_enabled: bool = False, pyi_enabled: bool = False) -> None:
         self._process_constraints(model)
@@ -83,6 +88,8 @@ class Printer(BasePrinter):
         self._print_nl("EqualityConstraint = model.equality")
         self._print_nl("FrequencyConstraint = model.frequency")
         self._print_nl("CardinalityConstraint = model.cardinality")
+        self._print_nl("RoleCardinalityConstraint = model.role_cardinality")
+        self._print_nl("ValueConstraint = model.value_constraint")
 
         self._nl()
         self._handle_concepts(model, enums_enabled, pyi_enabled)
@@ -141,6 +148,14 @@ class Printer(BasePrinter):
             self._nl()
             self._handle_cardinality_constraints()
 
+        if self._role_cardinality_constraints:
+            self._nl()
+            self._handle_role_cardinality_constraints()
+
+        if self._value_constraints:
+            self._nl()
+            self._handle_value_constraints()
+
     def _handle_concepts(self, model: Model, enums_enabled: bool, pyi_enabled: bool) -> None:
         concepts_map = model.concepts_map()
         enums_map = model.enums_map()
@@ -278,6 +293,16 @@ class Printer(BasePrinter):
             elements = self._get_constraint_values(constraint)
             self._print_nl(f"CardinalityConstraint({constraint.concept()._name}, [{', '.join(elements)}])")
 
+    def _handle_role_cardinality_constraints(self):
+        for constraint in self._role_cardinality_constraints:
+            elements = self._get_constraint_values(constraint)
+            self._print_nl(f"RoleCardinalityConstraint({self._get_role_name(constraint.roles()[0])}, [{', '.join(elements)}])")
+
+    def _handle_value_constraints(self):
+        for constraint in self._value_constraints:
+            elements = self._get_constraint_values(constraint)
+            self._print_nl(f"ValueConstraint({constraint.concept()._name}, [{', '.join(elements)}])")
+
     def _emit_role_sequence_constraints(self, constraints, constraint_type: str):
         for constraint in constraints:
             role_sequences = [[self._get_role_name(r) for r in ro_list] for ro_list in constraint.role_sequences()]
@@ -319,6 +344,10 @@ class Printer(BasePrinter):
                 self._frequency_constraints.add(c)
             elif isinstance(c, CardinalityConstraint):
                 self._cardinality_constraints.add(c)
+            elif isinstance(c, RoleCardinalityConstraint):
+                self._role_cardinality_constraints.add(c)
+            elif isinstance(c, ValueConstraint):
+                self._value_constraints.add(c)
 
     def _get_role_name(self, role: Role) -> str:
         return f"{self._get_relationship_name(role._relationship)}['{role._field_ref._name}']"
@@ -430,11 +459,11 @@ class InterfacePrinter(BasePrinter):
 
 
 # utility methods
-def _is_builtin_type(t: Concept) -> bool:
+def _is_builtin_type(t: qb.Concept) -> bool:
     return _get_concept_name(t) in Concept.builtins
 
 
-def _get_concept_name(t: Concept) -> str:
+def _get_concept_name(t: qb.Concept) -> str:
     return 'Integer' if t._name == 'Int' else t._name
 
 
