@@ -94,6 +94,9 @@ class Role(RelationshipFieldRef):
             sibling = self._relationship[1] if self._id == first_role._id else first_role
             self._sibling = sibling
         return self._sibling
+    
+    def siblings(self):
+        return [self._relationship[i] for i in range(self._relationship._arity()) if i != self._field_ix]
 
     def _part_of(self):
         return self._relationship
