@@ -24,10 +24,12 @@ class Concept(qb.Concept):
                     identify_args[k] = v
                 else:
                     raise ValueError(f"identify_by must be either a Concept or Relationship: {k}={v}")
-            # add constraints required for reference schema
-            self._dsl_model._ref_scheme_constraints(*identify_args.values())
         super().__init__(name, extends, model.qb_model(), identify_args)
         self._dsl_model._add_concept(self)
+        self._dsl_model.reasoner().new_concept(self)
+        if identify_by:
+            # once concept is added, add constraints required for reference schema
+            self._dsl_model._ref_scheme_constraints(*identify_args.values())
 
     def identify_by(self, *relations:Union[qb.Relationship, qb.RelationshipReading]):
         super().identify_by(*relations)
