@@ -41,7 +41,6 @@ class LQPExecutor(e.Executor):
         self.config = config or Config()
         self._resources = None
         self._last_model = None
-        self._error_ids = []
         self._last_sources_version = (-1, None)
 
     @property
@@ -52,7 +51,7 @@ class LQPExecutor(e.Executor):
                 resource_class = rai.clients.snowflake.Resources
                 if self.config.get("use_direct_access", USE_DIRECT_ACCESS):
                     resource_class = rai.clients.snowflake.DirectAccessResources
-                self._resources = resource_class(dry_run=self.dry_run, config=self.config)
+                self._resources = resource_class(dry_run=self.dry_run, config=self.config, generation=rai.Generation.QB)
                 if not self.dry_run:
                     self.engine = self._resources.get_default_engine_name()
                     if not self.keep_model:
@@ -230,22 +229,20 @@ class LQPExecutor(e.Executor):
         model_txn = None
         if self._last_model != model:
             with debugging.span("compile", metamodel=model) as install_span:
-                _, error_ids, model_txn = self.compiler.compile(model, {"fragment_id": b"model"})
+                _, model_txn = self.compiler.compile(model, {"fragment_id": b"model"})
                 install_span["compile_type"] = "model"
                 install_span["lqp"] = lqp_print.to_string(model_txn)
                 install_span["lqp_debug"] = lqp_print.to_string(model_txn, {"print_names": True, "print_debug": False})
                 self._last_model = model
-                self._error_ids = error_ids
 
         with debugging.span("compile", metamodel=task) as compile_span:
             query = f.compute_model(f.logical([task]))
             options = {
                 "wide_outputs": self.wide_outputs,
                 "fragment_id": b"query",
-                "error_attr_ids": self._error_ids,
             }
             result, final_model = self.compiler.compile_inner(query, options)
-            export_info, _, query_txn = result
+            export_info, query_txn = result
             compile_span["compile_type"] = "query"
             compile_span["lqp"] = lqp_print.to_string(query_txn)
             compile_span["lqp_debug"] = lqp_print.to_string(query_txn, {"print_names": True, "print_debug": False})
