@@ -20,9 +20,12 @@ __all__ = [
     "RelAtom",
     "Cast",
     "Var",
-    "Int128",
-    "UInt128",
-    "Constant",
+    "Int128Value",
+    "UInt128Value",
+    "DecimalValue",
+    "DateValue",
+    "DateTimeValue",
+    "BooleanValue",
     "Term",
     "Value",
     "SpecializedValue",
@@ -68,9 +71,12 @@ from lqp.ir import (
     RelAtom,
     Cast,
     Var,
-    Int128,
-    UInt128,
-    Constant,
+    Int128Value,
+    UInt128Value,
+    DecimalValue,
+    DateValue,
+    DateTimeValue,
+    BooleanValue,
     Term,
     Value,
     SpecializedValue,
