@@ -2,34 +2,30 @@ from relationalai.early_access.metamodel import ir, builtins as rel_builtins, he
 from relationalai.early_access.metamodel.visitor import collect_by_type
 from relationalai.early_access.lqp import ir as lqp, utils, types as lqp_types
 from relationalai.early_access.lqp.primitives import (
-    relname_to_lqp_name, lqp_operator, lqp_avg_op, is_monotype
+    relname_to_lqp_name, lqp_operator, lqp_avg_op, is_monotype, primitive_types
 )
 from relationalai.early_access.lqp.types import meta_type_to_lqp, type_from_constant
 from relationalai.early_access.lqp.constructors import (
-    mk_and, mk_exists, mk_or, mk_abstraction, mk_value, mk_var, mk_type
+    mk_abstraction, mk_and, mk_exists, mk_or, mk_primitive, mk_type, mk_value, mk_var
 )
 from relationalai.early_access.lqp.utils import TranslationCtx, gen_unique_var
 from relationalai.early_access.lqp.validators import assert_valid_input
 
+from decimal import Decimal as PyDecimal
+from datetime import datetime, date
 from typing import Tuple, cast, Union, Optional
 import re
 
 """ Main access point. Converts the model IR to an LQP transaction. """
-def to_lqp(model: ir.Model, fragment_name: bytes, error_attr_ids: Optional[list]) -> tuple[Optional[tuple], Optional[list], lqp.Transaction]:
+def to_lqp(model: ir.Model, fragment_name: bytes) -> tuple[Optional[tuple], lqp.Transaction]:
     assert_valid_input(model)
     ctx = TranslationCtx(model)
     decls: list[lqp.Declaration] = []
+    reads: list[lqp.Read] = []
 
-    # Extract new pyrel error attributes, if any.
-    # These can be used in a subsequent query to report errors.
-    # TODO (RAI-40312): Revisit this after reworking compiler/executor interface.
-    new_error_ids = _extract_pyrel_error_ids(ctx, model)
-    if error_attr_ids is not None:
-        # If we have error attributes, we need to add them to the context.
-        # This is used to generate the error attributes in the LQP.
-        for (rel_id, name) in error_attr_ids:
-            ctx.output_ids.append((rel_id, name))
-
+    # Add pyrel error attributes to reads.
+    for err_id in _extract_pyrel_error_ids(ctx, model):
+        ctx.output_ids.append(err_id)
 
     # LQP only accepts logical tasks
     # These are asserted at init time
@@ -39,7 +35,6 @@ def to_lqp(model: ir.Model, fragment_name: bytes, error_attr_ids: Optional[list]
         new_decls = _translate_to_decls(ctx, subtask)
         decls.extend(new_decls)
 
-    reads: list[lqp.Read] = []
     reads.extend(_get_output_reads(ctx.output_ids))
 
     export_info = None
@@ -65,7 +60,7 @@ def to_lqp(model: ir.Model, fragment_name: bytes, error_attr_ids: Optional[list]
     )
 
     lqp.validate_lqp(txn)
-    return (export_info, new_error_ids, txn)
+    return (export_info, txn)
 
 def _effect_bindings(effect: Union[ir.Output, ir.Update]) -> list[ir.Value]:
     if isinstance(effect, ir.Output):
@@ -225,16 +220,29 @@ def _translate_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.Formula) -> lq
     introduced_meta_projs = [arg for arg in rank.projection if arg not in rank.group and arg not in rank.args]
     projected_args, projected_eqs = _translate_bindings(ctx, list(introduced_meta_projs))
 
+    # rank expects an Int128 result, but the primitive will return an Int64 result.
+    # we need to set up an intermediary variable to hold the Int64 result, and a cast
+    # to convert it to Int128.
+    result_var, _ = _translate_term(ctx, rank.result)
+    # The primitive will return an Int64 result, so we need a var to hold the intermediary.
+    result_64_var = gen_unique_var(ctx, "result_64")
+    result_64_type = mk_type(lqp.TypeName.INT)
+
+    cast = lqp.Cast(input=result_64_var, result=result_var, meta=None)
+
     body = mk_and([body] + input_eqs + projected_eqs)
     abstr_args = input_args + projected_args
 
     if ascending:
-        return _translate_ascending_rank(ctx, rank, body, abstr_args)
+        ranker = _translate_ascending_rank(ctx, rank.limit, result_64_var, body, abstr_args)
     else:
-        return _translate_descending_rank(ctx, rank, body, abstr_args)
+        ranker = _translate_descending_rank(ctx, rank.limit, result_64_var, body, abstr_args)
 
-def _translate_descending_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.Formula, abstr_args) -> lqp.Formula:
-    result_var, result_type = _translate_term(ctx, rank.result)
+    return mk_exists([(result_64_var, result_64_type)], mk_and([ranker, cast]))
+
+def _translate_descending_rank(ctx: TranslationCtx, limit: int, result: lqp.Var, body: lqp.Formula, abstr_args) -> lqp.Formula:
+    result_var = result
+    result_type = mk_type(lqp.TypeName.INT)
 
     # Rename abstracted args in the body to new variable names
     var_map = {var.name: gen_unique_var(ctx, var.name) for (var, _) in abstr_args}
@@ -246,21 +254,13 @@ def _translate_descending_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.For
     count_res = gen_unique_var(ctx, "count_res")
 
     # Add one to the count to account for the rank starting at 1.
-    one, _, one_eq = constant_to_lqp_var(ctx, 1, types.Int, "one")
+    one, _, one_eq = constant_to_lqp_var(ctx, 1, types.Int64, "one")
     one_bigger = gen_unique_var(ctx, "one_bigger")
-    addition = lqp.Primitive(
-        name="rel_primitive_add",
-        terms=[count_res, one, one_bigger],
-        meta=None
-    )
+    addition = mk_primitive("rel_primitive_add", [count_res, one, one_bigger])
 
     # Subtract the rank from the count + 1
     asc_rank = gen_unique_var(ctx, "asc_rank")
-    subtraction = lqp.Primitive(
-        name="rel_primitive_subtract",
-        terms=[one_bigger, result_var, asc_rank],
-        meta=None
-    )
+    subtraction = mk_primitive("rel_primitive_subtract", [one_bigger, result_var, asc_rank])
 
     # Construct the ranking
     desc_ranking_terms = [asc_rank] + [v[0] for v in abstr_args]
@@ -272,7 +272,7 @@ def _translate_descending_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.For
     )
 
     # Count the number of rows in the body
-    count_var, count_type, count_eq = constant_to_lqp_var(ctx, 1, types.Int, "counter")
+    count_var, count_type, count_eq = constant_to_lqp_var(ctx, 1, types.Int64, "counter")
     desc_body = mk_and([body, count_eq])
     aggr_abstr_args = new_abstr_args + [(count_var, count_type)]
     count_aggr = lqp.Reduce(
@@ -300,19 +300,14 @@ def _translate_descending_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.For
 
     # If there is a limit, we need to add a filter to the ranking.
     # Wrap with a rank <= limit
-    if rank.limit != 0:
-        limit_term, _ = _translate_term(ctx, rank.limit)
-        limiter = lqp.Primitive(
-            name="rel_primitive_lt_eq",
-            terms=[result_var, limit_term],
-            meta=None
-        )
+    if limit != 0:
+        limit_term, _ = _translate_term(ctx, ir.Literal(types.Int128, limit))
+        limiter = mk_primitive("rel_primitive_lt_eq", [result_var, limit_term])
         ranking = mk_and([ranking, limiter])
 
     return ranking
 
-def _translate_ascending_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.Formula, abstr_args) -> lqp.Formula:
-    result_var, _ = _translate_term(ctx, rank.result)
+def _translate_ascending_rank(ctx: TranslationCtx, limit: int, result_var: lqp.Var, body: lqp.Formula, abstr_args) -> lqp.Formula:
     terms = [result_var] + [v[0] for v in abstr_args]
 
     # Rename abstracted args in the body to new variable names
@@ -321,7 +316,7 @@ def _translate_ascending_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.Form
     new_abstr_args = [(var_map[var.name], typ) for (var, typ) in abstr_args]
     sort_abstr = mk_abstraction(new_abstr_args, body)
 
-    if rank.limit == 0:
+    if limit == 0:
         return lqp.FFI(
             meta=None,
             name="rel_primitive_sort",
@@ -329,7 +324,7 @@ def _translate_ascending_rank(ctx: TranslationCtx, rank: ir.Rank, body: lqp.Form
             terms=terms,
         )
     else:
-        limit_var, limit_type, limit_eq = constant_to_lqp_var(ctx, rank.limit, types.Int, "limit")
+        limit_var, limit_type, limit_eq = constant_to_lqp_var(ctx, limit, types.Int64, "limit")
         limit_abstr = mk_abstraction([(limit_var, limit_type)], limit_eq)
         return lqp.FFI(
             meta=None,
@@ -353,7 +348,8 @@ def _translate_aggregate(ctx: TranslationCtx, aggr: ir.Aggregate, body: lqp.Form
         else:
             meta_output_terms.append(arg)
 
-    output_vars = [_translate_term(ctx, term)[0] for term in meta_output_terms]
+    output_terms = [_translate_term(ctx, term) for term in meta_output_terms]
+    output_vars = [term[0] for term in output_terms]
 
     body_conjs = [body]
     input_args, input_eqs = _translate_bindings(ctx, meta_input_terms)
@@ -366,21 +362,28 @@ def _translate_aggregate(ctx: TranslationCtx, aggr: ir.Aggregate, body: lqp.Form
     body_conjs.extend(projected_eqs)
     abstr_args: list[Tuple[lqp.Var, lqp.Type]] = projected_args + input_args
 
-    if aggr_name == "count" or aggr_name == "avg":
-        assert len(output_vars) == 1, "Count and avg expect a single output variable"
-
-        # Count sums up "1"
-        one_var, typ, eq = constant_to_lqp_var(ctx, 1, types.Int, "one")
+    if aggr_name == "count":
+        assert len(output_terms) == 1, "Count and avg expect a single output variable"
+        assert isinstance(meta_output_terms[0], ir.Var)
+        # Count sums up "1" for each row. We use the expected output type for the type
+        # of the count variable.
+        one_var, typ, eq = constant_to_lqp_var(ctx, 1, meta_output_terms[0].type, "one")
         body_conjs.append(eq)
         abstr_args.append((one_var, typ))
-
-    body = mk_and(body_conjs)
+        body = mk_and(body_conjs)
 
     # Average needs to wrap the reduce in Exists(Conjunction(Reduce, div))
     if aggr_name == "avg":
         assert len(output_vars) == 1, "avg should only have one output variable"
         output_var = output_vars[0]
 
+        # Count sums up "1" for each row. We make the reasonably safe assumption that there
+        # are less than 2^31 rows in the body.
+        one_var, count_type, eq = constant_to_lqp_var(ctx, 1, types.Int64, "one")
+        body_conjs.append(eq)
+        abstr_args.append((one_var, count_type))
+        body = mk_and(body_conjs)
+
         # The average will produce two output variables: sum and count.
         sum_result = gen_unique_var(ctx, "sum")
         count_result = gen_unique_var(ctx, "count")
@@ -395,20 +398,13 @@ def _translate_aggregate(ctx: TranslationCtx, aggr: ir.Aggregate, body: lqp.Form
             meta=None,
         )
 
-        div = lqp.Primitive(name="rel_primitive_divide", terms=[sum_result, count_result, output_var], meta=None)
+        div = mk_primitive("rel_primitive_divide", [sum_result, count_result, output_var])
         conjunction = mk_and([result, div])
 
         # Finally, we need to wrap everything in an `exists` to project away the sum and
         # count variables and only keep the result of the division.
-        result = mk_exists(
-            [
-                (sum_result, sum_type),
-                (count_result, mk_type(lqp.TypeName.INT))
-            ],
-            conjunction,
-        )
-
-        return result
+        result_terms = [(sum_result, sum_type), (count_result, count_type)]
+        return mk_exists(result_terms, conjunction)
 
     # `input_args`` hold the types of the input arguments, but they may have been modified
     # if we're dealing with a count, so we use `abstr_args` to find the type.
@@ -438,15 +434,11 @@ def _translate_to_formula(ctx: TranslationCtx, task: ir.Task) -> lqp.Formula:
         assert len(task.values) >= 1, "Construct should have at least one value"
         assert isinstance(task.values[0], ir.ScalarType), "Construct should start with a named ScalarType"
         name = task.values[0].name
-        terms = [_translate_term(ctx, name)]
+        terms = [_translate_term(ctx, ir.Literal(types.String, name))]
         terms.extend([_translate_term(ctx, arg) for arg in task.values[1:]])
         terms.append(_translate_term(ctx, task.id_var))
 
-        return lqp.Primitive(
-            name="rel_primitive_hash_tuple_uint128",
-            terms=[v for v, _ in terms],
-            meta=None
-        )
+        return mk_primitive("rel_primitive_hash_tuple_uint128", [v for v, _ in terms])
     elif isinstance(task, ir.Union):
         # TODO: handle hoisted vars if needed
         disjs = [_translate_to_formula(ctx, child) for child in task.tasks]
@@ -460,15 +452,15 @@ def _translate_to_formula(ctx: TranslationCtx, task: ir.Task) -> lqp.Formula:
     else:
         raise NotImplementedError(f"Unknown task type (formula): {type(task)}")
 
-# Only used for translating terms on atoms, which can be specialized values.
+# Only used for translating terms on relatoms, which can be specialized values.
 def _translate_relterm(ctx: TranslationCtx, term: ir.Value) -> Tuple[lqp.RelTerm, lqp.Type]:
     if isinstance(term, ir.Literal) and term.type == types.Symbol:
         if isinstance(term.value, str):
-            value = lqp.Value(value=term.value, cast_type=None, meta=None)
+            value = mk_value(term.value)
             return lqp.SpecializedValue(value=value, meta=None), meta_type_to_lqp(types.String)
         elif isinstance(term.value, int):
-            value = lqp.Value(value=term.value, cast_type=None, meta=None)
-            return lqp.SpecializedValue(value=value, meta=None), meta_type_to_lqp(types.Int)
+            value = mk_value(term.value)
+            return lqp.SpecializedValue(value=value, meta=None), meta_type_to_lqp(types.Int64)
         else:
             raise NotImplementedError(f"Cannot specialize literal of type {type(term.value)}")
     return _translate_term(ctx, term)
@@ -479,14 +471,12 @@ def _translate_term(ctx: TranslationCtx, term: ir.Value) -> Tuple[lqp.Term, lqp.
         t = meta_type_to_lqp(term.type)
         return mk_var(name), t
     elif isinstance(term, ir.Literal):
-        assert isinstance(term.value, lqp_types.PrimitiveValue), \
-            f"expected primitive value, got {type(term.value)}: {term.value}"
-        value = lqp.Value(value=term.value, cast_type=None, meta=None)
-        return value, meta_type_to_lqp(term.type)
+        v = _ir_constant_to_lqp_value(term.value, term.type)
+        return v, meta_type_to_lqp(term.type)
     else:
-        assert isinstance(term, lqp_types.PrimitiveValue), \
+        assert isinstance(term, ir.PyValue), \
             f"Cannot translate value {term!r} of type {type(term)} to LQP Term; not a PrimitiveValue."
-        value = lqp.Value(value=term, cast_type=None, meta=None)
+        value = _ir_constant_to_lqp_value(term, None)
         return value, type_from_constant(term)
 
 def _translate_to_atom(ctx: TranslationCtx, task: ir.Lookup) -> lqp.Formula:
@@ -526,25 +516,25 @@ def _translate_to_atom(ctx: TranslationCtx, task: ir.Lookup) -> lqp.Formula:
                 mk_value("UTC"),
                 terms[-1],
             ]
-            return lqp.Primitive(name=lqp_name, terms=extended_terms, meta=None)
+            return mk_primitive(lqp_name, extended_terms)
         elif task.relation.name == "like_match":
             # In the like_match API of QB, the variable is the first argument and the pattern the second,
             #   but in Rel, the expected argument order is reversed: pattern first, then variable.
             lqp_name = relname_to_lqp_name(task.relation.name)
-            return lqp.Primitive(name=lqp_name, terms=terms[::-1], meta=None)
+            return mk_primitive(lqp_name, terms[::-1])
         elif task.relation.name == "parse_decimal64" and len(terms) == 2:
             # If the builtin for parsing decimals is called directly we will need to add
             # the default parameters of 64 bits and precision 6.
             lqp_name = relname_to_lqp_name(task.relation.name)
-            bit_value = lqp.Value(value=64, cast_type=None, meta=None)
-            precision_value = lqp.Value(value=6, cast_type=None, meta=None)
+            bit_value = mk_value(64)
+            precision_value = mk_value(6)
             extended_terms = [
                 lqp.SpecializedValue(value=bit_value, meta=None),
                 lqp.SpecializedValue(value=precision_value, meta=None),
                 terms[0],
                 terms[1],
             ]
-            return lqp.Primitive(name=lqp_name, terms=extended_terms, meta=None)
+            return mk_primitive(lqp_name, extended_terms)
         elif (
             task.relation.name == "parse_decimal" or
             task.relation.name == "parse_decimal128"
@@ -552,15 +542,21 @@ def _translate_to_atom(ctx: TranslationCtx, task: ir.Lookup) -> lqp.Formula:
             # If the builtin for parsing decimals is called directly we will need to add
             # the default parameters of 128 bits and precision 10.
             lqp_name = relname_to_lqp_name(task.relation.name)
-            bit_value = lqp.Value(value=128, cast_type=None, meta=None)
-            precision_value = lqp.Value(value=10, cast_type=None, meta=None)
+            bit_value = mk_value(128)
+            precision_value = mk_value(10)
             extended_terms = [
                 lqp.SpecializedValue(value=bit_value, meta=None),
                 lqp.SpecializedValue(value=precision_value, meta=None),
                 terms[0],
                 terms[1],
             ]
-            return lqp.Primitive(name=lqp_name, terms=extended_terms, meta=None)
+            return mk_primitive(lqp_name, extended_terms)
+        elif task.relation.name == "date_year":
+            return _cast_primitive(ctx, task.relation.name, terms, term_types)
+        elif task.relation.name == "date_month":
+            return _cast_primitive(ctx, task.relation.name, terms, term_types)
+        elif task.relation.name == "date_day":
+            return _cast_primitive(ctx, task.relation.name, terms, term_types)
         else:
             lqp_name = relname_to_lqp_name(task.relation.name)
             if is_monotype(task.relation.name):
@@ -569,7 +565,7 @@ def _translate_to_atom(ctx: TranslationCtx, task: ir.Lookup) -> lqp.Formula:
                     f"Expected all terms to have the same type for monotype operator " \
                     f"`{task.relation.name}` but got {term_types} for terms {terms}"
 
-            return lqp.Primitive(name=lqp_name, terms=terms, meta=None)
+            return mk_primitive(lqp_name, terms)
 
     if helpers.is_external(task.relation):
         return lqp.RelAtom(name=task.relation.name, terms=terms, meta=None)
@@ -577,6 +573,29 @@ def _translate_to_atom(ctx: TranslationCtx, task: ir.Lookup) -> lqp.Formula:
     rid = get_relation_id(ctx, rel_name, task.relation.id)
     return lqp.Atom(name=rid, terms=terms, meta=None)
 
+def _cast_primitive(
+        ctx: TranslationCtx,
+        name: str,
+        terms: list[lqp.RelTerm],
+        term_types: list[lqp.Type]
+    ) -> lqp.Formula:
+    expected = primitive_types.get(name, None)
+    if expected is None:
+        return mk_primitive(relname_to_lqp_name(name), terms)
+    elif expected[-1] == term_types[-1].type_name:
+        return mk_primitive(relname_to_lqp_name(name), terms)
+    # TODO: check the rest of the terms
+    conversion_var = gen_unique_var(ctx, f"result_{expected[-1]}")
+    result = terms[-1]
+    assert isinstance(result, lqp.Var), f"Expected last term to be a Var, got {type(result)}: {result}"
+    terms = [*terms[:-1], conversion_var]
+
+    # Create a cast operation to convert the body to the result type
+    cast = lqp.Cast(input=conversion_var, result=result, meta=None)
+    primitive = mk_primitive(relname_to_lqp_name(name), terms)
+    body = mk_and([primitive, cast])
+    return mk_exists([(conversion_var, mk_type(expected[-1]))], body)
+
 def get_relation_id(ctx: TranslationCtx, orig_name: str, metamodel_id: int) -> lqp.RelationId:
     mid_str = str(metamodel_id)
     relation_id = lqp.RelationId(id=utils.lqp_hash(mid_str), meta=None)
@@ -602,23 +621,62 @@ def binding_to_lqp_var(ctx: TranslationCtx, binding: ir.Value) -> Tuple[lqp.Var,
         return var, typ, None
     elif isinstance(binding, ir.Literal):
         return constant_to_lqp_var(ctx, binding.value, binding.type)
-    elif isinstance(binding, lqp_types.PrimitiveValue):
+    elif isinstance(binding, ir.PyValue):
         return constant_to_lqp_var(ctx, binding, None)
     else:
         raise Exception(f"Unsupported binding type: {type(binding)}")
 
-def constant_to_lqp_var(
-    ctx: TranslationCtx,
-    value: lqp_types.PrimitiveValue,
-    type: Optional[ir.Type], name_hint: str = "cvar"
-) -> Tuple[lqp.Var, lqp.Type, lqp.Formula]:
-    var = gen_unique_var(ctx, name_hint)
-    if type is not None:
-        typ = meta_type_to_lqp(type)
+def _ir_constant_to_lqp_value(value: ir.PyValue, value_type: Optional[ir.Type]) -> lqp.Value:
+    if value_type:
+        typ = meta_type_to_lqp(value_type)
     else:
         typ = type_from_constant(value)
-    value_term = lqp.Value(value=value, cast_type=None, meta=None)
-    eq = lqp.Primitive(name="rel_primitive_eq", terms=[var, value_term], meta=None)
+
+    # Ensure int values match the requested integer type.
+    if typ.type_name == lqp.TypeName.INT and isinstance(value, int):
+        assert lqp_types.INT_MIN <= value <= lqp_types.INT_MAX, f"{value} out of range for a 64-bit INT value"
+        val = value
+    elif typ.type_name == lqp.TypeName.INT128 and isinstance(value, int):
+        assert lqp_types.INT128_MIN <= value <= lqp_types.INT128_MAX, f"{value} out of range for an INT128 value"
+        val = lqp.Int128Value(value=value, meta=None)
+    elif typ.type_name == lqp.TypeName.UINT128 and isinstance(value, int):
+        assert lqp_types.UINT128_MIN <= value <= lqp_types.UINT128_MAX, f"{value} out of range for an UINT128 value"
+        val = lqp.UInt128Value(value=value, meta=None)
+    elif typ.type_name == lqp.TypeName.FLOAT and isinstance(value, float):
+        val = value
+    elif typ.type_name == lqp.TypeName.STRING and isinstance(value, str):
+        val = value
+    elif typ.type_name == lqp.TypeName.DECIMAL and isinstance(value, PyDecimal):
+        precision = typ.parameters[0].value
+        scale = typ.parameters[1].value
+        assert isinstance(precision, int) and isinstance(scale, int)
+        val = lqp.DecimalValue(precision=precision, scale=scale, value=value, meta=None)
+    elif typ.type_name == lqp.TypeName.DATE and isinstance(value, date):
+        val = lqp.DateValue(value=value, meta=None)
+    elif typ.type_name == lqp.TypeName.DATETIME and isinstance(value, datetime):
+        val = lqp.DateTimeValue(value=value, meta=None)
+    elif typ.type_name == lqp.TypeName.BOOLEAN and isinstance(value, bool):
+        val = lqp.BooleanValue(value=value, meta=None)
+    else:
+        raise Exception(f"Unsupported type for constant: {typ.type_name} with value {value} of type {type(value)}")
+
+    return mk_value(val)
+
+def constant_to_lqp_var(
+    ctx: TranslationCtx,
+    value: ir.PyValue,
+    value_type: Optional[ir.Type],
+    name_hint: str = "cvar"
+) -> Tuple[lqp.Var, lqp.Type, lqp.Formula]:
+    var = gen_unique_var(ctx, name_hint)
+
+    if value_type:
+        typ = meta_type_to_lqp(value_type)
+    else:
+        typ = type_from_constant(value)
+
+    value_term = _ir_constant_to_lqp_value(value, value_type)
+    eq = mk_primitive("rel_primitive_eq", [var, value_term])
 
     return var, typ, eq
 
