@@ -1,12 +1,14 @@
 from relationalai.early_access.lqp import ir as lqp
 from relationalai.early_access.lqp.utils import UniqueNames
-from relationalai.early_access.lqp.constructors import mk_type
+from relationalai.early_access.lqp.constructors import mk_primitive, mk_type, mk_var
 
 rel_to_lqp = {
     "=": "rel_primitive_eq",
     "!=": "rel_primitive_neq",
     "%": "rel_primitive_remainder",
     "abs": "rel_primitive_abs",
+    "ceil": "rel_primitive_round_up",
+    "floor": "rel_primitive_round_down",
     "construct_date": "rel_primitive_construct_date",
     "construct_datetime": "rel_primitive_construct_datetime",
     "construct_datetime_ms_tz": "rel_primitive_construct_datetime",
@@ -17,6 +19,7 @@ rel_to_lqp = {
     "parse_decimal": "rel_primitive_parse_decimal",
     "parse_decimal64": "rel_primitive_parse_decimal",
     "parse_decimal128": "rel_primitive_parse_decimal",
+    "string": "rel_primitive_string",
     "starts_with": "rel_primitive_starts_with",
     "ends_with": "rel_primitive_ends_with",
     "contains": "rel_primitive_contains",
@@ -27,17 +30,33 @@ rel_to_lqp = {
     "upper": "rel_primitive_uppercase",
     "concat": "rel_primitive_concat",
     "replace": "rel_primitive_replace",
+    "strip": "rel_primitive_trim",
     "date_year": "rel_primitive_date_year",
     "date_month": "rel_primitive_date_month",
     "date_day": "rel_primitive_date_day",
+    "date_add": "rel_primitive_typed_add_date_period",
+    "date_subtract": "rel_primitive_typed_subtract_date_period",
+    "datetime_add": "rel_primitive_typed_add_datetime_period",
+    "datetime_subtract": "rel_primitive_typed_subtract_datetime_period",
+    "dates_period_days": "rel_primitive_date_days_between",
+    "datetimes_period_milliseconds": "rel_primitive_datetime_milliseconds_between",
     "range": "rel_primitive_range",
     "natural_log": "rel_primitive_natural_log",
     "sqrt": "rel_primitive_sqrt",
+    "isinf": "rel_primitive_isinf",
+    "isnan": "rel_primitive_isnan",
     # Division is monotype, but only on the input args. Until we distinguish between input
     # and output args, we can't use the same assertions for monotype-ness as the other ops.
     "/": "rel_primitive_divide_monotype",
 }
 
+# Mappings of primitive names to their lqp types that represent the raicode expectations.
+primitive_types = {
+    "date_year": [lqp.TypeName.DATE, lqp.TypeName.INT],
+    "date_month": [lqp.TypeName.DATE, lqp.TypeName.INT],
+    "date_day": [lqp.TypeName.DATE, lqp.TypeName.INT],
+}
+
 agg_to_lqp = {
     "min": "rel_primitive_min",
     "max": "rel_primitive_max",
@@ -54,7 +73,9 @@ rel_to_lqp_monotype = {
     ">=": "rel_primitive_gt_eq_monotype",
     ">": "rel_primitive_gt_monotype",
     "<": "rel_primitive_lt_monotype",
+    "//": "rel_primitive_trunc_divide_monotype",
     "maximum": "rel_primitive_max",
+    "minimum": "rel_primitive_min",
 }
 
 def relname_to_lqp_name(name: str) -> str:
@@ -75,25 +96,25 @@ def is_monotype(name: str) -> bool:
 def lqp_avg_op(names: UniqueNames, op_name: str, sum_name: str, sum_type: lqp.Type) -> lqp.Abstraction:
     count_type = mk_type(lqp.TypeName.INT)
     vars = [
-        (lqp.Var(name=names.get_name(sum_name), meta=None), sum_type),
-        (lqp.Var(name=names.get_name("counter"), meta=None), count_type),
-        (lqp.Var(name=names.get_name(sum_name), meta=None), sum_type),
-        (lqp.Var(name=names.get_name("one"), meta=None), count_type),
-        (lqp.Var(name=names.get_name("sum"), meta=None), sum_type),
-        (lqp.Var(name=names.get_name("count"), meta=None), count_type),
+        (mk_var(names.get_name(sum_name)), sum_type),
+        (mk_var(names.get_name("counter")), count_type),
+        (mk_var(names.get_name(sum_name)), sum_type),
+        (mk_var(names.get_name("one")), count_type),
+        (mk_var(names.get_name("sum")), sum_type),
+        (mk_var(names.get_name("count")), count_type),
     ]
 
     x1 = vars[0][0]
     x2 = vars[1][0]
     y1 = vars[2][0]
     y2 = vars[3][0]
-    sum = vars[4][0]
+    sumv = vars[4][0]
     count = vars[5][0]
 
     body = lqp.Conjunction(
         args=[
-            lqp.Primitive(name="rel_primitive_add_monotype", terms=[x1, y1, sum], meta=None),
-            lqp.Primitive(name="rel_primitive_add_monotype", terms=[x2, y2, count], meta=None)
+            mk_primitive("rel_primitive_add_monotype", [x1, y1, sumv]),
+            mk_primitive("rel_primitive_add_monotype", [x2, y2, count])
         ],
         meta=None
     )
@@ -101,13 +122,13 @@ def lqp_avg_op(names: UniqueNames, op_name: str, sum_name: str, sum_type: lqp.Ty
 
 # Default handler for aggregation operations in LQP.
 def lqp_agg_op(names: UniqueNames, op_name: str, aggr_arg_name: str, aggr_arg_type: lqp.Type) -> lqp.Abstraction:
-    x = lqp.Var(name=names.get_name(f"x_{aggr_arg_name}"), meta=None)
-    y = lqp.Var(name=names.get_name(f"y_{aggr_arg_name}"), meta=None)
-    z = lqp.Var(name=names.get_name(f"z_{aggr_arg_name}"), meta=None)
+    x = mk_var(names.get_name(f"x_{aggr_arg_name}"))
+    y = mk_var(names.get_name(f"y_{aggr_arg_name}"))
+    z = mk_var(names.get_name(f"z_{aggr_arg_name}"))
     ts = [(x, aggr_arg_type), (y, aggr_arg_type), (z, aggr_arg_type)]
 
     name = agg_to_lqp.get(op_name, op_name)
-    body = lqp.Primitive(name=name, terms=[x, y, z], meta=None)
+    body = mk_primitive(name, [x, y, z])
 
     return lqp.Abstraction(vars=ts, value=body, meta=None)
 
