@@ -46,8 +46,8 @@ class TranslationCtx:
         self.def_names = UniqueNames()
         # Map relation IDs to their original names for debugging and pretty printing.
         self.rel_id_to_orig_name = {}
-        self.output_ids = []
-        self.export_ids = []
+        self.output_ids: list[tuple[lqp.RelationId, str]] = []
+        self.export_ids: list[tuple[lqp.RelationId, int, lqp.Type]] = []
 
 def gen_unique_var(ctx: TranslationCtx, name_hint: str):
     """
