@@ -16,7 +16,7 @@ from typing import Optional
 def _comparator(name: str, input=True):
     overloads = [
         f.relation(name, [f.field("a", type, input), f.field("b", type, input)])
-        for type in [types.Int, types.Float, types.Decimal64, types.Decimal128, types.String, types.Date, types.DateTime, types.Hash, types.EntityTypeVar]
+        for type in [types.Bool, types.Int64, types.Int128, types.Float, types.Decimal64, types.Decimal128, types.String, types.Date, types.DateTime, types.Hash, types.EntityTypeVar]
     ]
     return f.relation(name, [f.field("a", types.Any, input), f.field("b", types.Any, input)], overloads=overloads)
 
@@ -29,7 +29,7 @@ eq = _comparator("=", False)
 
 # Arithmetic operators
 def _binary_op(name: str, with_string=False, result_type: Optional[ir.Type]=None):
-    overload_types = [types.Int, types.Float, types.Decimal64, types.Decimal128]
+    overload_types = [types.Int64, types.Int128, types.Float, types.Decimal64, types.Decimal128]
     if with_string:
         overload_types.append(types.String)
     overloads = [
@@ -54,7 +54,8 @@ div = f.relation(
     "/",
     [f.input_field("a", types.Number), f.input_field("b", types.Number), f.field("c", types.Number)],
     overloads=[
-        f.relation("/", [f.input_field("a", types.Int), f.input_field("b", types.Int), f.field("c", types.Float)]),
+        f.relation("/", [f.input_field("a", types.Int64), f.input_field("b", types.Int64), f.field("c", types.Float)]),
+        f.relation("/", [f.input_field("a", types.Int128), f.input_field("b", types.Int128), f.field("c", types.Float)]),
         f.relation("/", [f.input_field("a", types.Float), f.input_field("b", types.Float), f.field("c", types.Float)]),
         f.relation("/", [f.input_field("a", types.Decimal64), f.input_field("b", types.Decimal64), f.field("c", types.Decimal64)]),
         f.relation("/", [f.input_field("a", types.Decimal128), f.input_field("b", types.Decimal128), f.field("c", types.Decimal128)]),
@@ -63,13 +64,21 @@ div = f.relation(
 mod = _binary_op("%")
 power = _binary_op("^")
 
-trunc_div = f.relation("//", [f.input_field("a", types.Int), f.input_field("b", types.Int), f.field("c", types.Int)])
+trunc_div = f.relation(
+    "//",
+    [f.input_field("a", types.Number), f.input_field("b", types.Number), f.field("c", types.Number)],
+    overloads=[
+        f.relation("//", [f.input_field("a", types.Int64), f.input_field("b", types.Int64), f.field("c", types.Int64)]),
+        f.relation("//", [f.input_field("a", types.Int128), f.input_field("b", types.Int128), f.field("c", types.Int128)]),
+    ],
+)
 
 abs = f.relation(
     "abs",
     [f.input_field("a", types.Number), f.field("b", types.Number)],
     overloads=[
-        f.relation("abs", [f.input_field("a", types.Int), f.field("b", types.Int)]),
+        f.relation("abs", [f.input_field("a", types.Int64), f.field("b", types.Int64)]),
+        f.relation("abs", [f.input_field("a", types.Int128), f.field("b", types.Int128)]),
         f.relation("abs", [f.input_field("a", types.Float), f.field("b", types.Float)]),
         f.relation("abs", [f.input_field("a", types.Decimal64), f.field("b", types.Decimal64)]),
         f.relation("abs", [f.input_field("a", types.Decimal128), f.field("b", types.Decimal128)]),
@@ -80,7 +89,8 @@ natural_log = f.relation(
     "natural_log",
     [f.input_field("a", types.Number), f.field("b", types.Number)],
     overloads=[
-        f.relation("natural_log", [f.input_field("a", types.Int), f.field("b", types.Float)]),
+        f.relation("natural_log", [f.input_field("a", types.Int64), f.field("b", types.Float)]),
+        f.relation("natural_log", [f.input_field("a", types.Int128), f.field("b", types.Float)]),
         f.relation("natural_log", [f.input_field("a", types.Float), f.field("b", types.Float)]),
         f.relation("natural_log", [f.input_field("a", types.Decimal64), f.field("b", types.Decimal64)]),
         f.relation("natural_log", [f.input_field("a", types.Decimal128), f.field("b", types.Decimal128)]),
@@ -91,7 +101,8 @@ sqrt = f.relation(
     "sqrt",
     [f.input_field("a", types.Number), f.field("b", types.Number)],
     overloads=[
-        f.relation("sqrt", [f.input_field("a", types.Int), f.field("b", types.Float)]),
+        f.relation("sqrt", [f.input_field("a", types.Int64), f.field("b", types.Float)]),
+        f.relation("sqrt", [f.input_field("a", types.Int128), f.field("b", types.Float)]),
         f.relation("sqrt", [f.input_field("a", types.Float), f.field("b", types.Float)]),
         f.relation("sqrt", [f.input_field("a", types.Decimal64), f.field("b", types.Decimal64)]),
         f.relation("sqrt", [f.input_field("a", types.Decimal128), f.field("b", types.Decimal128)]),
@@ -102,7 +113,8 @@ maximum = f.relation(
     "maximum",
     [f.input_field("a", types.Number), f.input_field("b", types.Number), f.field("c", types.Number)],
     overloads=[
-        f.relation("maximum", [f.input_field("a", types.Int), f.input_field("b", types.Int), f.field("c", types.Int)]),
+        f.relation("maximum", [f.input_field("a", types.Int64), f.input_field("b", types.Int64), f.field("c", types.Int64)]),
+        f.relation("maximum", [f.input_field("a", types.Int128), f.input_field("b", types.Int128), f.field("c", types.Int128)]),
         f.relation("maximum", [f.input_field("a", types.Float), f.input_field("b", types.Float), f.field("c", types.Float)]),
         f.relation("maximum", [f.input_field("a", types.Decimal64), f.input_field("b", types.Decimal64), f.field("c", types.Decimal64)]),
         f.relation("maximum", [f.input_field("a", types.Decimal128), f.input_field("b", types.Decimal128), f.field("c", types.Decimal128)]),
@@ -113,38 +125,70 @@ minimum = f.relation(
     "minimum",
     [f.input_field("a", types.Number), f.input_field("b", types.Number), f.field("c", types.Number)],
     overloads=[
-        f.relation("minimum", [f.input_field("a", types.Int), f.input_field("b", types.Int), f.field("c", types.Int)]),
+        f.relation("minimum", [f.input_field("a", types.Int64), f.input_field("b", types.Int64), f.field("c", types.Int64)]),
+        f.relation("minimum", [f.input_field("a", types.Int128), f.input_field("b", types.Int128), f.field("c", types.Int128)]),
         f.relation("minimum", [f.input_field("a", types.Float), f.input_field("b", types.Float), f.field("c", types.Float)]),
         f.relation("minimum", [f.input_field("a", types.Decimal64), f.input_field("b", types.Decimal64), f.field("c", types.Decimal64)]),
         f.relation("minimum", [f.input_field("a", types.Decimal128), f.input_field("b", types.Decimal128), f.field("c", types.Decimal128)]),
     ],
 )
 
+ceil = f.relation(
+    "ceil",
+    [f.input_field("a", types.Number), f.field("b", types.Number)],
+    overloads=[
+        f.relation("ceil", [f.input_field("a", types.Int64), f.field("b", types.Int64)]),
+        f.relation("ceil", [f.input_field("a", types.Int128), f.field("b", types.Int128)]),
+        f.relation("ceil", [f.input_field("a", types.Float), f.field("b", types.Float)]),
+        f.relation("ceil", [f.input_field("a", types.Decimal64), f.field("b", types.Decimal64)]),
+        f.relation("ceil", [f.input_field("a", types.Decimal128), f.field("b", types.Decimal128)]),
+    ],
+)
+
+floor = f.relation(
+    "floor",
+    [f.input_field("a", types.Number), f.field("b", types.Number)],
+    overloads=[
+        f.relation("floor", [f.input_field("a", types.Int64), f.field("b", types.Int64)]),
+        f.relation("floor", [f.input_field("a", types.Int128), f.field("b", types.Int128)]),
+        f.relation("floor", [f.input_field("a", types.Float), f.field("b", types.Float)]),
+        f.relation("floor", [f.input_field("a", types.Decimal64), f.field("b", types.Decimal64)]),
+        f.relation("floor", [f.input_field("a", types.Decimal128), f.field("b", types.Decimal128)]),
+    ],
+)
+
 isinf = f.relation("isinf", [f.input_field("a", types.Float)])
 isnan = f.relation("isnan", [f.input_field("a", types.Float)])
 
 # Strings
 concat = f.relation("concat", [f.input_field("a", types.String), f.input_field("b", types.String), f.field("c", types.String)])
-num_chars = f.relation("num_chars", [f.input_field("a", types.String), f.field("b", types.Int)])
+num_chars = f.relation("num_chars", [f.input_field("a", types.String), f.field("b", types.Int128)])
 starts_with = f.relation("starts_with", [f.input_field("a", types.String), f.input_field("b", types.String)])
 ends_with = f.relation("ends_with", [f.input_field("a", types.String), f.input_field("b", types.String)])
 contains = f.relation("contains", [f.input_field("a", types.String), f.input_field("b", types.String)])
-substring = f.relation("substring", [f.input_field("a", types.String), f.input_field("b", types.Int), f.input_field("c", types.Int), f.field("d", types.String)])
+substring = f.relation("substring", [f.input_field("a", types.String), f.input_field("b", types.Int64), f.input_field("c", types.Int64), f.field("d", types.String)])
 like_match = f.relation("like_match", [f.input_field("a", types.String), f.field("b", types.String)])
 lower = f.relation("lower", [f.input_field("a", types.String), f.field("b", types.String)])
 upper = f.relation("upper", [f.input_field("a", types.String), f.field("b", types.String)])
+strip = f.relation("strip", [f.input_field("a", types.String), f.field("b", types.String)])
 
 # Dates
-date_year = f.relation("date_year", [f.input_field("a", types.Date), f.field("b", types.Int)])
-date_month = f.relation("date_month", [f.input_field("a", types.Date), f.field("b", types.Int)])
-date_day = f.relation("date_day", [f.input_field("a", types.Date), f.field("b", types.Int)])
+date_year = f.relation("date_year", [f.input_field("a", types.Date), f.field("b", types.Int64)])
+date_month = f.relation("date_month", [f.input_field("a", types.Date), f.field("b", types.Int64)])
+date_day = f.relation("date_day", [f.input_field("a", types.Date), f.field("b", types.Int64)])
+date_add = f.relation("date_add", [f.input_field("a", types.Date), f.input_field("b", types.Int64), f.field("c", types.Date)])
+dates_period_days = f.relation("dates_period_days", [f.input_field("a", types.Date), f.input_field("b", types.Date), f.field("c", types.Int64)])
+datetimes_period_milliseconds = f.relation("datetimes_period_milliseconds", [f.input_field("a", types.DateTime), f.input_field("b", types.DateTime), f.field("c", types.Int64)])
+date_subtract = f.relation("date_subtract", [f.input_field("a", types.Date), f.input_field("b", types.Int64), f.field("c", types.Date)])
+datetime_add = f.relation("datetime_add", [f.input_field("a", types.DateTime), f.input_field("b", types.Int64), f.field("c", types.DateTime)])
+datetime_subtract = f.relation("datetime_subtract", [f.input_field("a", types.DateTime), f.input_field("b", types.Int64), f.field("c", types.DateTime)])
 
 # Other
 range = f.relation("range", [
-    f.input_field("start", types.Int),
-    f.input_field("stop", types.Int),
-    f.input_field("step", types.Int),
-    f.field("result", types.Int),
+    f.input_field("start", types.Int64),
+    f.input_field("stop", types.Int64),
+    f.input_field("step", types.Int64),
+    f.field("result", types.Int64),
 ])
 
 hash = f.relation("hash", [f.input_field("args", types.AnyList), f.field("hash", types.Hash)])
@@ -208,7 +252,7 @@ def aggregation(name: str, params: list[ir.Field], overload_types: Optional[list
 # ])
 # note that count does not need "over" because it counts the projection
 count = aggregation("count", [
-    f.field("result", types.Int)
+    f.field("result", types.Int128)
 ])
 stats = aggregation("stats", [
     f.input_field("over", types.Number),
@@ -220,7 +264,8 @@ sum = aggregation("sum", [
     f.input_field("over", types.Number),
     f.field("result", types.Number)
 ], overload_types=[
-    (types.Int, types.Int),
+    (types.Int64, types.Int64),
+    (types.Int128, types.Int128),
     (types.Float, types.Float),
     (types.Decimal64, types.Decimal64),
     (types.Decimal128, types.Decimal128),
@@ -229,7 +274,8 @@ avg = aggregation("avg", [
     f.input_field("over", types.Number),
     f.field("result", types.Number)
 ], overload_types=[
-    (types.Int, types.Float), # nb. Float because Int / Int is Float
+    (types.Int64, types.Float), # nb. Float because Int / Int is Float
+    (types.Int128, types.Float), # nb. Float because Int / Int is Float
     (types.Float, types.Float),
     (types.Decimal64, types.Decimal64),
     (types.Decimal128, types.Decimal128),
@@ -238,7 +284,8 @@ max = aggregation("max", [
     f.input_field("over", types.Any),
     f.field("result", types.Any)
 ], overload_types=[
-    (types.Int, types.Int),
+    (types.Int64, types.Int64),
+    (types.Int128, types.Int128),
     (types.Float, types.Float),
     (types.Decimal64, types.Decimal64),
     (types.Decimal128, types.Decimal128),
@@ -251,7 +298,8 @@ min = aggregation("min", [
     f.input_field("over", types.Any),
     f.field("result", types.Any)
 ], overload_types=[
-    (types.Int, types.Int),
+    (types.Int64, types.Int64),
+    (types.Int128, types.Int128),
     (types.Float, types.Float),
     (types.Decimal64, types.Decimal64),
     (types.Decimal128, types.Decimal128),
@@ -268,6 +316,16 @@ parse_date = f.relation("parse_date", [f.input_field("a", types.String), f.input
 parse_datetime = f.relation("parse_datetime", [f.input_field("a", types.String), f.input_field("b", types.String), f.field("c", types.DateTime)])
 parse_decimal64 = f.relation("parse_decimal64", [f.input_field("a", types.String), f.field("b", types.Decimal64)])
 parse_decimal128 = f.relation("parse_decimal128", [f.input_field("a", types.String), f.field("b", types.Decimal128)])
+
+millisecond = f.relation("millisecond", [f.input_field("a", types.Int64), f.field("b", types.Int64)])
+second = f.relation("second", [f.input_field("a", types.Int64), f.field("b", types.Int64)])
+minute = f.relation("minute", [f.input_field("a", types.Int64), f.field("b", types.Int64)])
+hour = f.relation("hour", [f.input_field("a", types.Int64), f.field("b", types.Int64)])
+day = f.relation("day", [f.input_field("a", types.Int64), f.field("b", types.Int64)])
+week = f.relation("week", [f.input_field("a", types.Int64), f.field("b", types.Int64)])
+month = f.relation("month", [f.input_field("a", types.Int64), f.field("b", types.Int64)])
+year = f.relation("year", [f.input_field("a", types.Int64), f.field("b", types.Int64)])
+
 # Alias parse_decimal128 to parse_decimal
 parse_decimal = parse_decimal128
 
@@ -282,19 +340,19 @@ cast = f.relation(
 )
 
 # Date construction with less overhead
-construct_date = f.relation("construct_date", [f.input_field("year", types.Int), f.input_field("month", types.Int), f.input_field("day", types.Int), f.field("date", types.Date)])
-construct_datetime = f.relation("construct_datetime", [f.input_field("year", types.Int), f.input_field("month", types.Int), f.input_field("day", types.Int), f.input_field("hour", types.Int), f.input_field("minute", types.Int), f.input_field("second", types.Int), f.field("datetime", types.DateTime)])
-construct_datetime_ms_tz = f.relation("construct_datetime_ms_tz", [f.input_field("year", types.Int), f.input_field("month", types.Int), f.input_field("day", types.Int), f.input_field("hour", types.Int), f.input_field("minute", types.Int), f.input_field("second", types.Int), f.input_field("milliseconds", types.Int), f.input_field("timezone", types.String), f.field("datetime", types.DateTime)])
+construct_date = f.relation("construct_date", [f.input_field("year", types.Int64), f.input_field("month", types.Int64), f.input_field("day", types.Int64), f.field("date", types.Date)])
+construct_datetime = f.relation("construct_datetime", [f.input_field("year", types.Int64), f.input_field("month", types.Int64), f.input_field("day", types.Int64), f.input_field("hour", types.Int64), f.input_field("minute", types.Int64), f.input_field("second", types.Int64), f.field("datetime", types.DateTime)])
+construct_datetime_ms_tz = f.relation("construct_datetime_ms_tz", [f.input_field("year", types.Int64), f.input_field("month", types.Int64), f.input_field("day", types.Int64), f.input_field("hour", types.Int64), f.input_field("minute", types.Int64), f.input_field("second", types.Int64), f.input_field("milliseconds", types.Int64), f.input_field("timezone", types.String), f.field("datetime", types.DateTime)])
 
 # Solver helpers
 rel_primitive_solverlib_fo_appl = f.relation("rel_primitive_solverlib_fo_appl", [
-    f.input_field("op", types.Int),
+    f.input_field("op", types.Int64),
     f.input_field("args", types.AnyList),
     f.field("result", types.String),
 ])
 rel_primitive_solverlib_ho_appl = aggregation("rel_primitive_solverlib_ho_appl", [
     f.input_field("over", types.Any),
-    f.field("op", types.Int),
+    f.field("op", types.Int64),
     f.field("result", types.String),
 ])
 implies = f.relation("implies", [f.input_field("a", types.Bool), f.input_field("b", types.Bool)])
@@ -339,10 +397,13 @@ builtin_relations = _compute_builtin_relations()
 builtin_overloads = _compute_builtin_overloads()
 builtin_relations_by_name = dict((r.name, r) for r in builtin_relations)
 
-string_binary_builtins = [num_chars, starts_with, ends_with, contains, like_match, lower, upper]
+string_binary_builtins = [num_chars, starts_with, ends_with, contains, like_match, lower, upper, strip]
 
-date_builtins = [date_year, date_month, date_day]
+date_builtins = [date_year, date_month, date_day, date_add, date_subtract, datetime_add, datetime_subtract, dates_period_days, datetimes_period_milliseconds]
+
+date_periods = [year, month, week, day, hour, minute, second, millisecond]
 
 math_builtins = [abs, *abs.overloads, natural_log, *natural_log.overloads, sqrt, *sqrt.overloads,
                  maximum, *maximum.overloads, minimum, *minimum.overloads, mod, *mod.overloads,
-                 power, *power.overloads, trunc_div]
+                 power, *power.overloads, trunc_div, *trunc_div.overloads, ceil, *ceil.overloads,
+                 floor, *floor.overloads]
