@@ -38,12 +38,18 @@ class DependencyInfo():
             return self.input_bindings[node.id]
         return None
 
+    def has_inputs(self, node: ir.Task):
+        return node.id in self.input_bindings
+
     def task_outputs(self, node: ir.Task) -> Optional[OrderedSet[ir.Var]]:
         """ The output variables for this task, if any. """
         if node.id in self.output_bindings:
             return self.output_bindings[node.id]
         return None
 
+    def has_outputs(self, node: ir.Task):
+        return node.id in self.output_bindings
+
     def task_dependencies(self, task: ir.Task) -> OrderedSet[ir.Task]:
         """
         All dependencies for this task, if any. This includes tasks in outer contexts,
@@ -643,13 +649,15 @@ class BindingAnalysis(visitor.Visitor):
         super().visit_logical(node, parent)
         self._grounded.pop()
 
+        hoisted_vars = helpers.hoisted_vars(node.hoisted)
         if grounded_by_ancestors:
-            # inputs to this logical: grounded by ancestor while being used by a child
+            # inputs to this logical: grounded by ancestor while being used by a child,
+            # excluding variables hoisted by the logical
             vars = helpers.collect_vars(node)
-            self.input(node, grounded_by_ancestors & vars)
+            self.input(node, (grounded_by_ancestors & vars) - hoisted_vars)
 
         # outputs are vars declared as hoisted
-        self.output(node, helpers.hoisted_vars(node.hoisted))
+        self.output(node, hoisted_vars)
 
 
     def visit_union(self, node: ir.Union, parent: Optional[ir.Node]):
