@@ -3,14 +3,20 @@ Helpers to analyze the metamodel IR.
 """
 from __future__ import annotations
 
-from typing import cast, Tuple, Iterable
+from typing import cast, Tuple, Iterable, Optional
 from relationalai.early_access.metamodel import ir, visitor, builtins, types, factory as f
-from relationalai.early_access.metamodel.util import OrderedSet, FrozenOrderedSet, flatten_tuple, ordered_set
+from relationalai.early_access.metamodel.util import NameCache, OrderedSet, FrozenOrderedSet, flatten_tuple, ordered_set
 
 #--------------------------------------------------
 # Checks
 #--------------------------------------------------
 
+def is_concept_lookup(node: ir.Lookup|ir.Relation):
+    """ Whether this task is a concept lookup. """
+    if isinstance(node, ir.Lookup) and is_concept_lookup(node.relation):
+        return True
+    return builtins.concept_relation_annotation in node.annotations
+
 def is_external(relation: ir.Relation):
    """ Whether this relation is external, by being marked with the external annotation. """
    return builtins.external_annotation in relation.annotations
@@ -50,7 +56,7 @@ def relation_is_proper_subtype(r1: ir.Relation, r2: ir.Relation):
 
 def relation_name_prefix(relation: ir.Relation):
     prefix = ""
-    if len(relation.fields) > 0 and builtins.concept_relation_annotation not in relation.annotations:
+    if len(relation.fields) > 0 and not is_concept_lookup(relation):
         main_type = relation.fields[0].type
         if (isinstance(main_type, ir.ScalarType) and
             main_type not in types.builtin_types and
@@ -88,6 +94,25 @@ def get_outputs(lookup: ir.Lookup):
                     outputs.append(arg)
         return outputs
 
+def get_agg_outputs(lookup: ir.Aggregate):
+    """
+        Return an array with the arguments of this aggregate that are referring to output fields
+        on the relation being looked up.
+    """
+    outputs = []
+    # register variables depending on the input flag of the relation bound to the lookup
+    for idx, fld in enumerate(lookup.aggregation.fields):
+        arg = lookup.args[idx]
+        if isinstance(arg, Iterable):
+            # deal with ListType fields that pack arguments in a tuple
+            for element in arg:
+                if isinstance(element, ir.Var) and not fld.input:
+                    outputs.append(element)
+        else:
+            if isinstance(arg, ir.Var) and not fld.input:
+                outputs.append(arg)
+    return outputs
+
 #--------------------------------------------------
 # Filters
 #--------------------------------------------------
@@ -96,6 +121,10 @@ def aggregate_outputs(agg: ir.Aggregate) -> list[ir.Var]:
     """ Get the list of vars bound to the outputs of this aggregation. """
     return list(filter(lambda arg: isinstance(arg, ir.Var) and not is_aggregate_input(arg, agg), agg.args)) # type: ignore
 
+def aggregate_inputs(agg: ir.Aggregate) -> list[ir.Var]:
+    """ Get the list of vars bound to the args that are inputs of this aggregation. """
+    return list(filter(lambda arg: isinstance(arg, ir.Var) and is_aggregate_input(arg, agg), agg.args)) # type: ignore
+
 def effective_logicals(tasks: OrderedSet[ir.Task]) -> OrderedSet[ir.Logical]:
     """ Filter tasks to return only the Logical tasks that are effective. """
     return OrderedSet.from_iterable(filter(lambda t: is_effective_logical(t), tasks))
@@ -106,7 +135,11 @@ def nullable_logicals(tasks: OrderedSet[ir.Task]) -> OrderedSet[ir.Logical]:
 
 def hoisted_vars(hoisted: Iterable[ir.VarOrDefault]) -> list[ir.Var]:
     """ Extract the vars from defaults in the hoisted list, returning just Vars. """
-    return [v.var if isinstance(v, ir.Default) else v for v in hoisted]
+    return [hoisted_var(v) for v in hoisted]
+
+def hoisted_var(hoisted: ir.VarOrDefault) -> ir.Var:
+    """ Extract the var from VarOrDefault, returning just Var. """
+    return hoisted.var if isinstance(hoisted, ir.Default) else hoisted
 
 def vars(args: Tuple[ir.Value, ...]) -> list[ir.Var]:
     """ Filter this list of values, keeping only Vars. """
@@ -195,6 +228,11 @@ class RewriteContext():
         # new relations created during the pass
         self.relations: list[ir.Relation] = []
 
+
+#--------------------------------------------------
+# Rewrite helpers
+#--------------------------------------------------
+
 def extract(task: ir.Task, body: OrderedSet[ir.Task], exposed_vars: list[ir.Var], ctx: RewriteContext, name: str) -> ir.Relation:
     """
     Extract into this Analysiscontext a new top level Logical that contains this body plus a
@@ -223,3 +261,9 @@ def create_connection_relation(task: ir.Task, exposed_vars: list[ir.Var], ctx: R
     ctx.relations.append(connection)
 
     return connection
+
+
+def create_task_name(name_cache: NameCache, task: ir.Task, prefix: Optional[str]=None) -> str:
+    """ Helper to generate consistent names for tasks extracted from a logical. """
+    prefix = prefix if prefix else f"_{task.kind}"
+    return name_cache.get_name(task.id, prefix)
