@@ -494,8 +494,8 @@ class Data(Task):
     def __iter__(self) -> Iterator[tuple[Any, ...]]:
         return iter(self.data.itertuples(index=True))
 
-PyValue = PyUnion[str, int, float, PyDecimal, bool, date, datetime, None]
-Value = PyUnion[PyValue, Var, Default, Literal, Type, Relation, Tuple["Value", ...], FrozenOrderedSet["Value"]]
+PyValue = PyUnion[str, int, float, PyDecimal, bool, date, datetime]
+Value = PyUnion[Var, Default, Literal, Type, Relation, None, Tuple["Value", ...], FrozenOrderedSet["Value"]]
 
 @acceptor
 @reconstructor
@@ -607,7 +607,7 @@ class Printer(BasePrinter):
         else:
             self._indent_print_nl(depth, name)
 
-    def value_to_string(self, value:PyUnion[Value, Tuple[Value, ...]]) -> str:
+    def value_to_string(self, value:PyUnion[PyValue, Value, Tuple[Value, ...]]) -> str:
         """ Return a string representation of the value. """
         if isinstance(value, (int, float, bool)):
             return json.dumps(value)
@@ -620,11 +620,11 @@ class Printer(BasePrinter):
         elif value is None:
             return "None"
         elif isinstance(value, Var):
-            return self.name_cache.get_name(value.id, value.name)
+            return f"{self.name_cache.get_name(value.id, value.name)}::{self.value_to_string(value.type)}"
         elif isinstance(value, Default):
             return f"{self.name_cache.get_name(value.var.id, value.var.name)}={value.value}"
         elif isinstance(value, Literal):
-            return self.value_to_string(value.value)
+            return f"{self.value_to_string(value.value)}::{self.value_to_string(value.type)}"
         elif isinstance(value, ListType):
             return f"[{self.value_to_string(value.element_type)}]"
         elif isinstance(value, UnionType):
