@@ -3,5 +3,6 @@ from .list_types import RewriteListTypes
 from .gc_nodes import GarbageCollectNodes
 from .flatten import Flatten
 from .extract_keys import ExtractKeys
+from .extract_nested_logicals import ExtractNestedLogicals
 
-__all__ = ["Splinter", "RewriteListTypes", "GarbageCollectNodes", "Flatten", "ExtractKeys"]
+__all__ = ["Splinter", "RewriteListTypes", "GarbageCollectNodes", "Flatten", "ExtractKeys", "ExtractNestedLogicals"]
