@@ -13,7 +13,7 @@ from relationalai.early_access.metamodel.typer.typer import to_type, is_primitiv
 class Flatten(Pass):
     """
     Traverses the model's root to flatten it as much as possible. The result of this pass is
-    a Logical root where all nested tasks that represent a rule in Rel are extraced to the
+    a Logical root where all nested tasks that represent a rule in Rel are extracted to the
     top level.
 
     - nested logical with updates becomes a top-level logical (a rule)
@@ -130,9 +130,6 @@ class Flatten(Pass):
         self.name_cache = NameCache(start_from_one=True)
         self._handle_outputs = handle_outputs
 
-    def create_task_name(self, task: ir.Task, prefix: Optional[str]=None) -> str:
-        prefix = prefix if prefix else f"_{task.kind}"
-        return self.name_cache.get_name(task.id, prefix)
 
     #--------------------------------------------------
     # Public API
@@ -254,7 +251,7 @@ class Flatten(Pass):
 
             # extract a new logical for the aggregate, exposing aggregate group-by and results
             exposed_vars = OrderedSet.from_iterable(list(agg.group) + helpers.aggregate_outputs(agg))
-            name = self.create_task_name(agg)
+            name = helpers.create_task_name(self.name_cache, agg)
             connection = helpers.extract(agg, body, exposed_vars.get_list(), ctx.rewrite_ctx, name)
 
             # return a reference to the connection relation
@@ -275,7 +272,7 @@ class Flatten(Pass):
             exposed_vars_raw = list(rank.projection) + list(rank.group) + list(rank.args) +[rank.result]
             # deduplicate vars
             exposed_vars = OrderedSet.from_iterable(exposed_vars_raw)
-            name = self.create_task_name(rank)
+            name = helpers.create_task_name(self.name_cache, rank)
             connection = helpers.extract(rank, body, exposed_vars.get_list(), ctx.rewrite_ctx, name)
 
             # return a reference to the connection relation
@@ -314,7 +311,7 @@ class Flatten(Pass):
                 negated_references += 1
 
             # the connection relation to be used for this branch
-            name = self.create_task_name(branch, "_match")
+            name = helpers.create_task_name(self.name_cache, branch, "_match")
             relation = helpers.create_connection_relation(branch, exposed_vars, ctx.rewrite_ctx, name)
             branch_update = f.derive(relation, exposed_vars)
             ctx.extra_tasks.append(branch_update)
@@ -358,7 +355,7 @@ class Flatten(Pass):
         body = ctx.info.task_dependencies(union)
         exposed_vars = self.compute_exposed_vars(union, union.tasks, ctx)
 
-        name = self.create_task_name(union)
+        name = helpers.create_task_name(self.name_cache, union)
         relation = helpers.create_connection_relation(union, exposed_vars, ctx.rewrite_ctx, name)
         for branch in union.tasks:
 
@@ -411,7 +408,7 @@ class Flatten(Pass):
         if len(req.checks) > 1 and isinstance(domain, ir.Logical) and len(domain.body) > 1:
             body = OrderedSet.from_iterable(domain.body)
             vars = helpers.hoisted_vars(domain.hoisted)
-            name = self.create_task_name(req)
+            name = helpers.create_task_name(self.name_cache, req)
             connection = helpers.extract(req, body, vars, ctx.rewrite_ctx, name)
             domain = f.logical([f.lookup(connection, vars)], vars)
 
