@@ -56,19 +56,7 @@ class Splinter(c.Pass):
                 if connection:
                     # there's a connection, create a logical that looks up from the connection
                     # and derives into the effect
-                    effect_vars = collect_implicit_vars(effect)
-                    new_vars = []
-                    lookup_vars = []
-                    for v in effects_vars:
-                        if v in effect_vars:
-                            lookup_vars.append(v)
-                        else:
-                            new_var = f.wild(v.type)
-                            new_vars.append(new_var)
-                            lookup_vars.append(new_var)
-                    lookup = f.lookup(connection, lookup_vars)
-                    if new_vars:
-                        lookup = f.exists(tuple(new_vars), lookup)
+                    lookup = f.lookup(connection, effects_vars.get_list())
                     final.append(f.logical([lookup, effect]))
                 else:
                     # if there's no connection, the effect is self contained (probably literals)
