@@ -25,6 +25,8 @@ def to_name(type:ir.Type) -> str:
         return type.name
     elif isinstance(type, ir.UnionType):
         return '|'.join([to_name(t) for t in type.types])
+    elif isinstance(type, ir.ListType):
+        return f"[{to_name(type.element_type)}]"
     else:
         raise TypeError(f"Unknown type: {type}")
 
@@ -36,7 +38,7 @@ def to_name(type:ir.Type) -> str:
 TYPE_MAPPING = {
     str: types.String,
     bool: types.Bool,
-    int: types.Int,
+    int: types.Int128,
     float: types.Float,
     PyDecimal: types.Decimal128,
     datetime.date: types.Date,
@@ -44,20 +46,36 @@ TYPE_MAPPING = {
 }
 
 # map potential type conversions to their cost
-CONVERSION_LATTICE:set[tuple[ir.Type, ir.Type]] = {
-    # Int -> Decimal64 -> Decimal128 -> Float
-    (types.Int, types.Decimal64),
-    (types.Decimal64, types.Decimal128),
-    (types.Decimal128, types.Float),
+# The differences in the costs of converting from one type to another should be
+# smaller than the cost of a literal conversion.
+# Otherwise, we'll introduce spurious conversions of say Int->Decimal64 plus a cheap literal conversion
+# Float->Decimal64, rather than a single literal conversion Int->Float.
+# Use a large enough number here so that if all arguments are literals, it's cheaper to convert
+# the literals (cost 10) rather than not convert some literals and convert a non-literal instead.
+# For instance, for `y=1+x` is should be cheaper to convert 1 to float and not convert x and y, than it would be
+# convert both x and y to integer.
+# We do however, want to prefer converting to number types lower in the lattice.
+# So we make the cost depend only on the result type.
+# Making all the costs the same would introduce spurious unions of conversions.
+
+CONVERSION_LATTICE:dict[tuple[ir.Type, ir.Type], int] = {
+    # Make the cost depend only on the result type.
+    (types.Int64, types.Int128): 1000,
+    (types.Int64, types.Decimal64): 1002,
+    (types.Int64, types.Decimal128): 1003,
+    (types.Int64, types.Float): 1004,
+    (types.Int128, types.Decimal128): 1003,
+    (types.Int128, types.Float): 1004,
+    (types.Decimal64, types.Decimal128): 1003,
+    (types.Decimal64, types.Float): 1004,
+    (types.Decimal128, types.Float): 1004,
     # Int -> Hash (UInt128)
-    (types.Int, types.Hash),
-    # transitive closure of the above
-    (types.Int, types.Decimal128),
-    (types.Decimal64, types.Float),
-    (types.Int, types.Float),
+    (types.Int64, types.Hash): 1001,
+    (types.Int128, types.Hash): 1001,
+    (types.Int128, types.UInt128): 1001,
 }
 
-PRIMITIVES = [types.Int, types.Float, types.Decimal64, types.Decimal128, types.Bool, types.String, types.Date, types.DateTime, types.Hash]
+PRIMITIVES = [types.Int64, types.Int128, types.UInt128, types.Float, types.Decimal64, types.Decimal128, types.Bool, types.String, types.Date, types.DateTime, types.Hash]
 
 ENTITY_LIKES = [types.RowId]
 
@@ -117,9 +135,9 @@ def type_matches(actual:ir.Type, expected:ir.Type, allow_expected_parents=False)
                 return True
     if actual == types.Any or expected == types.Any:
         return True
-    if actual == types.Number and expected in (types.Int, types.Float, types.Decimal64, types.Decimal128):
+    if actual == types.Number and expected in (types.Int64, types.Int128, types.Float, types.Decimal64, types.Decimal128):
         return True
-    if expected == types.Number and actual in (types.Int, types.Float, types.Decimal64, types.Decimal128):
+    if expected == types.Number and actual in (types.Int64, types.Int128, types.Float, types.Decimal64, types.Decimal128):
         return True
     return False
 
@@ -127,14 +145,12 @@ def conversion_allowed(arg:ir.Value, actual:ir.Type, expected:ir.Type, arg_types
     return conversion_cost(arg, actual, expected, arg_types) < float("inf")
 
 def literal_conversion_allowed(actual:ir.Type, expected:ir.Type) -> bool:
-    if actual == types.Int and expected in (types.Float, types.Decimal64, types.Decimal128):
+    if actual in (types.Int64, types.Int128) and expected in (types.Hash, types.Int64, types.Int128, types.Float, types.Decimal64, types.Decimal128):
         return True
     elif actual == types.Float and expected in (types.Decimal64, types.Decimal128):
         return True
     elif actual in (types.Decimal64, types.Decimal128) and expected in (types.Decimal64, types.Decimal128):
         return True
-    elif actual == types.Int and expected == types.Hash:
-        return True
     return False
 
 def conversion_cost(arg: ir.Value, actual:ir.Type, expected:ir.Type, arg_types:set[ir.Type]) -> float:
@@ -143,13 +159,17 @@ def conversion_cost(arg: ir.Value, actual:ir.Type, expected:ir.Type, arg_types:s
 
     # if we have a type variable and all the types in the expression
     # match, then there's no promotion needed and this is a valid overload
-    elif expected is types.EntityTypeVar and not is_primitive(actual) and len(arg_types) == 1:
+    if expected is types.EntityTypeVar and not is_primitive(actual) and len(arg_types) == 1:
         return 0
 
     # Literal numbers can be converted to other number types (but not to value types).
-    if isinstance(arg, ir.Literal) or isinstance(arg, types.py_literal_types):
+    if isinstance(arg, ir.Literal):
         if literal_conversion_allowed(actual, expected):
-            return 1
+            return 50
+
+    # Value types that don't match
+    if is_value_type(actual) and is_value_type(expected):
+        return float("inf")
 
     base_actual = to_base_primitive(actual)
     base_expected = to_base_primitive(expected)
@@ -161,14 +181,7 @@ def conversion_cost(arg: ir.Value, actual:ir.Type, expected:ir.Type, arg_types:s
         return 0
 
     if (base_actual, base_expected) in CONVERSION_LATTICE:
-        # The cost for converting from one type to another should be the same regardless of the types involved.
-        # Otherwise, we'll introduce spurious conversions of say Int->Decimal64 plus a cheap literal conversion
-        # Float->Decimal64, rather than a single literal conversion Int->Float.
-        # Use a large enough number here so that if all arguments are literals, it's cheaper to convert
-        # the literals (cost 1, above) rather than not convert some literals, but convert a non-literal instead.
-        # For instance, for `y=1+x` is should be cheaper to convert 1 to float and not convert x and y, than it would be
-        # convert both x and y to integer.
-        return 100
+        return CONVERSION_LATTICE[(base_actual, base_expected)]
 
     return float("inf")
 
@@ -176,6 +189,15 @@ def merge_types(type1: ir.Type, type2: ir.Type, is_literal1:bool, is_literal2:bo
     if type1 == type2:
         return type1
 
+    # give precedence to nominal types (e.g. merging USD(decimal) with decimal gives USD(decimal))
+    base_primitive_type1 = to_base_primitive(type1)
+    base_primitive_type2 = to_base_primitive(type2)
+    if base_primitive_type1 == base_primitive_type2:
+        if is_base_primitive(type1):
+            return type2
+        elif is_base_primitive(type2):
+            return type1
+
     combined = ordered_set()
     types_to_process = [type1, type2]
 
@@ -216,6 +238,9 @@ def to_base_primitive(type:ir.Type) -> Optional[ir.Type]:
                 return found
     return None
 
+def is_value_type(type:ir.Type):
+    return isinstance(type, ir.ScalarType) and is_primitive(type) and not is_base_primitive(type)
+
 def is_base_primitive(type:ir.Type) -> bool:
     return type in PRIMITIVES
 
@@ -292,12 +317,11 @@ class UnresolvedType(TyperError):
 # Propagation Network
 #--------------------------------------------------
 
-# The core idea of the typer is to build a propagation network where nodes
-# are vars, fields, or overloaded lookups/updates/aggregates. The intuition
-# is that _all_ types in the IR ultimately flow from relation fields, so if
-# we figure those out we just need to propagate their types to unknown vars, which
-# may then flow into other fields and so on.
-
+# The core idea of the typer is to build a propagation network where nodes are vars, fields,
+# or overloaded lookups/updates/aggregates. The intuition is that _all_ types in the IR
+# ultimately flow from relation fields, so if we figure those out we just need to propagate
+# their types to unknown vars, which may then flow into other fields and so on.
+#
 # This means the network only needs to contain nodes that either directly flow into
 # an abstract node or are themselves abstract. We need to track overloads because
 # their arguments effectively act like abstract vars until we've resolved the final types.
@@ -313,7 +337,7 @@ class PropagationNetwork():
         # so that even if we have edges to them, we _still_ consider them roots
         # and properly propagate types from them at the beginning
         self.loaded_roots = set()
-        self.edges:dict[Node, OrderedSet[Node]] = defaultdict(lambda: ordered_set())
+        self.edges:dict[Node, dict[int, Node]] = defaultdict(dict)
         self.has_incoming = set()
         self.type_requirements:dict[Node, OrderedSet[ir.Field]] = defaultdict(lambda: ordered_set())
 
@@ -353,7 +377,7 @@ class PropagationNetwork():
             self.roots.remove(target)
         if source not in self.has_incoming:
             self.roots.add(source)
-        self.edges[source].add(target)
+        self.edges[source][target.id] = target
         self.has_incoming.add(target)
 
     def add_type(self, node:Node, type:ir.Type, is_literal:bool):
@@ -487,7 +511,8 @@ class PropagationNetwork():
         # of a node before the node itself.
 
         # push known type nodes through the edges
-        for source in work_list:
+        while work_list:
+            source = work_list.pop(0)
             unhandled.remove(source)
             source_type = self.resolve(source)
             is_literal = self.node_is_literal.get(source, False)
@@ -497,7 +522,7 @@ class PropagationNetwork():
                 self.invalid_type(source, source_type)
 
             # propagate our type to each outgoing edge
-            for out in edges.get(source, []):
+            for out in edges.get(source, {}).values():
                 # if this is an overload then we need to try and resolve it
                 if isinstance(out, (ir.Update, ir.Lookup, ir.Aggregate)):
                     found = self.resolve_overload(out)
@@ -573,7 +598,7 @@ class PropagationNetwork():
         link_strs = []
         for src, dsts in self.edges.items():
             nodes.add(src)
-            for dst in dsts:
+            for dst in dsts.values():
                 if len(link_strs) > max_edges:
                     break
                 nodes.add(dst)
@@ -661,7 +686,7 @@ class PropagationNetwork():
         nodes = ordered_set()
         for src, dsts in self.edges.items():
             nodes.add(src)
-            nodes.update(dsts)
+            nodes.update(dsts.values())
         for src, dsts in self.type_requirements.items():
             nodes.add(src)
             nodes.update(dsts)
@@ -777,7 +802,7 @@ class Analyzer(visitor.Visitor):
 
         # if this is a population check, then it's fine to pass a subtype in to do the check
         # e.g. Employee(Person) is a valid way to check if a person is an employee
-        allow_any_parents = builtins.concept_relation_annotation in rel.annotations
+        allow_any_parents = helpers.is_concept_lookup(rel)
         arg_types = set(self.net.resolve(arg) for arg, field in zip(node.args, rel.fields) if field.input)
         for arg, field in zip(node.args, rel.fields):
             field_type = field.type
@@ -864,7 +889,7 @@ class Analyzer(visitor.Visitor):
         rel = node.relation
         # if this is a population check, then it's fine to pass a subtype in to do the population
         # e.g. Employee(Person) should be a valid way to populate a person
-        allow_any_parents = builtins.concept_relation_annotation in rel.annotations
+        allow_any_parents = helpers.is_concept_lookup(rel)
         for arg, field in zip(node.args, rel.fields):
             field_type = field.type
             arg_type = self.net.resolve(arg)
@@ -890,10 +915,9 @@ class Analyzer(visitor.Visitor):
 # Replacer
 #--------------------------------------------------
 
-# Once we've pushed all the types through the network, we need to
-# to replace the types of fields and vars that we may have discovered.
-# We also need to replace overloaded lookups with the choosen overloads
-# and do any conversions that are needed.
+# Once we've pushed all the types through the network, we need to replace the types of
+# fields and vars that we may have discovered. We also need to replace overloaded lookups
+# with the choosen overloads and do any conversions that are needed.
 @dataclass
 class Replacer(visitor.Rewriter):
     net: PropagationNetwork = dataclasses.field(init=True)
@@ -921,6 +945,56 @@ class Replacer(visitor.Rewriter):
         v, t = self.convert_literal_value(node.value, node.type)
         return node.reconstruct(value=v, type=t)
 
+    def handle_update(self, node: ir.Update, parent: ir.Node):
+        node = super().handle_update(node, parent)
+
+        # we may do conversions, so we can end up with multiple tasks
+        # in this branch and we need to track what the final args are
+        tasks = []
+        final_args = []
+        for arg, field in zip(node.args, node.relation.fields):
+            arg_type = to_type(arg)
+            field_type = to_type(field)
+            # the typer previously made sure that this should be valid so
+            # a type mismatch means we need to convert as long as this isn't
+            # a type variable
+            if field_type != types.EntityTypeVar and not type_matches(arg_type, field_type):
+                arg_base = to_base_primitive(arg_type)
+                field_base = to_base_primitive(field_type)
+                if arg_base is not None and field_base is not None and conversion_allowed(arg, arg_base, field_base):
+                    new_arg = self.convert(arg, arg_base, field_base, tasks)
+                    final_args.append(new_arg)
+                else:
+                    final_args.append(arg)
+            else:
+                final_args.append(arg)
+        tasks.append(node.reconstruct(args=tuple(final_args)))
+
+        if len(tasks) == 1:
+            return tasks[0]
+        else:
+            return f.logical(tasks, [])
+
+    def handle_aggregate(self, node: ir.Aggregate, parent: ir.Node):
+        node = super().handle_aggregate(node, parent)
+        lookup_args = list(node.args)
+        hoists = helpers.get_agg_outputs(node)
+
+        if len(node.aggregation.fields) == len(lookup_args):
+            tasks = []
+            for i in range(len(lookup_args)):
+                if not conversion_allowed(lookup_args[i], to_type(lookup_args[i]), node.aggregation.fields[i].type):
+                    self.net.mismatch(node, node.aggregation.fields[i].type, to_type(lookup_args[i]))
+                elif not type_matches(to_type(lookup_args[i]), node.aggregation.fields[i].type):
+                    lookup_args[i] = self.convert(lookup_args[i], to_type(lookup_args[i]), node.aggregation.fields[i].type, tasks)
+            tasks.append(node.reconstruct(args=tuple(lookup_args)))
+            if len(tasks) == 1:
+                return tasks[0]
+            else:
+                return f.logical(tasks, hoists)
+
+        return node
+
     def handle_lookup(self, node: ir.Lookup, parent: ir.Node):
         node = super().handle_lookup(node, parent)
         lookup_args = node.args
@@ -931,7 +1005,37 @@ class Replacer(visitor.Rewriter):
         hoists = helpers.get_outputs(node)
         branches:list[ir.Logical] = []
 
-        if node.relation == builtins.eq:
+        # Special case this solver primitive because the type system cannot handle what it
+        # needs, which is an input field types as List[Union[Int64,Float64,String,Hash]].
+        # So this code makes sure that any in128 that flows into the second argument, which
+        # is represented as a tuple, is converted into an int64, which is lossy but should
+        # work for most cases
+        if node.relation == builtins.rel_primitive_solverlib_fo_appl and len(node.relation.fields) == 3 and len(lookup_args) == 3:
+            tasks = []
+            # Make lookup_args mutable.
+            lookup_args = list(lookup_args)
+
+            # Convert the first and third arguments to the field types.
+            for i in [0, 2]:
+                if not conversion_allowed(lookup_args[i], to_type(lookup_args[i]), node.relation.fields[i].type):
+                    self.net.mismatch(node, node.relation.fields[i].type, to_type(lookup_args[i]))
+                elif not type_matches(to_type(lookup_args[i]), node.relation.fields[i].type):
+                    lookup_args[i] = self.convert(lookup_args[i], to_type(lookup_args[i]), node.relation.fields[i].type, tasks)
+
+            varargs = lookup_args[1]
+            final_varargs = []
+            assert isinstance(varargs, tuple)
+            for arg in varargs:
+                t = to_type(arg)
+                if t == types.Int128:
+                    final_varargs.append(self.convert(arg, t, types.Int64, tasks))
+                else:
+                    final_varargs.append(arg)
+            lookup_args[1] = tuple(final_varargs)
+            tasks.append(node.reconstruct(args=tuple(lookup_args)))
+            branches.append(f.logical(tasks, hoists))
+
+        elif node.relation == builtins.eq:
             tasks = []
 
             # We need to handle eq specially because its arguments can be converted symmetrically
@@ -999,7 +1103,8 @@ class Replacer(visitor.Rewriter):
                 else:
                     return node
             else:
-                self.net.mismatch(node, src_type, tgt_type)
+                # TODO: we are simply ignoring this, assuming there's some way to cast it
+                # self.net.mismatch(node, src_type, tgt_type)
                 return node
         else:
             if node.id in self.net.resolved_overloads:
@@ -1048,10 +1153,10 @@ class Replacer(visitor.Rewriter):
         elif isinstance(arg, (int, float, PyDecimal)):
             value = arg
         if value is not None:
-            if isinstance(value, int) and expected == types.Int:
+            if isinstance(value, int) and expected in (types.Int64, types.Int128):
                 return f.literal(value, expected)
             if isinstance(value, (int, float, PyDecimal)) and expected == types.Float:
-                return float(value)
+                return f.literal(float(value), expected)
             if isinstance(value, (int, float, PyDecimal)) and expected in (types.Decimal64, types.Decimal128):
                 # Converting str(value) rather than value avoids precision loss with Float->Decimal conversion.
                 return f.literal(PyDecimal(str(value)), expected)
@@ -1060,6 +1165,7 @@ class Replacer(visitor.Rewriter):
             name = arg.name + "_" + to_name(expected)
         expected_base = to_base_primitive(expected) or expected
         new_arg = f.var(name, expected_base)
+        self.new_relations.add(builtins.cast)
         tasks.append(f.lookup(builtins.cast, (expected_base, arg, new_arg)))
         self.new_relations.add(builtins.cast)
         return new_arg
@@ -1072,8 +1178,10 @@ class Replacer(visitor.Rewriter):
             return datetime.datetime.fromisoformat(value), types.DateTime
         if isinstance(value, str) and types.is_subtype(t, types.Date):
             return datetime.date.fromisoformat(value), types.Date
-        if isinstance(value, int) and types.is_subtype(t, types.Int):
-            return value, types.Int
+        if isinstance(value, int) and types.is_subtype(t, types.Int64):
+            return value, types.Int64
+        if isinstance(value, int) and types.is_subtype(t, types.Int128):
+            return value, types.Int128
         if isinstance(value, (int, float, PyDecimal)) and types.is_subtype(t, types.Float):
             return float(value), types.Float
         if isinstance(value, (int, float, PyDecimal)) and types.is_subtype(t, types.Decimal64):
