@@ -5,8 +5,6 @@ from . import ir, util
 import sys
 from typing import cast, Optional
 from functools import reduce
-from decimal import Decimal as PyDecimal
-from datetime import date, datetime
 
 #
 # Basic Types
@@ -16,17 +14,20 @@ Any = ir.ScalarType("Any", util.frozen())
 Hash = ir.ScalarType("Hash", util.frozen())
 String = ir.ScalarType("String", util.frozen())
 Number = ir.ScalarType("Number", util.frozen())
-Int = ir.ScalarType("Int", util.frozen())
+Int64 = ir.ScalarType("Int64", util.frozen())
+Int128 = ir.ScalarType("Int128", util.frozen())
+UInt128 = ir.ScalarType("UInt128", util.frozen())
 Float = ir.ScalarType("Float", util.frozen())
 Decimal64 = ir.ScalarType("Decimal64", util.frozen())
 Decimal128 = ir.ScalarType("Decimal128", util.frozen())
+Decimal = Decimal128
 Bool = ir.ScalarType("Bool", util.frozen())
 Binary = ir.ScalarType("Binary", util.frozen()) # 0 or 1
 Date = ir.ScalarType("Date", util.frozen())
 DateTime = ir.ScalarType("DateTime", util.frozen())
 Symbol = ir.ScalarType("Symbol", util.frozen())
 Sha1 = ir.ScalarType("SHA1", util.frozen())
-RowId = ir.ScalarType("RowId", util.frozen(Int))
+RowId = ir.ScalarType("RowId", util.frozen(UInt128))
 Enum = ir.ScalarType("Enum", util.frozen())
 
 # a special type that represent an entity that is of a
@@ -49,8 +50,7 @@ def _compute_builtin_types() -> list[ir.Type]:
     return types
 
 # Types that can appear in literal values in the AST. These must be included in all models.
-literal_types = util.ordered_set(*[String, Bool, Int, Float, Decimal128, Date, DateTime]).frozen()
-py_literal_types = (str, bool, int, float, date, datetime, PyDecimal)
+literal_types = util.ordered_set(*[String, Bool, Int128, Float, Decimal128, Date, DateTime]).frozen()
 
 builtin_types = _compute_builtin_types()
 builtin_scalar_types_by_name = dict((t.name, t) for t in cast(list[ir.ScalarType], util.filter_by_type(builtin_types, ir.ScalarType)))
@@ -59,7 +59,7 @@ def is_any(t: ir.Type) -> bool:
     return t == Any
 
 def is_value_base_type(t: ir.Type) -> bool:
-    return isinstance(t, ir.ScalarType) and t in [Hash, String, Number, Int, Float, Decimal64, Decimal128, Bool, Binary, Symbol, Sha1, Date, DateTime]
+    return isinstance(t, ir.ScalarType) and t in [Hash, String, Number, Int64, Int128, Float, Decimal64, Decimal128, Bool, Binary, Symbol, Sha1, Date, DateTime]
 
 def is_value_type(t: ir.Type) -> bool:
     if is_value_base_type(t):
