@@ -1,5 +1,4 @@
 
-import time
 from relationalai.early_access.builder import Integer, define
 from relationalai.early_access.builder import std
 
@@ -10,8 +9,8 @@ def create_labeled_grid(model, side_length, directed = True, label = "A"):
     Create a grid Graph instance from a Model. The number of edges in each side of the grid
     is given by the parameter `side`.
     """
-    Node = model.Concept(f"GridNode_{time.time_ns()}")
-    Edge = model.Relationship(f"{{src:{Node}}} has an edge to {{dst:{Node}}} with label {{String}} {time.time_ns()}")
+    Node = model.Concept("GridNode")
+    Edge = model.Relationship(f"edge1 {{{Node}}} {{{Node}}} {{String}}")
 
     n, m = Integer.ref(), Integer.ref()
     u, v = Node.ref(), Node.ref()
