@@ -17,8 +17,8 @@ graph = Graph.from_edge_list(model, edge_list)
 Node = graph.Node
 
 # create source and target sets:
-Source = model.Relationship("{Node} is a source node")
-Target = model.Relationship("{Node} is a target node")
+Source = model.Relationship("source3 {Node}")
+Target = model.Relationship("target3 {Node}")
 
 for src in sources:
     define(Source(graph.Node.new(id=src)))
