@@ -9,7 +9,7 @@ def ball_with_repetition(g:Graph, Source, max_length):
     edge = g.Edge
     Node = g.Node
 
-    ball = g.model.Relationship(f"At hop {{d:Integer}} ball has node {{v:{Node}}}") # , [Integer, Node])
+    ball = g.model.Relationship(f"ball_rep {{Integer}} {{{Node}}}") # , [Integer, Node])
 
     u, v, src = Node.ref(), Node.ref(), Node.ref()
     level, m = Integer.ref(), Integer.ref()
