@@ -9,8 +9,8 @@ def two_balls_repetition(g:Graph, Source, Target, max_length):
     edge = g.Edge
     Node = g.Node
 
-    source_ball = g.model.Relationship(f"At hop {{Integer}} the source ball has node {{{Node}}}")
-    target_ball = g.model.Relationship(f"At hop {{Integer}} the target ball has node {{{Node}}}")
+    source_ball = g.model.Relationship(f"source_ball1 {{Integer}} {{{Node}}}")
+    target_ball = g.model.Relationship(f"target_ball1 {{Integer}} {{{Node}}}")
 
     radius_S = max_length // 2
     radius_T = max_length - radius_S
