@@ -3,8 +3,8 @@ from relationalai.early_access.builder import define, max
 
 
 def setup_iteration(model, condition, initial = 0, final = 50):
-    iter = model.Relationship("Current value of the iterator is {Integer}")
-    next = model.Relationship("{Integer} is a candidate for the next value of the iterator")
+    iter = model.Relationship("iter1 {Integer}")
+    next = model.Relationship("next1 {Integer}")
 
     m, n = Integer.ref(), Integer.ref()
 
