@@ -5,11 +5,12 @@ from decimal import Decimal as PyDecimal
 
 from relationalai.early_access.metamodel import ir, compiler as c, builtins as bt, types, visitor, helpers, factory as f
 from relationalai.early_access.metamodel.typer import Checker, InferTypes
-from relationalai.early_access.metamodel.typer.typer import to_base_primitive, to_type
+from relationalai.early_access.metamodel.typer.typer import to_base_primitive, to_type, PRIMITIVES
 from relationalai.early_access.metamodel.rewrite import Flatten
 from relationalai.early_access.metamodel.rewrite import ExtractKeys
 
 from relationalai.early_access.metamodel.rewrite import Splinter
+from relationalai.early_access.metamodel.rewrite import ExtractNestedLogicals
 from relationalai.early_access.metamodel.visitor import ReadWriteVisitor
 from relationalai.early_access.metamodel.util import OrderedSet, group_by, NameCache, ordered_set
 
@@ -26,13 +27,14 @@ class Compiler(c.Compiler):
     def __init__(self):
         super().__init__([
             Checker(),
+            ExtractNestedLogicals(), # before InferTypes to avoid extracting casts
             InferTypes(),
             rewrite.CDC(),
             ExtractKeys(),
             # rewrite.ExtractCommon(),
             Flatten(),
-            rewrite.QuantifyVars(),
             Splinter(),
+            rewrite.QuantifyVars(),
         ])
         self.model_to_rel = ModelToRel()
 
@@ -80,6 +82,34 @@ class ModelToRel:
     def _generate_builtin_defs(self, model, reads:OrderedSet[str]) -> list[rel.Def]:
         defs = []
 
+        if "pyrel_inf" in reads:
+            defs.append(
+                rel.Def("pyrel_inf",
+                    tuple([rel.Var("x")]),
+                    rel.atom("::std::common::infinity", tuple([rel.MetaValue(64), rel.Var("x")])),
+                    tuple([rel.Annotation("inline", ())]),
+                ),
+            )
+
+        if "pyrel_NaN" in reads:
+            defs.append(
+                rel.Def("pyrel_NaN",
+                    tuple([rel.Var("x")]),
+                    rel.atom("::std::common::nan", tuple([rel.MetaValue(64), rel.Var("x")])),
+                    tuple([rel.Annotation("inline", ())]),
+                ),
+            )
+
+        if "pyrel_ID" in reads:
+            defs.append(rel.Def("pyrel_ID",
+                tuple([rel.Var("x")]),
+                rel.Or(OrderedSet[rel.Expr].from_iterable([
+                    rel.atom("::std::common::UInt128", tuple([rel.Var("x")])),
+                    rel.atom("::std::common::Missing", tuple([rel.Var("x")])),
+                ])),
+                tuple([rel.Annotation("inline", ())]),
+            ))
+
         if "pyrel_Decimal64" in reads or "pyrel_parse_decimal64" in reads:
             defs.append(
                 rel.Def("pyrel_Decimal64",
@@ -118,27 +148,152 @@ class ModelToRel:
                 ),
             )
 
+
+        if "pyrel_num_chars" in reads:
+            defs.append(
+                rel.Def("pyrel_num_chars",
+                    tuple([rel.Var("x"), rel.Var("y")]),
+                    rel.Exists(
+                        tuple([rel.Var("z")]),
+                        rel.And(ordered_set(
+                            rel.atom("::std::common::num_chars", tuple([rel.Var("x"), rel.Var("z")])),
+                            rel.Atom(self._convert_abs(types.Int64, types.Int128), tuple([rel.Var("z"), rel.Var("y")])),
+                        )),
+                    ),
+                    tuple([rel.Annotation("inline", ())]),
+                ),
+            )
+
+        if "pyrel_count" in reads:
+            defs.append(
+                rel.Def("pyrel_count",
+                    tuple([rel.Var("{R}"), rel.Var("y")]),
+                    rel.Exists(
+                        tuple([rel.Var("z")]),
+                        rel.And(ordered_set(
+                            rel.atom("::std::common::count", tuple([rel.Var("R"), rel.Var("z")])),
+                            rel.Atom(self._convert_abs(types.Int64, types.Int128), tuple([rel.Var("z"), rel.Var("y")])),
+                        )),
+                    ),
+                    tuple([rel.Annotation("inline", ())]),
+                ),
+            )
+
+        if "pyrel_sort" in reads:
+            defs.append(
+                rel.Def("pyrel_sort",
+                    tuple([rel.Var("{R}"), rel.Var("n"), rel.Var("r", varargs=True)]),
+                    rel.Exists(
+                        tuple([rel.Var("z")]),
+                        rel.And(ordered_set(
+                            rel.atom("::std::common::sort", tuple([rel.Var("R"), rel.Var("z"), rel.Var("r", varargs=True)])),
+                            rel.Atom(self._convert_abs(types.Int64, types.Int128), tuple([rel.Var("z"), rel.Var("n")])),
+                        )),
+                    ),
+                    tuple([rel.Annotation("inline", ())]),
+                ),
+            )
+
+        if "pyrel_reverse_sort" in reads:
+            defs.append(
+                rel.Def("pyrel_reverse_sort",
+                    tuple([rel.Var("{R}"), rel.Var("n"), rel.Var("r", varargs=True)]),
+                    rel.Exists(
+                        tuple([rel.Var("z")]),
+                        rel.And(ordered_set(
+                            rel.atom("::std::common::reverse_sort", tuple([rel.Var("R"), rel.Var("z"), rel.Var("r", varargs=True)])),
+                            rel.Atom(self._convert_abs(types.Int64, types.Int128), tuple([rel.Var("z"), rel.Var("n")])),
+                        )),
+                    ),
+                    tuple([rel.Annotation("inline", ())]),
+                ),
+            )
+
+        if "pyrel_top" in reads:
+            defs.append(
+                rel.Def("pyrel_top",
+                    tuple([rel.Var("k"), rel.Var("{R}"), rel.Var("n"), rel.Var("r", varargs=True)]),
+                    rel.Exists(
+                        tuple([rel.Var("z")]),
+                        rel.And(ordered_set(
+                            rel.atom("::std::common::top", tuple([rel.Var("k"), rel.Var("R"), rel.Var("z"), rel.Var("r", varargs=True)])),
+                            rel.Atom(self._convert_abs(types.Int64, types.Int128), tuple([rel.Var("z"), rel.Var("n")])),
+                        )),
+                    ),
+                    tuple([rel.Annotation("inline", ())]),
+                ),
+            )
+
+        if "pyrel_bottom" in reads:
+            defs.append(
+                rel.Def("pyrel_bottom",
+                    tuple([rel.Var("k"), rel.Var("{R}"), rel.Var("n"), rel.Var("r", varargs=True)]),
+                    rel.Exists(
+                        tuple([rel.Var("z")]),
+                        rel.And(ordered_set(
+                            rel.atom("::std::common::bottom", tuple([rel.Var("k"),rel.Var("R"), rel.Var("z"), rel.Var("r", varargs=True)])),
+                            rel.Atom(self._convert_abs(types.Int64, types.Int128), tuple([rel.Var("z"), rel.Var("n")])),
+                        )),
+                    ),
+                    tuple([rel.Annotation("inline", ())]),
+                ),
+            )
+
+        if "pyrel_dates_period_days" in reads:
+            defs.append(
+                rel.Def("pyrel_dates_period_days",
+                    tuple([rel.Var("a"), rel.Var("b"), rel.Var("c")]),
+                    rel.Exists(
+                        tuple([rel.Var("d")]),
+                        rel.And(ordered_set(
+                            rel.atom("::std::common::dates_period_days",
+                                     tuple([rel.Var("a"), rel.Var("b"), rel.Var("d")])),
+                            rel.atom("::std::common::^Day",
+                                     tuple([rel.Var("c"), rel.Var("d")]))
+                        )),
+                    ),
+                    tuple([rel.Annotation("inline", ())]),
+                    ),
+            )
+
+        if "pyrel_datetimes_period_milliseconds" in reads:
+            defs.append(
+                rel.Def("pyrel_datetimes_period_milliseconds",
+                    tuple([rel.Var("a"), rel.Var("b"), rel.Var("c")]),
+                    rel.Exists(
+                        tuple([rel.Var("m")]),
+                        rel.And(ordered_set(
+                            rel.atom("::std::common::datetimes_period_milliseconds",
+                                     tuple([rel.Var("a"), rel.Var("b"), rel.Var("m")])),
+                            rel.atom("::std::common::^Millisecond",
+                                     tuple([rel.Var("c"), rel.Var("m")]))
+                        )),
+                    ),
+                    tuple([rel.Annotation("inline", ())]),
+                    ),
+            )
+
         return defs
 
     @staticmethod
     def _convert_abs(from_type: ir.Type, to_type: ir.Type):
-        if from_type == types.Int and to_type == types.Float:
+        if from_type == types.Int64 and to_type == types.Float:
             return rel.Identifier("::std::common::int_float_convert")
-        elif from_type == types.Float and to_type == types.Int:
+        elif from_type == types.Float and to_type == types.Int64:
             return rel.Identifier("::std::common::float_int_convert")
         else:
             input_type = u.rel_typename(from_type)
             output_type = u.rel_typename(to_type)
             return rel.RelationalAbstraction(
-                tuple([rel.Var("x", type=input_type), rel.Var("y", type=output_type)]),
+                tuple([rel.Var("val_x", type=input_type), rel.Var("val_y", type=output_type)]),
                 rel.Exists(
                     tuple([rel.Var("type_x"), rel.Var("type_y")]),
                     rel.And(ordered_set(
                         # Since we declared them to be the types we're converting from and to, we can just use the types of x and y here.
                         # The Rel compiler will use the static type of the variable to compute the Type values.
-                        rel.atom("rel_primitive_typeof", tuple([rel.Var("x"), rel.Var("type_x")])),
-                        rel.atom("rel_primitive_typeof", tuple([rel.Var("y"), rel.Var("type_y")])),
-                        rel.atom("rel_primitive_convert", tuple([rel.Var("type_x"), rel.Var("type_y"), rel.Var("x"), rel.Var("y")])),
+                        rel.atom("rel_primitive_typeof", tuple([rel.Var("val_x"), rel.Var("type_x")])),
+                        rel.atom("rel_primitive_typeof", tuple([rel.Var("val_y"), rel.Var("type_y")])),
+                        rel.atom("rel_primitive_convert", tuple([rel.Var("type_x"), rel.Var("type_y"), rel.Var("val_x"), rel.Var("val_y")])),
                     )),
                 )
             )
@@ -161,6 +316,7 @@ class ModelToRel:
         reads = m.relations - rw.writes(root) - bt.builtin_relations - bt.builtin_overloads - bt.builtin_annotations - annotations
         reads = list(filter(lambda r: not r.name.startswith('^') and not helpers.is_external(r), reads))
 
+        primitive_type_names = OrderedSet.from_iterable([t.name for t in PRIMITIVES])
         declares: list[rel.Declare] = []
         for r in reads:
             if r.name in rel.infix or r.name in u.OPERATORS:
@@ -169,6 +325,11 @@ class ModelToRel:
             if helpers.is_from_cast(r):
                 continue
 
+            # TODO: should address the root of the issue
+            # In some cases we might end up with explicit Concepts for primitives in the model
+            if r.name in primitive_type_names:
+                continue
+
             # In case parameter name starts with ':' use its name instead of type name
             def requires_name(fld: ir.Field):
                 if isinstance(fld.type, ir.ScalarType):
@@ -184,7 +345,7 @@ class ModelToRel:
             # Example: declare test(:a, _x0 in Int, _x1 in String) requires true
             declares.append(rel.Declare(
                 rel.atom(self._relation_name(r), head),
-                True  # `requires true` does not generate any constraints, that affects performance on the RAI side
+                rel.true  # `requires true` does not generate any constraints, that affects performance on the RAI side
             ))
         return declares
 
@@ -219,10 +380,10 @@ class ModelToRel:
             # deal with raw sources
             if isinstance(effect, ir.Update) and effect.relation == bt.raw_source:
                 # TODO: remove this once the type checker checks this.
-                assert(len(effect.args) == 2 and isinstance(effect.args[0], str) and isinstance(effect.args[1], str))
-                if effect.args[0] != "rel":
+                assert(len(effect.args) == 2 and isinstance(effect.args[0], ir.Literal) and isinstance(effect.args[1], ir.Literal))
+                if effect.args[0].value != "rel":
                     return []
-                return [rel.RawSource(cast(str, effect.args[1]))]
+                return [rel.RawSource(cast(str, effect.args[1].value))]
             else:
                 args, lookups, rel_equiv = self._effect_args(effect)
                 if lookups:
@@ -396,15 +557,15 @@ class ModelToRel:
         has_limit = rank.limit != 0
 
         if ascending:
-            rel_name = "::std::common::top" if has_limit else "::std::common::sort"
+            rel_name = "pyrel_top" if has_limit else "pyrel_sort"
         else:
-            rel_name = "::std::common::bottom" if has_limit else "::std::common::reverse_sort"
+            rel_name = "pyrel_bottom" if has_limit else "pyrel_reverse_sort"
         return rel_name, has_limit
 
     def generate_body_expr(self, tasks: list[ir.Task]):
         """ Helper to generate the an expression from the tasks, wrapping in Ands if necessary. """
         if not tasks:
-            return True
+            return rel.true
         elif len(tasks) == 1:
             return self.handle(tasks[0])
         else:
@@ -416,9 +577,6 @@ class ModelToRel:
 
     def handle(self, n: ir.Node):
         """ Dispatch to the appropriate ir.Node handler. """
-        if isinstance(n, ir.PyValue):
-            t = to_type(n)
-            return self.handle_value(t, n)
         handler = getattr(self, f"handle_{n.kind}", None)
         if handler:
             return handler(n)
@@ -444,7 +602,8 @@ class ModelToRel:
             )
 
         if type is None:
-            return v
+            return cast(Union[rel.Primitive, rel.RelationalAbstraction, rel.MetaValue, rel.Var], v)
+
         # only wrap if v is a primitive (i.e. not a metavalue or a var, for example).
         base = to_base_primitive(type) or type
         if type == types.Symbol and isinstance(v, (str, int, float, bool)):
@@ -479,8 +638,20 @@ class ModelToRel:
                     tuple([x]),
                     rel.atom("::std::common::parse_decimal", tuple([rel.MetaValue(128), rel.MetaValue(u.DECIMAL128_SCALE), str(v), x]))
                 )
+        elif base == types.Int128 and isinstance(v, int):
+            x = rel.Var("_int")
+            return rel.RelationalAbstraction(
+                tuple([x]),
+                rel.atom("::std::common::int", tuple([128, v, x]))
+            )
+        elif (base == types.UInt128 or base == types.RowId) and isinstance(v, int):
+            x = rel.Var("_uint")
+            return rel.RelationalAbstraction(
+                tuple([x]),
+                rel.atom("::std::common::uint", tuple([128, v, x]))
+            )
         else:
-            return v
+            return cast(Union[rel.Primitive, rel.RelationalAbstraction, rel.MetaValue, rel.Var], v)
 
     def handle_value_list(self, types, values) -> list[Union[rel.Primitive, rel.MetaValue, rel.RelationalAbstraction, rel.Var]]:
         result = []
@@ -684,6 +855,7 @@ class ModelToRel:
         else:
             raise Exception(f"Cannot retrieve effect params from node {type(n)}")
 
+        assert len(orig_args) == len(handled_args)
         args, lookups, rel_equiv = [], [], []
         for idx, handled in enumerate(handled_args):
             if handled is None:
@@ -695,46 +867,79 @@ class ModelToRel:
                 rel_var = self.handle(var)
                 args.append(rel_var)
                 rel_equiv.append(rel.create_eq(rel_var, handled))
+            elif isinstance(handled, bool):
+                # boolean constants need to be bound to a var in the body which is used in the head
+                var = ir.Var(types.Any, "head")
+                args.append(self.handle(var))
+                lookups.append(f.lookup(bt.eq, [var, ir.Literal(types.Bool, handled)]))
             elif not isinstance(handled, rel.Var):
+                # other constants
                 args.append(handled)
             else:
+                orig = orig_args[idx]
+                assert isinstance(orig, ir.Var)
+
                 # Count how many times this argument has been seen before
                 cnt = handled_args[:idx].count(handled)
                 if cnt == 0:
-                    args.append(handled)
+                    arg_type = self._ir_var_to_rel_type(orig)
+                    args.append(rel.Var(handled.name, False, arg_type) if arg_type else handled)
                     continue
+
                 # Deduplicate variable
-                new_var = ir.Var(types.Any, handled.name + "_dup" + str(cnt))
-                rel_var = self.handle(new_var)
-                args.append(rel_var)
+                new_var = ir.Var(orig.type, handled.name + "_dup" + str(cnt))
+                rel_var = self.handle_var(new_var)
+                arg_type = self._ir_var_to_rel_type(orig)
+                args.append(rel.Var(rel_var.name, False, arg_type) if arg_type else rel_var)
                 rel_equiv.append(rel.create_eq(rel_var, handled))
         return tuple(args), lookups, rel_equiv
 
+    def _ir_var_to_rel_type(self,v: ir.Var) -> str:
+        primitive_type = to_base_primitive(v.type)
+        if primitive_type:
+            if primitive_type == types.Hash:
+                return "::std::common::UInt128"
+            elif primitive_type == types.Decimal64:
+                return "pyrel_Decimal64"
+            elif primitive_type == types.Decimal128:
+                return "pyrel_Decimal128"
+            elif primitive_type == types.Bool:
+                return "Boolean"
+            return str(primitive_type).strip()
+        elif v.type != types.Any and v.type != types.Enum:
+            return "pyrel_ID"
+        return ""
+
     def _rel_reads(self, root, reads:OrderedSet[str]|None = None) -> OrderedSet[str]:
         if reads is None:
             reads = OrderedSet()
 
-        if isinstance(root, list):
+        if isinstance(root, (tuple, list)):
             for r in root:
                 self._rel_reads(r, reads)
 
+        if isinstance(root, rel.Var):
+            if root.type is not None and not root.type.startswith("::std::common::"):
+                reads.add(root.type)
+
         elif isinstance(root, rel.Declare):
             assert isinstance(root.premise, rel.Atom)
-            for arg in root.premise.args:
-                if (isinstance(arg, rel.Var) and
-                    arg.type is not None and
-                    not arg.type.startswith("::std::common::")):
-                    reads.add(arg.type)
+            self._rel_reads(root.premise.args, reads)
+            self._rel_reads(root.requires, reads)
 
         elif isinstance(root, rel.Def):
+            self._rel_reads(root.params, reads)
             self._rel_reads(root.body, reads)
 
         elif isinstance(root, rel.Atom):
             if isinstance(root.expr, rel.Identifier):
                 reads.add(root.expr.name)
+            else:
+                self._rel_reads(root.expr, reads)
             self._rel_reads(root.args, reads)
 
         elif isinstance(root, rel.RelationalAbstraction):
+            self._rel_reads(root.head, reads)
             self._rel_reads(root.body, reads)
 
         elif isinstance(root, rel.And):
@@ -767,4 +972,8 @@ class ModelToRel:
             for arg in root.body:
                 self._rel_reads(arg, reads)
 
+        elif isinstance(root, rel.Var):
+            if root.type is not None and not root.type.startswith("::std::common::"):
+                reads.add(root.type)
+
         return reads
