@@ -10,6 +10,18 @@ from typing import Optional, Tuple, Union as PyUnion
 from relationalai.early_access.rel import rel_utils
 from relationalai.early_access.metamodel.util import OrderedSet, Printer as BasePrinter, ordered_set
 import json
+import re
+
+#--------------------------------------------------
+# Helpers
+#--------------------------------------------------
+
+def safe_name(name: str) -> str:
+    return re.sub(r"[ ,\.]", "_", name)
+
+#--------------------------------------------------
+# Node
+#--------------------------------------------------
 
 @dataclass(frozen=True)
 class Node:
@@ -109,6 +121,10 @@ def atom(name: str, args: Tuple[Expr, ...]):
     """ Helper to create a Atom where expression is an identifier. """
     return Atom(Identifier(name), args)
 
+# Represent the true {()} and false {} relations in Rel
+true = Identifier("true")
+false = Identifier("false")
+
 @dataclass(frozen=True)
 class RelationalAbstraction(CompositeExpr):
     """ ($head): $body """
@@ -197,18 +213,22 @@ class Printer(BasePrinter):
                     self._print(sep)
             self.print_node(s, indent)
 
-    def _print_value(self, value):
+    def _print_value(self, value, convert:bool=False):
         if isinstance(value, tuple):
             self._print("(")
             for i, v in enumerate(value):
                 if i != 0:
                     self._print(", ")
-                self._print_value(v)
+                self._print_value(v, convert=True)
             self._print(")")
         elif isinstance(value, str):
             self._print(json.dumps(value))
         elif isinstance(value, bool):
-            self._print(str(value).lower())
+            self._print(f"rel_primitive_boolean_{str(value).lower()}")
+        elif isinstance(value, int) and convert:
+            self._print("::std::common::int[128,")
+            self._print(str(value))
+            self._print("]")
         else:
             self._print(str(value))
 
@@ -237,7 +257,7 @@ class Printer(BasePrinter):
             for anno in node.annotations:
                 self.print_node(anno, indent)
                 self._nl()
-            self._indent_print(indent, f"def {node.name}")
+            self._indent_print(indent, f"def {safe_name(node.name)}")
             if node.params:
                 self._print("(")
                 self._join(node.params)
@@ -310,7 +330,7 @@ class Printer(BasePrinter):
                 self._print(f" in {node.type if node.type != 'Enum' else 'UInt128'}")
 
         elif isinstance(node, Identifier):
-            self._indent_print(indent, f"{node.name}")
+            self._indent_print(indent, safe_name(node.name))
 
         elif isinstance(node, Atom):
             if isinstance(node.expr, Identifier) and node.expr.name in infix:
