@@ -1,6 +1,6 @@
 __version__ = "0.0.0"
 
-from .solvers import SolverModel, make_name, all_different, implies, Solver
+from .solvers import SolverModel, make_name, all_different, implies, Solver, Provider
 
 __all__ = [
     "SolverModel",
@@ -8,4 +8,5 @@ __all__ = [
     "all_different",
     "implies",
     "Solver",
+    "Provider",
 ]
