@@ -10,33 +10,39 @@ from relationalai.early_access.rel.executor import RelExecutor
 # NOTE(coey) using Solver class from old solvers library to avoid code duplication
 from relationalai.experimental.solvers import Solver
 
+from relationalai.experimental.solvers import Provider as Provider
+
 _Any = Union[b.Producer, str, float, int]
 _String = Union[b.Producer, str]
 _Number = Union[b.Producer, float, int]
 
 class SolverModel:
-    def __init__(self, model: b.Model):
-        self._id = next(b._global_id)
+    def __init__(self, model: b.Model, num_type: str):
+        assert num_type in ["cont", "int"], "Invalid numerical type, must be 'cont' or 'int'"
         self._model = model # TODO can we remove? only used for _model._to_executor
+        self._num_type = num_type
+        self._id = next(b._global_id)
         self.variable_relationships = ordered_set()
         prefix_u = f"SolverModel_{self._id}_"
         prefix_l = f"solvermodel_{self._id}_"
 
-        Variable = self.Variable = model.Concept(prefix_u + "Variable")
+        Variable = model.Concept(prefix_u + "Variable")
+        self.Variable = Variable
         self.MinObjective = model.Concept(prefix_u + "MinObjective")
         self.MaxObjective = model.Concept(prefix_u + "MaxObjective")
         self.Constraint = model.Concept(prefix_u + "Constraint")
-        self.result_info = model.Relationship("{key:str} has {val}", short_name=(prefix_l + "result_info"))
-        self.point = model.Relationship("{var} has {val}", short_name=(prefix_l + "point"))
-        self.points = model.Relationship("point {i:int} for {var} has {val}", short_name=(prefix_l + "points"))
 
-        # add rules to make variable lower/upper bounds, fixed values, and types into constraints
-        for (typ, op) in _var_types.items():
-            expr = _make_fo_appl_with_res(op, Variable)
-            b.where(Variable.type(typ)).define(self.Constraint.new(serialized=expr))
-        for (prop, op) in [("lower", ">="), ("upper", "<="), ("fixed", "=")]:
-            expr = _make_fo_appl_with_res(_fo_comparisons[op], Variable, getattr(Variable, prop))
-            b.define(self.Constraint.new(serialized=expr))
+        self._model_info = {
+            "num_variables": Variable,
+            "num_constraints": self.Constraint,
+            "num_min_objectives": self.MinObjective,
+            "num_max_objectives": self.MaxObjective,
+        }
+
+        res_type = "int" if num_type == "int" else "float"
+        self.result_info = model.Relationship("{key:str} has {val:str}", short_name=(prefix_l + "result_info"))
+        self.point = model.Relationship(f"{{Variable}} has {{val:{res_type}}}", short_name=(prefix_l + "point"))
+        self.points = model.Relationship(f"point {{i:int}} for {{Variable}} has {{val:{res_type}}}", short_name=(prefix_l + "points"))
 
         b.define(b.RawSource("rel", textwrap.dedent(f"""
         declare {self.MinObjective._name}
@@ -44,9 +50,6 @@ class SolverModel:
         declare {self.Constraint._name}
 
         declare {prefix_l}variable_name
-        declare {prefix_l}variable_lower
-        declare {prefix_l}variable_upper
-        declare {prefix_l}variable_fixed
         declare {prefix_l}minobjective_name
         declare {prefix_l}maxobjective_name
         declare {prefix_l}constraint_name
@@ -85,19 +88,28 @@ class SolverModel:
 
         # handle optional variable properties
         for (key, val) in kwargs.items():
-            if key in ("name", "type"):
+            if key == "name":
                 assert isinstance(val, _String), f"Expected {key} to be a string, got {type(val)}"
-            elif key in ("lower", "upper", "fixed", "start"):
+                defs.append(var.name(val))
+            elif key == "type":
+                assert isinstance(val, str), f"Expected {key} to be a string, got {type(val)}"
+                assert val in _var_types, f"Invalid variable type {val} for `solve_for`"
+                ser = _make_fo_appl_with_res(_var_types[val], var)
+                defs.append(self.Constraint.new(serialized=ser))
+            elif key in ("lower", "upper", "fixed"):
                 assert isinstance(val, _Number), f"Expected {key} to be a number, got {type(val)}"
+                op = ">=" if key == "lower" else ("<=" if key == "upper" else "=")
+                ser = _make_fo_appl_with_res(_fo_comparisons[op], var, val)
+                defs.append(self.Constraint.new(serialized=ser))
             else:
                 raise ValueError(f"Invalid keyword argument {key} for `solve_for`")
-            defs.append(getattr(var, key)(val))
+
         b.define(*defs)
 
         if populate:
             # TODO do something different in future. maybe delete/insert into variable relationships after solve.
             # get variable values from the result point (populated by the solver)
-            val = b.Number.ref()
+            val = (b.Integer if self._num_type == "int" else b.Float).ref()
             b.define(rel(val)).where(self.point(var, val))
         return var
 
@@ -151,16 +163,14 @@ class SolverModel:
     def summarize(self):
         to_count = [
             self.Variable,
-            self.Variable.type("int"),
-            self.Variable.type("bin"),
             self.MinObjective.serialized,
             self.MaxObjective.serialized,
             self.Constraint.serialized,
         ]
         counts = b.select(*[(b.count(c) | 0) for c in to_count]).to_df() # TODO(coey) do we need the |0?
         assert counts.shape == (1, len(to_count)), f"Unexpected counts shape {counts.shape}"
-        (vars, int_vars, bool_vars, min_objs, max_objs, cons) = counts.iloc[0]
-        print(f"Solver model has {vars} variables ({int_vars} integer, {bool_vars} binary), {min_objs} minimization objectives, {max_objs} maximization objectives, and {cons} constraints")
+        (vars, min_objs, max_objs, cons) = counts.iloc[0]
+        print(f"Solver model has {vars} variables, {min_objs} minimization objectives, {max_objs} maximization objectives, and {cons} constraints")
         return None
 
     # print the variables and components of the model in human-readable format
@@ -171,7 +181,6 @@ class SolverModel:
         if var_df.empty:
             print("No variables defined in the solver model.")
             return None
-        # TODO print type of variable also
         print(f"{var_df.shape[0]} variables:")
         print(var_df.to_string(index=False, header=False))
 
@@ -245,7 +254,7 @@ class SolverModel:
 
         # 3. Extract result.
         print("extract result")
-        executor.execute_raw(textwrap.dedent(f"""
+        extract_str = textwrap.dedent(f"""
         def raw_result {{
             load_binary["snowflake://APP_STATE.RAI_INTERNAL_STAGE/job-results/{job_id}/result.binpb"]
         }}
@@ -260,10 +269,29 @@ class SolverModel:
         def delete[:{self.points._name}]: {self.points._name}
 
         def insert(:{self.result_info._name}, key, val):
-            exists((k) | extracted(k, val) and ::std::mirror::lower(k, key))
-        def insert(:{self.point._name}, var, val): extracted(:point, var, val)
-        def insert(:{self.points._name}, i, var, val): extracted(:points, var, i, val)
-        """), readonly=False)
+            exists((k) | string(extracted[k], val) and ::std::mirror::lower(k, key))
+        """)
+        if self._num_type == "int":
+            extract_str += textwrap.dedent(f"""
+            def insert(:{self.point._name}, var, val):
+                exists((x) | extracted(:point, var, x) and
+                    ::std::mirror::convert(std::mirror::typeof[Int128], x, val)
+                )
+            def insert(:{self.points._name}, i, var, val):
+                exists((j, x) | extracted(:points, var, j, x) and
+                    ::std::mirror::convert(std::mirror::typeof[Int128], j, i) and
+                    ::std::mirror::convert(std::mirror::typeof[Int128], x, val)
+                )
+            """)
+        else:
+            extract_str += textwrap.dedent(f"""
+            def insert(:{self.point._name}, var, val): extracted(:point, var, val)
+            def insert(:{self.points._name}, i, var, val):
+                exists((j) | extracted(:points, var, j, val) and
+                    ::std::mirror::convert(std::mirror::typeof[Int128], j, i)
+                )
+            """)
+        executor.execute_raw(extract_str, readonly=False)
 
         print("finished solve")
         return None
@@ -277,30 +305,23 @@ class SolverModel:
         assert isinstance(executor, RelExecutor)
         executor.execute_raw(textwrap.dedent(f"""
         def delete[:{self.point._name}]: {self.point._name}
-        def insert(:{self.point._name}, var, val): {self.points._name}({i}, var, val)
+        def insert(:{self.point._name}, var, val): {self.points._name}(int128[{i}], var, val)
         """), readonly=False)
         return None
 
     # print summary of the solver result
     def summarize_result(self):
-        to_get = ["error", "termination_status", "solve_time_sec", "objective_value", "solver_version", "solution_count"]
-        k = b.String.ref()
-        v = b.Concept.builtins["Any"].ref()
+        to_get = ["error", "termination_status", "solve_time_sec", "objective_value", "solver_version", "result_count"]
+        k, v = b.String.ref(), b.String.ref()
         df = b.select(k, v).where(self.result_info(k, v), k.in_(to_get)).to_df()
-        # TODO maybe print in a sentence like below
-        # (er, st, ti, ob, so) = ...
-        # if er:
-        #     print(f"Solver encountered an error: {er}")
-        # print(f"Solver {so} terminated with status {st} after {ti} seconds.")
-        # if ob:
-        #     print(f"Solver reported an objective value of {ob}.")
+        assert not df.empty, "No result information"
         print(df.to_string(index=False, header=False))
         return df
 
     # select variable names and values in the primal result point(s)
     def variable_values(self, multiple: bool = False):
         var = self.Variable.ref()
-        val = b.Number.ref()
+        val = (b.Integer if self._num_type == "int" else b.Float).ref()
         if multiple:
             i = b.Integer.ref()
             return b.select(i, var.name, val).where(self.points(i, var, val))
@@ -309,15 +330,27 @@ class SolverModel:
 
     # get scalar result information after solving
     def __getattr__(self, name: str):
-        if name in {"error", "termination_status", "solver_version", "printed_model", "solve_time_sec", "objective_value", "result_count"}:
-            val = b.Concept.builtins["Any"].ref()
+        df = None
+        if name in self._model_info:
+            df = b.select(b.count(self._model_info[name])).to_df()
+        elif name in {"error", "termination_status", "solver_version", "printed_model", "solve_time_sec", "objective_value", "result_count"}:
+            val = b.String.ref()
             df = b.select(val).where(self.result_info(name, val)).to_df()
+        # extract scalar from df
+        if df is not None:
             if not df.shape == (1, 1):
-                raise ValueError(f"Expected exactly one result for {name}, but df has shape {df.shape}")
-            return df.iloc[0, 0]
+                raise ValueError(f"Expected exactly one value for {name}, but df has shape {df.shape}")
+            v = df.iloc[0, 0]
+            if isinstance(v, str):
+                if name == "solve_time_sec":
+                    return float(v)
+                elif name == "objective_value":
+                    return int(v) if self._num_type == "int" else float(v)
+                elif name == "result_count":
+                    return int(v)
+            return v
         return None
 
-
 # TODO maybe structure rewriting code to be more like the compiler passes rather than in one big if-else
 def _rewrite(expr: Any, sm: SolverModel) -> Any:
     if isinstance(expr, (int, float, str)):
@@ -373,7 +406,7 @@ def _rewrite(expr: Any, sm: SolverModel) -> Any:
         assert isinstance(op, str)
         if op in _ho_operators:
             appl = b.Relationship.builtins["rel_primitive_solverlib_ho_appl"]
-            agg = b.Aggregate(appl, b.TupleArg(start_args), sym_arg, _ho_operators[op])
+            agg = b.Aggregate(appl, *start_args, sym_arg, _ho_operators[op])
             agg._group = expr._group
             agg._where = expr._where
             return agg
@@ -473,3 +506,5 @@ _ho_operators = {
     "count": 84,
     "all_different": 90,
 }
+
+
