@@ -19,6 +19,7 @@ import contextvars
 from .metamodel import Action, Task
 from .environments import runtime_env, IPythonEnvironment, SnowbookEnvironment
 from . import debugging
+from .tools.constants import SHOW_FULL_TRACES
 
 
 #--------------------------------------------------
@@ -344,7 +345,14 @@ class RAIException(Exception, metaclass=RAIPostInitMeta):
         else:
             return self.message
 
-    def clone(self):
+    def clone(self, config=None):
+        show_full_traces = SHOW_FULL_TRACES
+        if config is not None:
+            show_full_traces = config.get("show_full_traces", SHOW_FULL_TRACES)
+
+        if show_full_traces:
+            return self
+
         try:
             raise AssertionError
         except AssertionError:
@@ -376,6 +384,51 @@ class RAIException(Exception, metaclass=RAIPostInitMeta):
             [green]{provider_call}[/green]
             """)
 
+class NonExistentConfigFileError(RAIException):
+    def __init__(self, file_name: str):
+        self.file_name = file_name
+        self.message =  f"The specified configuration file '{file_name}' does not exist."
+        self.name =  "Configuration file not found"
+
+        # Format the content and raw_content
+        self.content = self.format_message()
+
+        super().__init__(self.message, self.name, self.content)
+
+    def format_message(self):
+        return textwrap.dedent(f"""
+        Configuration file '{self.file_name}' does not exist.
+
+        To initialize one, use:
+
+        [green]rai init[/green]
+        """)
+
+class InvalidActiveProfileError(RAIException):
+    def __init__(self, active_profile_name: str):
+        self.active_profile_name = active_profile_name
+        self.message = f"The specified active profile '{self.active_profile_name}' set in the config is not found."
+        self.name = f"Active profile '{active_profile_name}' not found"
+
+        # Format the content and raw_content
+        self.content = self.format_message()
+
+        super().__init__(self.message, self.name, self.content)
+
+    def format_message(self):
+        return textwrap.dedent(f"""
+        The specified active profile '{self.active_profile_name}' is not found.
+        Please check your configuration and ensure that the profile exists.
+
+        To switch to a different profile, use the following command:
+
+        [green]rai profile:switch[/green]
+
+        To create a new profile, use:
+
+        [green]rai init[/green]
+        """)
+
 class ModelError(RAIException):
     error_locations = {}
 
@@ -443,8 +496,8 @@ class RAIExceptionSet(RAIException):
         for exception in self.exceptions:
             exception.pprint()
 
-    def clone(self):
-        return RAIExceptionSet([exception.clone() for exception in self.exceptions])
+    def clone(self, config=None):
+        return RAIExceptionSet([exception.clone(config) for exception in self.exceptions])
 
 # --------------------------------------------------
 # Warning
@@ -587,7 +640,7 @@ class IntegrityConstraintViolation(RAIException):
         {marked}
         """)
 
-    def clone(self):
+    def clone(self, config=None):
         return IntegrityConstraintViolation(self.violation, self.source)
 
 class ModelWarning(RAIWarning):
@@ -1467,13 +1520,19 @@ https://relational.ai/notebooks/relationalai-installation.ipynb
 """)
 
 class SnowflakeMissingConfigValuesException(RAIException):
-    def __init__(self, missing_keys: list[str]):
+    def __init__(self, missing_keys: list[str], profile: str | None = None, config_file_path: str | None = None):
         sorted_missing_keys = sorted(missing_keys)
-        self.message = f"""
-[green]{', '.join(sorted_missing_keys)}[/green]\n
-[yellow]Please check your configuration file and make sure all required values are set.
-"""
         self.name = f"Missing {len(missing_keys)} required configuration value{'s' if len(missing_keys) > 1 else ''}"
+        config_info = f"File: {config_file_path}\n" if config_file_path else ""
+        profile_info = f"Profile: '{profile}'\n" if profile else ""
+        self.message = f"""
+{self.name}:
+
+[green]{', '.join(sorted_missing_keys)}[/green]
+
+{config_info}{profile_info}
+[yellow]Please update the configuration file above to include the missing values.
+"""
         self.content = self.format_message()
 
         super().__init__(self.message, self.name, self.content)
@@ -1649,7 +1708,6 @@ class SnowflakeInvalidSource(RAIException):
         {marked}
         """)
 
-
 class SnowflakeRaiAppNotStarted(RAIException):
     def __init__(self, app_name: str):
         self.app_name = app_name
@@ -1667,7 +1725,6 @@ class SnowflakeRaiAppNotStarted(RAIException):
         [green]call {self.app_name}.app.activate();[/green]
         """)
 
-
 # --------------------------------------------------
 # RAI Warnings
 # --------------------------------------------------
@@ -1743,7 +1800,6 @@ class InvalidLoopWarning(RAIWarning):
     def mark_source(self):
         return mark_source(self.source, self.start_line, self.end_line)
 
-
 class InvalidTryWarning(RAIWarning):
     def __init__(self, task: Task | Action, start_line: int, end_line: int):
         self.name = "Invalid try"
