@@ -1,5 +1,4 @@
 from __future__ import annotations
-import time
 from typing import Any, List
 from dataclasses import dataclass
 import textwrap
@@ -19,6 +18,10 @@ rel_sv = rel._tagged(Builtins.SingleValued)
 APP_NAME = relationalai.clients.snowflake.APP_NAME
 
 ENGINE_TYPE_SOLVER = "SOLVER"
+# TODO (dba) The ERP still uses `worker` instead of `engine`. Change
+# this once we fix this in the ERP.
+ENGINE_ERRORS = ["worker is suspended", "create/resume", "worker not found", "no workers found", "worker was deleted"]
+ENGINE_NOT_READY_MSGS = ["worker is in pending", "worker is provisioning", "worker is not ready to accept jobs"]
 
 # --------------------------------------------------
 # SolverModel object.
@@ -218,7 +221,7 @@ class SolverModel:
         # TODO(coey) Currently we must run a dummy query to install the pyrel rules in a separate txn
         # to the solve_output updates. Ideally pyrel would offer an option to flush the rules separately.
         self.graph.exec_raw("")
-        self.graph.exec_raw(
+        response = self.graph.exec_raw(
             textwrap.dedent(f"""
             ic model_not_empty("Solver model is empty.") requires not empty({scope}model_string)
 
@@ -239,9 +242,26 @@ class SolverModel:
             def export {{ config }}
             """)
         )
+        txn = response.transaction or {}
+        # The above `exec_raw` will throw an error if the transaction
+        # gets aborted. But in the case it gets cancelled, by the user
+        # or the system, it won't throw. In that case we also did not
+        # upload the input model.
+        if txn["state"] == "":
+            txn = solver.provider.resources.get_transaction(txn["id"]) or {}
+        if txn["state"] != "COMPLETED":
+            raise Exception(f"Transaction that materializes the solver inputs did not complete! ID: `{txn['id']}` State `{txn['state']}`")
 
         # 2. Execute job and wait for completion.
-        job_id = solver._exec_job(payload, log_to_console=log_to_console)
+        try:
+            job_id = solver._exec_job(payload, log_to_console=log_to_console)
+        except Exception as e:
+            err_message = str(e).lower()
+            if any(kw in err_message.lower() for kw in ENGINE_ERRORS + ENGINE_NOT_READY_MSGS):
+                solver._auto_create_solver_async()
+                job_id = solver._exec_job(payload, log_to_console=log_to_console)
+            else:
+                raise e
 
         # 3. Extract result.
         res = self.graph.exec_raw(
@@ -484,6 +504,8 @@ class Solver:
         self,
         solver_name: str,
         engine_name: str | None = None,
+        engine_size: str | None = None,
+        auto_suspend_mins: int | None = None,
         resources: SnowflakeResources | None = None,
     ):
         self.provider = Provider(resources=resources)
@@ -497,21 +519,31 @@ class Solver:
                 if "solvers" in exp_config:
                     settings = exp_config["solvers"].copy()
 
-        # Engine field is not necessary for the solver settings.
+        # Engine configuration fields are not necessary for the solver
+        # settings so we `pop` them from the settings object. Default
+        # size and auto_suspend_mins are set in the `Provider` methods.
         engine_name = engine_name or settings.pop("engine", None)
         if not engine_name:
             engine_name = self.provider.resources.get_user_based_engine_name()
+        self.engine_name = engine_name
+
+        self.engine_size = engine_size or settings.pop("engine_size", None)
+        self.engine_auto_suspend_mins = auto_suspend_mins or settings.pop("auto_suspend_mins", None)
 
         # The settings are used when creating a solver engine, they
         # may configure each individual solver.
-        self.settings = settings
+        self.engine_settings = settings
 
-        return self._auto_create_solver_async(engine_name)
+        return self._auto_create_solver_async()
 
     # --------------------------------------------------
     # Helper
     # --------------------------------------------------
-    def _auto_create_solver_async(self, name: str):
+    def _auto_create_solver_async(self):
+        name = self.engine_name
+        auto_suspend_mins = self.engine_auto_suspend_mins
+        size = self.engine_size
+        settings = self.engine_settings
         with Spinner(
             "Checking solver status",
             leading_newline=True,
@@ -522,11 +554,31 @@ class Solver:
             if len(engines) != 0:
                 engine = engines[0]
             if engine:
+                # TODO (dba) Logic engines support altering the
+                # auto_suspend_mins setting. Currently, we don't have
+                # this capability for solver engines, so users need to
+                # recreate or use another engine. For both the size
+                # and Gurobi configuration the user anyways has to
+                # create a new one as this configuration must happen
+                # when the engine is created.
+                settings_cannot_be_altered_msg = "The configuration of a solver engine happens when the engine is created and _cannot_ be changed. You either need to specify a new engine name or delete the current engine.\n\nSee `solvers.Provider().delete_solver()`."
                 # Make sure that the solver requested is enabled
                 # on the engine.
                 if self.solver_name not in engine["solvers"]:
                     raise Exception(
-                        f"Solver `{self.solver_name}` is not enabled on `{name}`."
+                        f"Solver `{self.solver_name}` is not enabled on `{name}`.\n\n" + settings_cannot_be_altered_msg
+                    )
+
+                # Make sure size and auto_suspend_mins settings match
+                # what the user requests.
+                if size is not None and size != engine["size"]:
+                    raise Exception(
+                        f"Engine `{name}` has size setting of `{engine['size']}` but size is requested to be `{size}`.\n\n" + settings_cannot_be_altered_msg
+                    )
+
+                if auto_suspend_mins is not None and auto_suspend_mins != engine["auto_suspend_mins"]:
+                    raise Exception(
+                        f"Engine `{name}` has auto_suspend_mins setting of `{engine['auto_suspend_mins']}` but auto_suspend_mins is requested to be `{auto_suspend_mins}`.\n\n" + settings_cannot_be_altered_msg
                     )
 
                 if engine["state"] == "PENDING":
@@ -562,7 +614,7 @@ class Solver:
                 # Validate Gurobi config.
                 if self.solver_name == "gurobi":
                     is_gurobi_configured = False
-                    gurobi_config = self.settings.get("gurobi", {})
+                    gurobi_config = settings.get("gurobi", {})
                     if all(
                         k in gurobi_config
                         for k in ["license_secret_name", "external_access_integration"]
@@ -570,18 +622,16 @@ class Solver:
                         is_gurobi_configured = True
                     if not is_gurobi_configured:
                         raise Exception(
-                            "Gurobi is not properly configured. You need to provide both `license_secret_name` and `external_access_integration`."
+                            "Gurobi is not properly configured. You need to provide both `license_secret_name` and `external_access_integration` in its configuration, see https://docs.relational.ai/build/reasoners/prescriptive/solver-backends/gurobi/#usage"
                         )
-                self.provider.create_solver_async(name, settings=self.settings)
+                self.provider.create_solver_async(name, settings=settings, size=size, auto_suspend_mins=auto_suspend_mins)
+                engine = self.provider.get_solver(name)
                 spinner.update_messages(
                     {
                         "finished_message": f"Starting solver {name}...",
                     }
                 )
 
-            # Fetch the engine object and cache it.
-            engine = self.provider.get_solver(name)
-            self.engine_cached_at_secs = time.time()
             self.engine = engine
 
     def _exec_job_async(self, payload):
@@ -595,15 +645,6 @@ class Solver:
         return res[0]["ID"]
 
     def _exec_job(self, payload, log_to_console=True):
-        # We cache the engine object, s.t. we don't have to do a
-        # lookup each time we submit a job. We use the auto suspend
-        # settings of the engine as a rough estimate on how often we
-        # should do this. We want to catch a suspended engine, so that
-        # we can resume it below.
-        if (time.time() - self.engine_cached_at_secs) >= self.engine["auto_suspend_mins"] * 60:
-            self.engine = self.provider.get_solver(self.engine["name"])
-            self.engine_cached_at_secs = time.time()
-
         # Make sure the engine is ready.
         if self.engine["state"] != "READY":
             poll_with_specified_overhead(lambda: self._is_solver_ready(), 0.1)
@@ -632,7 +673,7 @@ class Solver:
         state = result["state"]
         if state != "READY" and state != "PENDING":
             # Might have suspended or otherwise gone. Recreate.
-            self._auto_create_solver_async(self.engine["name"])
+            self._auto_create_solver_async()
         return state == "READY"
 
     def _check_job_status(self, state):
