@@ -918,13 +918,20 @@ def create_engine_flow(cfg:config.Config, name=None, size=None, auto_suspend_min
         else:
             exit_with_error(f"[yellow]Engine '{name}' already exists")
 
-    if not size:
+    # If no size is provided via the params and no size is set in the config,
+    if not size and not cfg.get("engine_size", None):
         rich.print("")
-        if cfg.get("engine_size", None):
-            size = cfg.get("engine_size")
-        else:
-            choices = provider.get_engine_sizes()
-            size = controls.fuzzy("Engine size:", choices=choices)
+        cloud_provider = provider.get_cloud_provider()
+        choices = provider.get_engine_sizes(cloud_provider)
+        size = controls.fuzzy(f"Engine size ({cloud_provider.upper()}):", choices=choices)
+    # If a size is provided via the params or the config, validate it
+    elif size or cfg.get("engine_size", None):
+        # If size is None but config has engine_size, use config value
+        if size is None:
+            size = cfg.get("engine_size", None)
+        valid_sizes = provider.get_engine_sizes()
+        if not isinstance(size, str) or size not in valid_sizes:
+            exit_with_error(f"\nInvalid engine size [yellow]{size}[/yellow] provided. Please check your config.\n\nValid sizes: [green]{valid_sizes}[/green]")
     assert isinstance(size, str), "engine_size must be a string"
 
     rich.print("")
@@ -1536,7 +1543,13 @@ def imports_stream(
         except UnsupportedTypeError as err:
             exit_with_error(f"\n\n[yellow]The [bold]{provider.platform}[/bold] integration doesn't support streaming from [bold]'{err.type}'[/bold] sources.")
         except Exception as e:
-            if "use setup_cdc()" in f"{e}":
+            if "relations are not empty" in f"{e}":
+                # Handle LeftOverRelationException directly here
+                from relationalai.errors import LeftOverRelationException
+                exception = LeftOverRelationException(import_source.name, model)
+                exception.pprint()
+                sys.exit(1)
+            elif "use setup_cdc()" in f"{e}":
                 exit_with_error("\n\n[yellow]Imports are not configured.\n[yellow]To start use '[cyan]rai imports:setup[/cyan]' to set up imports.")
             elif "stream already exists" in f"{e}":
                 exit_with_error(f"\n\n[yellow]Stream [cyan]'{import_source.name.upper()}'[/cyan] already exists.")
@@ -1609,7 +1622,8 @@ def imports_snapshot(source:str|None, model:str|None, name:str|None, type:str|No
 @cli.command(name="imports:delete", help="Delete an import from RAI")
 @click.option("--object", help="Object")
 @click.option("--model", help="Model")
-def imports_delete(object, model):
+@click.option("--force", help="Force delete stream and relations", is_flag=True)
+def imports_delete(object, model, force):
     divider(flush=True)
     ensure_config()
     provider = cast(snowflake.Resources, get_resource_provider())
@@ -1632,7 +1646,7 @@ def imports_delete(object, model):
         except Exception as e:
             return exit_with_error(f"\n\n[yellow]Error fetching imports: {e}")
 
-    if not imports:
+    if not imports and not force:
         rich.print()
         exit_with_error("[yellow]No imports to delete")
 
@@ -1648,9 +1662,11 @@ def imports_delete(object, model):
         rich.print()
 
     for object in objects:
-        with Spinner(f"Removing {object}", f"{object} removed successfully"):
+        spinner_message = f"Removing {object}" + (" and relations" if force else "")
+        success_message = f"{object}" + (" and relations" if force else "") + " removed successfully"
+        with Spinner(spinner_message, success_message):
             try:
-                provider.delete_import(object, model)
+                provider.delete_import(object, model, force)
             except Exception as e:
                 exit_with_error(f"\n\n[yellow]Error deleting import: {e}")
     divider()
