@@ -376,6 +376,8 @@ class Spinner:
         self.trailing_newline = trailing_newline
         self.last_message = ""
         self.display = None
+        # Add lock to prevent race conditions between spinner thread and main thread
+        self._update_lock = threading.Lock()
 
     def _set_delay(self, delay: float|int|None) -> None:
         """Set appropriate delay based on environment and user input."""
@@ -409,36 +411,40 @@ class Spinner:
             return full_message
 
     def update(self, message:str|None=None, color:str|None=None, file:TextIO|None=None, starting=False):
-        if message is None:
-            message = self.get_message(starting=starting)
-        if self.is_jupyter:
-            # @NOTE: IPython isn't available in CI. This won't ever get invoked w/out IPython available though.
-            from IPython.display import HTML, display # pyright: ignore[reportMissingImports]
-            color_string = ""
-            if color:
-                color_value = Color.parse(color)
-                rgb_tuple = color_value.get_truecolor()
-                rgb_hex = f"#{rgb_tuple[0]:02X}{rgb_tuple[1]:02X}{rgb_tuple[2]:02X}"
-                color_string = f"color: {rgb_hex};" if color is not None else ""
-            content = HTML(f"<span style='font-family: monospace;{color_string}'>{message}</span>")
-            if self.display is not None:
-                self.display.update(content)
+        # Use lock to prevent race conditions between spinner thread and main thread
+        with self._update_lock:
+            if message is None:
+                message = self.get_message(starting=starting)
+            if self.is_jupyter:
+                # @NOTE: IPython isn't available in CI. This won't ever get invoked w/out IPython available though.
+                from IPython.display import HTML, display # pyright: ignore[reportMissingImports]
+                color_string = ""
+                if color:
+                    color_value = Color.parse(color)
+                    rgb_tuple = color_value.get_truecolor()
+                    rgb_hex = f"#{rgb_tuple[0]:02X}{rgb_tuple[1]:02X}{rgb_tuple[2]:02X}"
+                    color_string = f"color: {rgb_hex};" if color is not None else ""
+                content = HTML(f"<span style='font-family: monospace;{color_string}'>{message}</span>")
+                if self.display is not None:
+                    self.display.update(content)
+                else:
+                    self.display = display(content, display_id=True)
             else:
-                self.display = display(content, display_id=True)
-        else:
-            if self.can_use_terminal_colors() and color is not None:
-                rich_message = f"[{color}]{message}"
-            else:
-                rich_message = message
-            rich_string = rich_str(rich_message)
-            def width(word):
-                return sum(wcwidth(c) for c in word)
-            diff = width(self.last_message) - width(rich_string)
-            self.reset_cursor()
-            rich.print(rich_message + (" " * diff), file=file, end="")
-            if (file or sys.stdout).isatty() or self.in_notebook:
-                (file or sys.stdout).flush()
-            self.last_message = rich_string
+                if self.can_use_terminal_colors() and color is not None:
+                    rich_message = f"[{color}]{message}"
+                else:
+                    rich_message = message
+                rich_string = rich_str(rich_message)
+                def width(word):
+                    return sum(wcwidth(c) for c in word)
+                diff = width(self.last_message) - width(rich_string)
+                self.reset_cursor()
+                # Use rich.print with lock protection
+                output_file = file or sys.stdout
+                rich.print(rich_message + (" " * diff), file=output_file, end="", flush=False)
+                if output_file.isatty() or self.in_notebook:
+                    output_file.flush()
+                self.last_message = rich_string
     
     def can_use_terminal_colors(self):
         return not self.is_snowflake_notebook
@@ -459,7 +465,10 @@ class Spinner:
             self.reset_cursor()
 
     def reset_cursor(self):
-        if not self.is_jupyter:
+        if self.is_tty:
+            # Clear the entire line and move cursor to beginning
+            sys.stdout.write("\r\033[K")
+        elif not self.is_jupyter:
             sys.stdout.write("\r")
 
     def __enter__(self):
@@ -481,7 +490,8 @@ class Spinner:
         if exception is not None:
             if self.failed_message is not None:
                 self.update(f"{self.failed_message} {value}", color="yellow", file=sys.stderr)
-                rich.print()
+                # Use rich.print with explicit newline to ensure proper formatting
+                rich.print(file=sys.stderr)
                 return True
             return False
         if self.delay: # will be None for non-interactive environments
@@ -490,6 +500,7 @@ class Spinner:
         if self.finished_message != "":
             final_message = f"▰▰▰▰ {self.finished_message}"
             self.update(final_message, color="green")
+            # Use rich.print with explicit newline to ensure proper formatting
             rich.print()
         elif self.finished_message == "":
             self.update("")
@@ -497,6 +508,24 @@ class Spinner:
         if self.trailing_newline:
             rich.print()
 
+class DebuggingSpan:
+    span: debugging.Span
+    def __init__(self, span_type: str):
+        self.span_type = span_type
+        self.span_attrs = {}
+
+    def attrs(self, **kwargs):
+        self.span_attrs = kwargs
+        return self
+
+    def __enter__(self):
+        self.span = debugging.span_start(self.span_type, **self.span_attrs)
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        debugging.span_end(self.span)
+
+
 class SpanSpinner(Spinner):
     span: debugging.Span
     def __init__(self, span_type: str, *spinner_args, **spinner_kwargs):
